package ch.usi.inf.sape.algoprof.example.graph.directed.adjacencymatrix.basic;

import ch.usi.inf.sape.algoprof.statistics.Algorithm;
import ch.usi.inf.sape.algoprof.statistics.AlgorithmVisitor;
import ch.usi.inf.sape.algoprof.statistics.Configuration;


public class Graph implements Algorithm {

	private final boolean[] marks;
	private final boolean [][] edges;
	private int counter;
	private int size;
	
	public Graph(final int nodeCount) {
		marks = new boolean[nodeCount];
		edges = new boolean[nodeCount][nodeCount];
		counter = 0;
		size = nodeCount * nodeCount;
		makeComplete(nodeCount);
		
	}
	
	private void makeComplete(final int nodeCount) {
		for (int n=0; n<nodeCount; n++) {
			counter++;
			for (int ds=0; ds<nodeCount; ds++) {
				counter++;
				if (n!=ds) {
					edges[n][ds] = true;
				}
			}
		}
				
	}
	
	public int getNumberOfNodes() {
		return marks.length;
	}
	
	public String getData() {
		return size + "\t" + counter + "\n";
	}
	
	public void printToDot() {
		System.out.println("graph G {");
		counter = 0;
		//size = getNumberOfNodes()^2;
		for (int n=0; n<getNumberOfNodes(); n++) {
			counter++;
			System.out.println("  "+n);
			for (int ds=0; ds<getNumberOfNodes(); ds++) {
				counter++;
				if (edges[n][ds]) {
					System.out.println("  "+n+" -> "+ds);
				}
			}
		}
		System.out.println("}");
	}
	
	public void recursivePrint(Main m) {
		// clear marks
		counter = 0;
		size = getNumberOfNodes();
		for (int n=0; n<getNumberOfNodes(); n++) {
			counter++;
			marks[n] = false;
		}
		if (Configuration.ENABLE_MANUAL_INSTRUMENTATION) {
			accept(m.getVisitor("mark"));
			
		}
		counter = 0;
		size = getNumberOfNodes()*getNumberOfNodes();
		// starting at each node (there is no specific root), start a DFS
		System.out.println("graph G {");
		for (int n=0; n<getNumberOfNodes(); n++) {
			counter++;
			if (Configuration.ENABLE_MANUAL_INSTRUMENTATION) System.out.println(marks[n]);
			recursivePrintFrom(n);
		}
		if (Configuration.ENABLE_MANUAL_INSTRUMENTATION) {
			accept(m.getVisitor("recursivePrint"));
			
		}
		System.out.println("}");
	}
	
	public void recursivePrintFrom(final int nodeId) {
		counter++;
		if (!marks[nodeId] && getNumberOfNodes()>0) {
			marks[nodeId] = true;
			System.out.println("  "+nodeId);
			final boolean[] edge = edges[nodeId];
			for (int ds=0; ds<getNumberOfNodes(); ds++) {
				counter++;
				if (edge[ds]) {
					System.out.println("  "+nodeId+" -> "+ds);
				}
				
				recursivePrintFrom(ds);
			}
		}
	}

	public void accept(AlgorithmVisitor visitor) {
		visitor.visit(this);
		
	}

}
