package ch.usi.inf.sape.algoprof.example.graph.directed.linked.basic;

import java.util.ArrayList;

import ch.usi.inf.sape.algoprof.statistics.Algorithm;
import ch.usi.inf.sape.algoprof.statistics.AlgorithmVisitor;
import ch.usi.inf.sape.algoprof.statistics.Configuration;


public class Graph implements Algorithm {

	private final ArrayList<Node> nodes;
	private int counter;
	private int size;
	private Main m;
	
	public Graph(final int nodeCount, Main main) {
		nodes = new ArrayList<Node>();
		counter = 0;
		size = nodeCount;
		this.m = main;
		makeComplete(nodeCount);
	}
	
	private void makeComplete(final int nodeCount) {
		for (int n=0; n<nodeCount; n++) {
			counter++;
			final Node node = new Node();
			node.setId(n);
			addNode(node);
		}
		if (Configuration.ENABLE_MANUAL_INSTRUMENTATION) {
			accept(m.getVisitor("makeNodes"));
		}
		counter = 0;
		size = nodeCount*nodeCount;
		for (int n=0; n<nodeCount; n++) {
			counter++;
			final Node node = getNode(n);
			for (int ds=0; ds<nodeCount; ds++) {
				counter++;
				final Node directSuccessor = getNode(ds);
				if (node!=directSuccessor) {
					node.addDirectSuccessor(directSuccessor);
				}
			}
		}
		if (Configuration.ENABLE_MANUAL_INSTRUMENTATION) {
			accept(m.getVisitor("makeComplete"));
		}
	}
	
	public void addNode(final Node node) {
		nodes.add(node);
	}
	
	public int getNumberOfNodes() {
		return nodes.size();
	}
	
	public Node getNode(final int i) {
		return nodes.get(i);
	}
	
	public void printToDot() {
		System.out.println("graph G {");
		counter = 0;
		for (int n=0; n<getNumberOfNodes(); n++) {
			counter++;
			final Node node = getNode(n);
			System.out.println("  "+node.getId());
			for (int ne=0; ne<node.getOutDegree(); ne++) {
				counter++;
				final Node directSuccessor = node.getDirectSuccessor(ne);
				System.out.println("  "+node.getId()+" -> "+directSuccessor.getId());
			}
		}
	
		System.out.println("}");
	}
	
	public void recursivePrint() {
		// clear marks
		counter = 0;
		size = getNumberOfNodes();
		for (int n=0; n<getNumberOfNodes(); n++) {
			counter++;
			getNode(n).clear();
		}
		if (Configuration.ENABLE_MANUAL_INSTRUMENTATION) {
			accept(m.getVisitor("mark"));
		}
		// starting at each node (there is no specific root), start a DFS
		counter = 0;
		size = getNumberOfNodes()*getNumberOfNodes();
		System.out.println("graph G {");
		for (int n=0; n<getNumberOfNodes(); n++) {
			counter++;
			recursivePrintFrom(getNode(n));
		}
		if (Configuration.ENABLE_MANUAL_INSTRUMENTATION) {
			accept(m.getVisitor("recursivePrint"));
		}
		System.out.println("}");
	}
	
	public void recursivePrintFrom(final Node node) {
		if (!node.isMarked()&&getNumberOfNodes()>0) {
			node.mark();
			System.out.println("  "+node.getId());
			for (int ne=0; ne<node.getOutDegree(); ne++) {
				counter++;
				final Node directSuccessor = node.getDirectSuccessor(ne);
				System.out.println("  "+node.getId()+" -> "+directSuccessor.getId());
				counter++;
				recursivePrintFrom(directSuccessor);
			}
		}
	}
	
	public String getData() {
		return size + "\t" + counter + "\n";
	}
	
	public void accept(AlgorithmVisitor visitor) {
		visitor.visit(this);
		
	}
}
