package ch.usi.inf.sape.algoprof.example.graph.directed.linked.basic.bidi;

import java.util.Random;

import ch.usi.inf.sape.algoprof.statistics.Algorithm;
import ch.usi.inf.sape.algoprof.statistics.AlgorithmCounterVisitor;
import ch.usi.inf.sape.algoprof.statistics.AlgorithmRepetitionTreeVisitor;
import ch.usi.inf.sape.algoprof.statistics.AlgorithmVisitor;
import ch.usi.inf.sape.algoprof.statistics.Configuration;


public class Main implements Algorithm {

	private static final Random RANDOM = new Random(0);
	
	private AlgorithmCounterVisitor makeVisitor = null;
	private AlgorithmCounterVisitor makeNodesVisitor = null;
	private AlgorithmCounterVisitor recursivePrintVisitor = null;
	private AlgorithmCounterVisitor markVisitor = null;
	private AlgorithmCounterVisitor toDotVisitor = null;
	
	public AlgorithmVisitor getVisitor(String algoName) {
		if (algoName.equals("makeComplete")) {
			return makeVisitor;
		} else if (algoName.equals("printToDot")) {
			return toDotVisitor;
		} else if (algoName.equals("mark")) {
			return markVisitor;
		} else if (algoName.equals("recursivePrint")) {
			return recursivePrintVisitor;
		} else if (algoName.equals("makeNodes")) {
			return makeNodesVisitor;
		}
		return null;
	}
	
	public static void main(final String[] args) {
		final int maxNodes = Integer.parseInt(args[0]);
		
		System.out.println("Complete directed graph with nodes in [0,"+maxNodes+"]");
		System.out.println("=> edges in [0,"+computeEdgeCount(maxNodes)+"]");
		
		new Main().harness(maxNodes);
	
	}
	
	public void harness(final int maxNodes) {
		boolean once = true;
		
		AlgorithmVisitor av = null;
		if (Configuration.ENABLE_MANUAL_INSTRUMENTATION) {
			markVisitor = new AlgorithmCounterVisitor("mark", "/src/ch/usi/inf/sape/algoprof/example/graph/directed/linked/basic/bidi/", " modification");
			makeNodesVisitor = new AlgorithmCounterVisitor("makeNodes", "/src/ch/usi/inf/sape/algoprof/example/graph/directed/linked/basic/bidi/", "java.util.List<Node> construction");
			makeVisitor = new AlgorithmCounterVisitor("makeComplete", "/src/ch/usi/inf/sape/algoprof/example/graph/directed/linked/basic/bidi/", "Node=>java.util.List<Node> modification");
			toDotVisitor = new AlgorithmCounterVisitor("printToDot", "/src/ch/usi/inf/sape/algoprof/example/graph/directed/linked/basic/bidi/", "Node=>java.util.List<Node> traversal");
			recursivePrintVisitor = new AlgorithmCounterVisitor("recursivePrint", "/src/ch/usi/inf/sape/algoprof/example/graph/directed/linked/basic/bidi/", "Node=>java.util.List<Node> traversal");
			av = new AlgorithmRepetitionTreeVisitor();
			accept(av);
		}
		for (int nodes=0; nodes<=maxNodes; nodes++) {
			Graph g = testForSize(nodes, once);
			if (once && Configuration.ENABLE_MANUAL_INSTRUMENTATION) {
				once = false;
				g.accept(av);
			} 
			
		}
		
		if (Configuration.ENABLE_MANUAL_INSTRUMENTATION) {
			accept(makeVisitor);
			accept(makeNodesVisitor);
			accept(recursivePrintVisitor);
			accept(toDotVisitor);
			accept(markVisitor);
		}
		
	}


	private static int computeEdgeCount(final int nodes) {
		return nodes*nodes-nodes;
	}

	private Graph testForSize(final int nodes, boolean once) {
		// construct
		System.out.println("--- nodes="+nodes+" ---");
		System.out.println("--> edges="+computeEdgeCount(nodes));
		final Graph graph = new Graph(nodes, this);
		
		// traversal
		//System.out.println("iterativePrint");
		//tree.iterativePrint();
		
		// traversal
		System.out.println("printToDot");
		graph.printToDot();
		if (Configuration.ENABLE_MANUAL_INSTRUMENTATION) {
			graph.accept(toDotVisitor);
		}

		// traversal
		System.out.println("recursivePrint");
		graph.recursivePrint();
		
		return graph;
		
		// traversal
		//System.out.println("nodeRecursivePrint");
		//graph.nodeRecursivePrint();
		
		// traversal
		//System.out.println("iterativeLength: "+tree.iterativeLength());
		
		// traversal
		//System.out.println("recursiveLength: "+tree.recursiveLength());
		
		// traversal
		//System.out.println("nodeRecursiveLength: "+tree.nodeRecursiveLength());
		
		// traversal
		//final String target = "n"+RANDOM.nextInt(size+1);
		//System.out.println("iterativeIndexOf("+target+"): "+tree.iterativeIndexOf(target));
		
		// traversal
		//System.out.println("recursiveIndexOf("+target+"): "+tree.recursiveIndexOf(target));
		
		// traversal
		//System.out.println("nodeRecursiveIndexOf("+target+"): "+tree.nodeRecursiveIndexOf(target));
		
	}
	
	public void accept(AlgorithmVisitor visitor) {
		visitor.visit(this);
		
	}

}
