package ch.usi.inf.sape.algoprof.example.graph.undirected.linked.basic;

import java.util.ArrayList;

import ch.usi.inf.sape.algoprof.statistics.Algorithm;
import ch.usi.inf.sape.algoprof.statistics.AlgorithmVisitor;
import ch.usi.inf.sape.algoprof.statistics.Configuration;


public class Graph implements Algorithm {

	private final ArrayList<Node> nodes;
	private Main m;
	private int counter;
	private int size;
	
	public Graph(final int nodeCount, Main m) {
		nodes = new ArrayList<Node>();
		this.m = m;
		counter = 0;
		size = nodeCount;
		makeComplete(nodeCount);
	}
	
	private void makeComplete(final int nodeCount) {
		for (int n=0; n<nodeCount; n++) {
			counter++;
			final Node node = new Node();
			node.setId(n);
			for (int ne=0; ne<getNumberOfNodes(); ne++) {
				counter++;
				final Node neighbor = getNode(ne);
				node.addNeighbor(neighbor);
			}
			addNode(node);
		}
	}
	
	public void addNode(final Node node) {
		nodes.add(node);
	}
	
	public int getNumberOfNodes() {
		return nodes.size();
	}
	
	public Node getNode(final int i) {
		return nodes.get(i);
	}
	
	public void printToDot() {
		System.out.println("graph G {");
		counter = 0;
		for (int n=0; n<getNumberOfNodes(); n++) {
			counter++;
			final Node node = getNode(n);
			System.out.println("  "+node.getId());
			for (int ne=0; ne<node.getDegree(); ne++) {
				counter++;
				final Node neighbor = node.getNeighbor(ne);
				System.out.println("  "+node.getId()+" -- "+neighbor.getId());
			}
		}
		System.out.println("}");
	}
	
	public void recursivePrint() {
		// clear marks
		counter=0;
		size = getNumberOfNodes();
		for (int n=0; n<getNumberOfNodes(); n++) {
			counter++;
			getNode(n).clear();
		}
		if (Configuration.ENABLE_MANUAL_INSTRUMENTATION) {
			accept(m.getVisitor("mark"));
		}
		// starting at each node (there is no specific root), start a DFS
		System.out.println("graph G {");
		counter = 0;
		
		for (int n=0; n<getNumberOfNodes(); n++) {
			counter++;
			recursivePrintFrom(getNode(n));
		}
		if (Configuration.ENABLE_MANUAL_INSTRUMENTATION) {
			accept(m.getVisitor("recursivePrint"));
		}
		System.out.println("}");
	}
	
	public void recursivePrintFrom(final Node node) {
		if (!node.isMarked()&&getNumberOfNodes()>0) {
			node.mark();
			System.out.println("  "+node.getId());
			for (int ne=0; ne<node.getDegree(); ne++) {
				counter++;
				final Node neighbor = node.getNeighbor(ne);
				System.out.println("  "+node.getId()+" -- "+neighbor.getId());
				recursivePrintFrom(neighbor);
				counter++;
			}
		}
	}
	
	public String getData() {
		return size + "\t" + counter + "\n";
	}
	
	public void accept(AlgorithmVisitor visitor) {
		visitor.visit(this);
		
	}
	
}
