package ch.usi.inf.sape.algoprof.example.list.array.naive.generic;


import java.util.Random;


import ch.usi.inf.sape.algoprof.statistics.Algorithm;
import ch.usi.inf.sape.algoprof.statistics.AlgorithmCounterVisitor;
import ch.usi.inf.sape.algoprof.statistics.AlgorithmRepetitionTreeVisitor;
import ch.usi.inf.sape.algoprof.statistics.AlgorithmVisitor;
import ch.usi.inf.sape.algoprof.statistics.Configuration;


public class Main implements Algorithm {

	private static final Random RANDOM = new Random(0);
	
	private AlgorithmCounterVisitor makeVisitor = null;
	private AlgorithmCounterVisitor recursivePrintVisitor = null;
	private AlgorithmCounterVisitor iterativePrintVisitor = null;
	private AlgorithmCounterVisitor iterativePrintForEachVisitor = null;
	private AlgorithmCounterVisitor iterativeIndexVisitor = null;
	private AlgorithmCounterVisitor recursiveIndexVisitor = null;
	
	public AlgorithmVisitor getVisitor(String algoName) {
		if (algoName.equals("make")) {
			return makeVisitor;
		} else if (algoName.equals("recursiveIndex")) {
			return recursiveIndexVisitor;
		} else if (algoName.equals("iterativeIndex")) {
			return iterativeIndexVisitor;
		} else if (algoName.equals("recursivePrint")) {
			return recursivePrintVisitor;
		} else if (algoName.equals("iterativePrint")) {
			return iterativePrintVisitor;
		} else if (algoName.equals("iterativePrintForEach")) {
			return iterativePrintForEachVisitor;
		}
		return null;
	}
	
	public static void main(final String[] args) {
		final int maxSize = Integer.parseInt(args[0]);
		
		new Main().harnesss(maxSize);
	}
	
	public void harnesss(final int maxSize) {
		boolean once = true;
		
		AlgorithmVisitor av = null;
		if (Configuration.ENABLE_MANUAL_INSTRUMENTATION) {
			iterativePrintVisitor = new AlgorithmCounterVisitor("iterativePrint", "/src/ch/usi/inf/sape/algoprof/example/list/array/naive/generic/", "T[] traversal");
			iterativePrintForEachVisitor = new AlgorithmCounterVisitor("iterativePrintForEach", "/src/ch/usi/inf/sape/algoprof/example/list/array/naive/generic/", "T[] traversal");
			recursivePrintVisitor = new AlgorithmCounterVisitor("recursivePrint", "/src/ch/usi/inf/sape/algoprof/example/list/array//naive/generic/", "T[] traversal");
			iterativeIndexVisitor = new AlgorithmCounterVisitor("iterativeIndex", "/src/ch/usi/inf/sape/algoprof/example/list/array/naive/generic/", "T[] traversal");
			recursiveIndexVisitor = new AlgorithmCounterVisitor("recursiveIndex", "/src/ch/usi/inf/sape/algoprof/example/list/array/naive/generic/", "T[] traversal");
			makeVisitor = new AlgorithmCounterVisitor("make", "/src/ch/usi/inf/sape/algoprof/example/list/array/naive/generic/", "T[] for List construction");
			av = new AlgorithmRepetitionTreeVisitor();
			accept(av);
		}
		// change input size (list length) between 0 and maxSize, inclusive
		//int[] arr = new int[] {1,2,3,4,5,10,15,20,30,50,70,100,150,200,300,400,500,600,700,800,900,1000,1200,1400,1600,1800,2000, 3000, 5000};
		//for (int i: arr) {
		for (int i=0; i<=maxSize; i++) {	
			ArrayList<String> l = testForSize(i);
			if (once && Configuration.ENABLE_MANUAL_INSTRUMENTATION) {
				once = false;
				l.accept(av);
			} 
			
		}
		
		if (Configuration.ENABLE_MANUAL_INSTRUMENTATION) {
			accept(makeVisitor);
			accept(recursivePrintVisitor);
			accept(iterativePrintVisitor);
			accept(recursiveIndexVisitor);
			accept(iterativeIndexVisitor);
			accept(iterativePrintForEachVisitor);
		}
		
	}

	private ArrayList<String> testForSize(final int size) {
		// construct
		System.out.println("--- testForSize("+size+") ---");
		final ArrayList<String> list = new ArrayList<String>();
		for (int i = 0; i < size; i++) {
			list.append("n"+i);
			list.increaseCounter();
		}
		if (Configuration.ENABLE_MANUAL_INSTRUMENTATION) {
			list.accept(makeVisitor);
		}
		
		// traversal
		System.out.println("iterativePrint");
		list.iterativePrint();
		if (Configuration.ENABLE_MANUAL_INSTRUMENTATION) {
			list.accept(iterativePrintVisitor);
		}
		
		// traversal
		System.out.println("iterativePrintForEach");
		list.iterativePrintForEach();
		if (Configuration.ENABLE_MANUAL_INSTRUMENTATION) {
			list.accept(iterativePrintForEachVisitor);
		}
		// traversal
		System.out.println("recursivePrint");
		list.recursivePrint();
		if (Configuration.ENABLE_MANUAL_INSTRUMENTATION) {
			list.accept(recursivePrintVisitor);
		}
		// traversal
		final String target = "n"+RANDOM.nextInt(size+1);
		System.out.println("iterativeIndexOf("+target+"): "+list.iterativeIndexOf(target));
		if (Configuration.ENABLE_MANUAL_INSTRUMENTATION) {
			list.accept(iterativeIndexVisitor);
		}
		// traversal
		System.out.println("recursiveIndexOf("+target+"): "+list.recursiveIndexOf(target));
		if (Configuration.ENABLE_MANUAL_INSTRUMENTATION) {
			list.accept(recursiveIndexVisitor);
		}
		return list;
		
	}

	public void accept(AlgorithmVisitor visitor) {
		visitor.visit(this);
		
	}
	
}
