package ch.usi.inf.sape.algoprof.example.list.linked.basic;

import ch.usi.inf.sape.algoprof.statistics.Algorithm;
import ch.usi.inf.sape.algoprof.statistics.AlgorithmVisitor;


public class LinkedList implements Algorithm {

	private Node head;
	private int counter;
	private int size;
	
	public LinkedList() {
		head = null;
		size = 0;
	}
	
	public void increaseCounter() {
		counter++;
	}
	
	public void emptyCounter() {
		counter = 0;
	}
	
	public void prepend(final String value) {
		head = new Node(value, head);
		size++;
	}
	
	// length
	public int iterativeLength() {
		Node current = head;
		int length = 0;
		while (current!=null) {
			counter++;
			length++;
			current = current.getRest();
		}
		return length;
	}
	
	public int nodeRecursiveLength() {
		if (head==null) {
			return 0;
		} else {
			counter = head.recursiveLength();
			return head.recursiveLength();
		}
	}
	
	public int recursiveLength() {
		return recursiveLengthFrom(head);
	}
	
	private int recursiveLengthFrom(final Node node) {
		counter++;
		if (node==null) {
			return 0;
		} else {
			return 1+recursiveLengthFrom(node.getRest());
		}
	}
	
	// indexOf
	public int iterativeIndexOf(final String value) {
		Node current = head;
		int index = 0;
		while ( current!=null && (!current.getValue().equals(value)) ) {
			counter++;
			index++;
			current = current.getRest();
		}
		if (current==null) {
			return -1;
		} else {
			return index;
		}
	}
	
	public int nodeRecursiveIndexOf(final String value) {
		if (head==null) {
			return -1;
		} else {
			counter = head.recursiveIndexOf(value, 0);
			return head.recursiveIndexOf(value, 0);
		}
	}
	
	public int recursiveIndexOf(final String value) {
		return recursiveIndexOfFrom(head, value, 0);
	}
	
	private int recursiveIndexOfFrom(final Node node, final String value, int index) {
		counter++;
		if (node==null) {
			return -1;
		} else {
			if (node.getValue().equals(value)) {
				return index;
			} else {
				return recursiveIndexOfFrom(node.getRest(), value, index+1);
			}
		}
	}

	// print
	public void iterativePrint() {
		Node current = head;
		int length = 0;
		while (current!=null) {
			counter++;
			System.out.print(current.getValue()+" ");
			length++;
			current = current.getRest();
		}
		System.out.println();
	}
	
	public void nodeRecursivePrint() {
		if (head==null) {
			System.out.println();;
		} else {
			counter = head.recursiveLength();
			head.recursivePrint();
		}
	}
	
	public void recursivePrint() {
		recursivePrintFrom(head);
		System.out.println();
	}
	
	private void recursivePrintFrom(final Node node) {
		counter++;
		if (node==null) {
			return;
		} else {
			System.out.print(node.getValue()+" ");
			recursivePrintFrom(node.getRest());
		}
	}

	public void accept(AlgorithmVisitor visitor) {
		visitor.visit(this);
		
	}
	public String getData() {
		return size + "\t" + counter + "\n";
	}

}
