package ch.usi.inf.sape.algoprof.example.list.linked.generic;


import java.util.Random;
import ch.usi.inf.sape.algoprof.statistics.Algorithm;
import ch.usi.inf.sape.algoprof.statistics.AlgorithmCounterVisitor;
import ch.usi.inf.sape.algoprof.statistics.AlgorithmRepetitionTreeVisitor;
import ch.usi.inf.sape.algoprof.statistics.AlgorithmVisitor;
import ch.usi.inf.sape.algoprof.statistics.Configuration;


public class Main implements Algorithm {

	private static final Random RANDOM = new Random(0);
	
	private AlgorithmCounterVisitor makeVisitor = null;
	private AlgorithmCounterVisitor recursivePrintVisitor = null;
	private AlgorithmCounterVisitor iterativePrintVisitor = null;
	private AlgorithmCounterVisitor nodeRecursivePrintVisitor = null;
	private AlgorithmCounterVisitor recursiveLengthVisitor = null;
	private AlgorithmCounterVisitor iterativeLengthVisitor = null;
	private AlgorithmCounterVisitor nodeRecursiveLengthVisitor = null;
	private AlgorithmCounterVisitor nodeRecursiveIndexVisitor = null;
	private AlgorithmCounterVisitor iterativeIndexVisitor = null;
	private AlgorithmCounterVisitor recursiveIndexVisitor = null;
	
	public AlgorithmVisitor getVisitor(String algoName) {
		if (algoName.equals("make")) {
			return makeVisitor;
		} else if (algoName.equals("recursiveIndex")) {
			return recursiveIndexVisitor;
		} else if (algoName.equals("iterativeIndex")) {
			return iterativeIndexVisitor;
		} else if (algoName.equals("nodeRecursiveIndex")) {
			return nodeRecursiveIndexVisitor;
		} else if (algoName.equals("recursivePrint")) {
			return recursivePrintVisitor;
		} else if (algoName.equals("iterativePrint")) {
			return iterativePrintVisitor;
		} else if (algoName.equals("nodeRecursivePrint")) {
			return nodeRecursivePrintVisitor;
		} else if (algoName.equals("recursiveLength")) {
			return recursiveLengthVisitor;
		} else if (algoName.equals("iterativeLength")) {
			return iterativeLengthVisitor;
		} else if (algoName.equals("nodeRecursiveLength")) {
			return nodeRecursiveLengthVisitor;
		} 
		
		return null;
	}
	
	
	public static void main(final String[] args) {
		final int maxSize = Integer.parseInt(args[0]);
		
		new Main().harnesss(maxSize);
	}
	
	public void harnesss(final int maxSize) {
		boolean once = true;
		
		AlgorithmVisitor av = null;
		if (Configuration.ENABLE_MANUAL_INSTRUMENTATION) {
			iterativePrintVisitor = new AlgorithmCounterVisitor("iterativePrint", "/src/ch/usi/inf/sape/algoprof/example/list/linked/generic/", "LinkedList T traversal");
			recursivePrintVisitor = new AlgorithmCounterVisitor("recursivePrint", "/src/ch/usi/inf/sape/algoprof/example/list/linked/generic/", "LinkedList T traversal");
			nodeRecursivePrintVisitor = new AlgorithmCounterVisitor("nodeRecursivePrint", "/src/ch/usi/inf/sape/algoprof/example/list/linked/generic/", "LinkedList T traversal");
			iterativeIndexVisitor = new AlgorithmCounterVisitor("iterativeIndex", "/src/ch/usi/inf/sape/algoprof/example/list/linked/generic/", "LinkedList T traversal");
			recursiveIndexVisitor = new AlgorithmCounterVisitor("recursiveIndex", "/src/ch/usi/inf/sape/algoprof/example/list//linked/generic/", "LinkedList T traversal");
			nodeRecursiveIndexVisitor = new AlgorithmCounterVisitor("nodeRecursiveIndex", "/src/ch/usi/inf/sape/algoprof/example/list//linked/generic/", "LinkedList T traversal");
			iterativeLengthVisitor = new AlgorithmCounterVisitor("iterativeLength", "/src/ch/usi/inf/sape/algoprof/example/list/linked/generic/", "LinkedList T traversal");
			recursiveLengthVisitor = new AlgorithmCounterVisitor("recursiveLength", "/src/ch/usi/inf/sape/algoprof/example/list//linked/generic/", "LinkedList T traversal");
			nodeRecursiveLengthVisitor = new AlgorithmCounterVisitor("nodeRecursiveLength", "/src/ch/usi/inf/sape/algoprof/example/list//linked/generic/", "LinkedList T traversal");
			makeVisitor = new AlgorithmCounterVisitor("make", "/src/ch/usi/inf/sape/algoprof/example/list/linked/generic/", "LinkedList T construction");
			av = new AlgorithmRepetitionTreeVisitor();
			accept(av);
		}
		// change input size (list length) between 0 and maxSize, inclusive
		//int[] arr = new int[] {1,2,3,4,5,10,15,20,30,50,70,100,150,200,300,400,500,600,700,800,900,1000,1200,1400,1600,1800,2000, 3000, 5000};
		//for (int i: arr) {
		for (int i=0; i<=maxSize; i++) {	
			LinkedList l = testForSize(i);
			if (once && Configuration.ENABLE_MANUAL_INSTRUMENTATION) {
				once = false;
				l.accept(av);
			} 
			
		}
		
		if (Configuration.ENABLE_MANUAL_INSTRUMENTATION) {
			accept(makeVisitor);
			accept(recursivePrintVisitor);
			accept(iterativePrintVisitor);
			accept(nodeRecursivePrintVisitor);
			accept(recursiveIndexVisitor);
			accept(iterativeIndexVisitor);
			accept(nodeRecursiveIndexVisitor);
			accept(recursiveLengthVisitor);
			accept(iterativeLengthVisitor);
			accept(nodeRecursiveLengthVisitor);
		}
		
	}
	private LinkedList testForSize(final int size) {
		// construct
		System.out.println("--- testForSize("+size+") ---");
		final LinkedList list = new LinkedList();
		list.emptyCounter();
		for (int i = 0; i < size; i++) {
			list.increaseCounter();
			list.prepend("n"+i);
		}
		if (Configuration.ENABLE_MANUAL_INSTRUMENTATION) {
			list.accept(makeVisitor);
		}
		
		// traversal
		System.out.println("iterativePrint");
		list.emptyCounter();
		list.iterativePrint();
		if (Configuration.ENABLE_MANUAL_INSTRUMENTATION) {
			list.accept(iterativePrintVisitor);
		}
		// traversal
		System.out.println("recursivePrint");
		list.emptyCounter();
		list.recursivePrint();
		if (Configuration.ENABLE_MANUAL_INSTRUMENTATION) {
			list.accept(recursivePrintVisitor);
		}
		// traversal
		System.out.println("nodeRecursivePrint");
		list.emptyCounter();
		list.nodeRecursivePrint();
		if (Configuration.ENABLE_MANUAL_INSTRUMENTATION) {
			list.accept(nodeRecursivePrintVisitor);
		}
		// traversal
		list.emptyCounter();
		System.out.println("iterativeLength: "+list.iterativeLength());
		if (Configuration.ENABLE_MANUAL_INSTRUMENTATION) {
			list.accept(iterativeLengthVisitor);
		}
		// traversal
		list.emptyCounter();
		System.out.println("recursiveLength: "+list.recursiveLength());
		if (Configuration.ENABLE_MANUAL_INSTRUMENTATION) {
			list.accept(recursiveLengthVisitor);
		}
		// traversal
		list.emptyCounter();
		System.out.println("nodeRecursiveLength: "+list.nodeRecursiveLength());
		if (Configuration.ENABLE_MANUAL_INSTRUMENTATION) {
			list.accept(nodeRecursiveLengthVisitor);
		}
		// traversal
		final String target = "n"+RANDOM.nextInt(size+1);
		list.emptyCounter();
		System.out.println("iterativeIndexOf("+target+"): "+list.iterativeIndexOf(target));
		if (Configuration.ENABLE_MANUAL_INSTRUMENTATION) {
			list.accept(iterativeIndexVisitor);
		}
		// traversal
		list.emptyCounter();
		System.out.println("recursiveIndexOf("+target+"): "+list.recursiveIndexOf(target));
		if (Configuration.ENABLE_MANUAL_INSTRUMENTATION) {
			list.accept(recursiveIndexVisitor);
		}
		// traversal
		list.emptyCounter();
		System.out.println("nodeRecursiveIndexOf("+target+"): "+list.nodeRecursiveIndexOf(target));
		if (Configuration.ENABLE_MANUAL_INSTRUMENTATION) {
			list.accept(nodeRecursiveIndexVisitor);
		}
		return list;
		
	}

	public void accept(AlgorithmVisitor visitor) {
		visitor.visit(this);
	}
	
}
