package ch.usi.inf.sape.algoprof.example.sort.insertion;

import ch.usi.inf.sape.algoprof.statistics.Algorithm;
import ch.usi.inf.sape.algoprof.statistics.AlgorithmVisitor;


public class List implements Algorithm {

	private Node head;
	private Node tail;
	private int counter;
	private int size;
	
	
	public List() {
		head = null;
		tail = null;
		counter = 0;
		size = 0;
	}
	
	public void append(int value) {
		size++;
		final Node node = new Node(value);
		if (tail==null) {
			tail = node;
			head = tail;
		} else {
			tail.next = node;
			node.prev = tail;
			tail = tail.next;
		}
	}
	
	public void print() {
		Node node = head;
		counter = 0;
		while (node!=null) {
			counter++;
			System.out.print(node.getValue()+", ");
			node = node.next;
		}
		System.out.println();
	}
	
	public void sort() {
		counter = 0;
		if (head==null || head.next==null) {
			// empty or singleton -> sorted
			return;
		}
		int comparisonCount = 0;
		int length = 1;
		//node firstUnsorted = head.next;
		Node firstUnsorted = new Node(3);
		boolean firstTime = true;
		while (firstUnsorted!=null) {
			if (firstTime) {
				firstTime = false;
				firstUnsorted = head.next;
			}
			counter++;
			//System.out.println("Sort step: firstUnsorted="+firstUnsorted.value);
			//System.out.print("           ");
			//print();
			Node target = firstUnsorted;
			Node nextUnsorted = firstUnsorted.next;
			comparisonCount++; // testing condition whether to enter inner loop
			while (target.prev!=null && target.prev.value>target.value) {
				final Node candidate = target.prev;
				//System.out.println("     swap: "+candidate.value+" > "+target.value);
				// BEFORE: pred <-> candidate <-> target <-> succ
				// swap target and target.prev
				// AFTER: pred <-> target <-> candidate <-> succ
				final Node pred = candidate.prev;
				final Node succ = target.next;
				if (pred!=null) {
					pred.next = target;
				} else {
					head = target;
				}
				target.prev = pred;
				if (succ!=null) {
					succ.prev = candidate;
				} else {
					tail = candidate;
				}
				candidate.next = succ;
				target.next = candidate;
				candidate.prev = target;
				comparisonCount++; // testing condition whether to stay in inner loop
				counter++;
			}
			// // I originally had this interesting "complexity bug":
			//firstUnsorted = firstUnsorted.next;
			firstUnsorted = nextUnsorted;
			// // The buggy version worked correctly, but it meant that the outer loop would not only touch each element once,
			// // but that it would fall back to an earlier element and thus touch some elements several times
			// // (because firstUnsorted points to a Node that was just moved toward the beginning of the List)
			length++;
		}
		//System.out.println(length+"\t"+comparisonCount);
	}
	
	public void setData(AlgorithmVisitor visitor, int counter) {
		this.counter = counter;
		accept(visitor);
	}
	
	public String getData() {
		return size + "\t" + counter + "\n";
	}

	public void accept(AlgorithmVisitor visitor) {
		visitor.visit(this);
		
	}
	
}
