package ch.usi.inf.sape.algoprof.example.tree.array.perfect.binary;

import java.util.Random;

import ch.usi.inf.sape.algoprof.statistics.Algorithm;
import ch.usi.inf.sape.algoprof.statistics.AlgorithmCounterVisitor;
import ch.usi.inf.sape.algoprof.statistics.AlgorithmRepetitionTreeVisitor;
import ch.usi.inf.sape.algoprof.statistics.AlgorithmVisitor;
import ch.usi.inf.sape.algoprof.statistics.Configuration;


public class Main implements Algorithm {

	private static final Random RANDOM = new Random(0);
	private AlgorithmCounterVisitor makeVisitor = null;
	private AlgorithmCounterVisitor recursivePrintVisitor = null;

	public AlgorithmVisitor getVisitor(String algoName) {
		if (algoName.equals("make")) {
			return makeVisitor;
		} else if (algoName.equals("recursivePrint")) {
			return recursivePrintVisitor;
		}
		return null;
	}
	
	public static void main(final String[] args) {
		final int maxHeight = Integer.parseInt(args[0]);
		
		System.out.println("Perfect binary trees with height in [0,"+maxHeight+"]");
		System.out.println("=> nodes in [1,"+computeNodeCount(maxHeight)+"]");
		
		new Main().harness(maxHeight);
	}
	
	public void harness(final int maxHeight) {
		boolean once = true;
		
		AlgorithmVisitor av = null;
		if (Configuration.ENABLE_MANUAL_INSTRUMENTATION) {
			recursivePrintVisitor = new AlgorithmCounterVisitor("recursivePrint", "/src/ch/usi/inf/sape/algoprof/example/tree/array/perfect/binary/", "BinaryTree int[] traversal");
			makeVisitor = new AlgorithmCounterVisitor("make", "/src/ch/usi/inf/sape/algoprof/example/tree/array/perfect/binary/", "BinaryTree int[] Node modification");
			av = new AlgorithmRepetitionTreeVisitor();
			accept(av);
		}
		// change input size (list length) between 0 and maxSize, inclusive
		//int[] arr = new int[] {1,2,3,4,5,10,15,20,30,50,70,100,150,200,300,400,500,600,700,800,900,1000,1200,1400,1600,1800,2000, 3000, 5000};
		//for (int i: arr) {
		for (int i=0; i<=maxHeight; i++) {	
			BinaryTree tree = testForSize(i);
			if (once && Configuration.ENABLE_MANUAL_INSTRUMENTATION) {
				once = false;
				tree.accept(av);
			} 
			
		}
		
		if (Configuration.ENABLE_MANUAL_INSTRUMENTATION) {
			accept(makeVisitor);
			accept(recursivePrintVisitor);
		}
		
	}


	private static int computeNodeCount(final int height) {
		return (int)(Math.pow(2, height+1)-1);
	}

	private BinaryTree testForSize(final int height) {
		// construct
		System.out.println("--- height="+height+" ---");
		System.out.println("--> nodes="+computeNodeCount(height));
		final BinaryTree tree = new BinaryTree(height);
		if (Configuration.ENABLE_MANUAL_INSTRUMENTATION) {
			tree.accept(makeVisitor);
		}
		// traversal
		//System.out.println("iterativePrint");
		//tree.iterativePrint();
		
		// traversal
		System.out.println("recursivePrint");
		tree.recursivePrint();
		if (Configuration.ENABLE_MANUAL_INSTRUMENTATION) {
			tree.accept(recursivePrintVisitor);
		}
		
		// traversal
		//System.out.println("iterativeLength: "+tree.iterativeLength());
		
		// traversal
		//System.out.println("recursiveLength: "+tree.recursiveLength());
		
		// traversal
		//System.out.println("nodeRecursiveLength: "+tree.nodeRecursiveLength());
		
		// traversal
		//final String target = "n"+RANDOM.nextInt(size+1);
		//System.out.println("iterativeIndexOf("+target+"): "+tree.iterativeIndexOf(target));
		
		// traversal
		//System.out.println("recursiveIndexOf("+target+"): "+tree.recursiveIndexOf(target));
		
		// traversal
		//System.out.println("nodeRecursiveIndexOf("+target+"): "+tree.nodeRecursiveIndexOf(target));
		
		return tree;
		
	}

	public void accept(AlgorithmVisitor visitor) {
		visitor.visit(this);
		
	}
	

}
