package ch.usi.inf.sape.algoprof.example.tree.linked.perfect.nary.basic.bidi;

import java.util.Random;
import ch.usi.inf.sape.algoprof.statistics.Algorithm;
import ch.usi.inf.sape.algoprof.statistics.AlgorithmCounterVisitor;
import ch.usi.inf.sape.algoprof.statistics.AlgorithmRepetitionTreeVisitor;
import ch.usi.inf.sape.algoprof.statistics.AlgorithmVisitor;
import ch.usi.inf.sape.algoprof.statistics.Configuration;


public class Main implements Algorithm {

	private static final Random RANDOM = new Random(0);
	private AlgorithmCounterVisitor makeVisitor = null;
	private AlgorithmCounterVisitor recursivePrintVisitor = null;
	private AlgorithmCounterVisitor nodeRecursivePrintVisitor = null;

	public AlgorithmVisitor getVisitor(String algoName) {
		if (algoName.equals("make")) {
			return makeVisitor;
		} else if (algoName.equals("recursivePrint")) {
			return recursivePrintVisitor;
		} else if (algoName.equals("nodeRecursivePrint")) {
			return nodeRecursivePrintVisitor;
		}
		return null;
	}	

	
	public static void main(final String[] args) {
		final int maxHeight = Integer.parseInt(args[0]);
		final int maxBranchingFactor = Integer.parseInt(args[1]);
		
		System.out.println("Perfect n-ary trees with height in [0,"+maxHeight+"] and branching factor in [2,"+maxBranchingFactor+"]");
		System.out.println("=> nodes in [1,"+computeNodeCount(maxHeight, maxBranchingFactor)+"]");
		new Main().harness(maxHeight, maxBranchingFactor);
	}
	
	public void harness(final int maxHeight, final int maxBranchingFactor) {
		boolean once = true;
		
		AlgorithmVisitor av = null;
		if (Configuration.ENABLE_MANUAL_INSTRUMENTATION) {
			recursivePrintVisitor = new AlgorithmCounterVisitor("recursivePrint", "/src/ch/usi/inf/sape/algoprof/example/tree/linked/perfect/nary/basic/bidi/", "NaryTree Node traversal");
			nodeRecursivePrintVisitor = new AlgorithmCounterVisitor("nodeRecursivePrint", "/src/ch/usi/inf/sape/algoprof/example/tree/linked/perfect/nary/basic/bidi/", "NaryTree Node traversal");
			makeVisitor = new AlgorithmCounterVisitor("make", "/src/ch/usi/inf/sape/algoprof/example/tree/linked/perfect/nary/basic/bidi/", "NaryTree int[] Node modification");
			av = new AlgorithmRepetitionTreeVisitor();
			accept(av);
		}
		// change input size (list length) between 0 and maxSize, inclusive
		//int[] arr = new int[] {1,2,3,4,5,10,15,20,30,50,70,100,150,200,300,400,500,600,700,800,900,1000,1200,1400,1600,1800,2000, 3000, 5000};
		//for (int i: arr) {
		
		for (int height=0; height<=maxHeight; height++) {
			for (int branchingFactor=2; branchingFactor<=maxBranchingFactor; branchingFactor++) {
				NaryTree tree = testForSize(height, branchingFactor);
				if (once && Configuration.ENABLE_MANUAL_INSTRUMENTATION) {
					once = false;
					tree.accept(av);
				} 
			}
		}
				
		if (Configuration.ENABLE_MANUAL_INSTRUMENTATION) {
			accept(makeVisitor);
			accept(recursivePrintVisitor);
			accept(nodeRecursivePrintVisitor);
		}
		
	}

	private static int computeNodeCount(final int height, final int branchingFactor) {
		return (int)(Math.pow(branchingFactor, height+1)-1)/(branchingFactor-1);
	}
	
	private NaryTree testForSize(final int height, final int branchingFactor) {
		// construct
		System.out.println("--- height="+height+", branchingFactor="+branchingFactor+" ---");
		System.out.println("--> nodes="+computeNodeCount(height, branchingFactor));
		final NaryTree tree = new NaryTree(height, branchingFactor);
		if (Configuration.ENABLE_MANUAL_INSTRUMENTATION) {
			tree.accept(makeVisitor);
		}
		// traversal
		//System.out.println("iterativePrint");
		//tree.iterativePrint();
		
		// traversal
		System.out.println("recursivePrint");
		tree.recursivePrint();
		if (Configuration.ENABLE_MANUAL_INSTRUMENTATION) {
			tree.accept(recursivePrintVisitor);
		}
		// traversal
		System.out.println("nodeRecursivePrint");
		tree.nodeRecursivePrint();
		if (Configuration.ENABLE_MANUAL_INSTRUMENTATION) {
			tree.accept(nodeRecursivePrintVisitor);
		}
		// traversal
		//System.out.println("iterativeLength: "+tree.iterativeLength());
		
		// traversal
		//System.out.println("recursiveLength: "+tree.recursiveLength());
		
		// traversal
		//System.out.println("nodeRecursiveLength: "+tree.nodeRecursiveLength());
		
		// traversal
		//final String target = "n"+RANDOM.nextInt(size+1);
		//System.out.println("iterativeIndexOf("+target+"): "+tree.iterativeIndexOf(target));
		
		// traversal
		//System.out.println("recursiveIndexOf("+target+"): "+tree.recursiveIndexOf(target));
		
		// traversal
		//System.out.println("nodeRecursiveIndexOf("+target+"): "+tree.nodeRecursiveIndexOf(target));
		return tree;
		
	}

	public void accept(AlgorithmVisitor visitor) {
		visitor.visit(this);
		
	}
	
}
