package ch.usi.inf.sape.algoprof.statistics;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;


import ch.usi.inf.sape.algoprof.example.array.multidimensional.IntegerArray;
import ch.usi.inf.sape.algoprof.example.array.multidimensional.Main;
import ch.usi.inf.sape.algoprof.example.graph.directed.adjacencymatrix.basic.Graph;
import ch.usi.inf.sape.algoprof.example.graph.directed.linked.basic.Node;
import ch.usi.inf.sape.algoprof.example.list.array.doubling.basic.ArrayList;
import ch.usi.inf.sape.algoprof.example.list.array.naive.inheritance2.PingValue;
import ch.usi.inf.sape.algoprof.example.list.array.naive.inheritance2.PongValue;
import ch.usi.inf.sape.algoprof.example.list.linked.basic.LinkedList;
import ch.usi.inf.sape.algoprof.example.list.linked.inheritance.PingNode;
import ch.usi.inf.sape.algoprof.example.list.linked.inheritance.PongNode;
import ch.usi.inf.sape.algoprof.example.sort.insertion.List;
import ch.usi.inf.sape.algoprof.example.sort.insertion2.InsertionSortAlgorithm;
import ch.usi.inf.sape.algoprof.example.tree.array.perfect.binary.BinaryTree;
import ch.usi.inf.sape.algoprof.example.tree.linked.perfect.nary.basic.NaryTree;

public class AlgorithmCounterVisitor implements AlgorithmVisitor {

	private BufferedWriter rScript;
	private BufferedWriter dataFile;
	private String path;
	private String appendix;
	private String xlab; 
	
	public String getName() {
		return appendix;
	}
	
	private void flushAndExecute (String cost) {
		Script script = new Script(cost, appendix, path, xlab);
		try {
			dataFile.flush();
			dataFile.close();
			RExecutor re = new RExecutor(script);
			re.run(rScript, System.getProperty("user.dir") + path+appendix+".r");
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	public AlgorithmCounterVisitor(final String appendix, final String path, final String xlab) {
		try {
			this.xlab = xlab;
			this.appendix = appendix;
			this.path = path;
			rScript = new BufferedWriter(new FileWriter(new File(System.getProperty("user.dir") + path + appendix + ".r")));
			dataFile = new BufferedWriter(new FileWriter(new File(System.getProperty("user.dir") + path + appendix + ".txt")));
			dataFile.write("Size\tSteps\n");
		} catch (IOException e) {
			
			e.printStackTrace();
		}
	}
	
	public void visit(IntegerArray ia) {
		try {
			dataFile.write(ia.getData());
		} catch (IOException e) {
			e.printStackTrace();
		}
		
	}

	public void visit(Main m) {
		flushAndExecute("size");
	}

	public void visit(
			ch.usi.inf.sape.algoprof.example.array.multidimensional.local.IntegerArray ia) {
		try {
			dataFile.write(ia.getData());
		} catch (IOException e) {
			e.printStackTrace();
		}
		
	}

	public void visit(
			ch.usi.inf.sape.algoprof.example.array.multidimensional.local.Main m) {
		flushAndExecute("size");
		
	}

	public void visit(Graph g) {
		try {
			dataFile.write(g.getData());
		} catch (IOException e) {
			e.printStackTrace();
		}
		
	}

	public void visit(
			ch.usi.inf.sape.algoprof.example.graph.directed.adjacencymatrix.basic.Main m) {
		if (appendix.equals("makeComplete")) {
			flushAndExecute("size + sqrt(size)");
		} else if (appendix.equals("printToDot")) {
			flushAndExecute("size + sqrt(size)");
		} else if (appendix.equals("mark")) {
			flushAndExecute("size");
		} else if (appendix.equals("recursivePrint")) {
			flushAndExecute("size*2 + sqrt(size)");
		}	
	}

	public void visit(
			ch.usi.inf.sape.algoprof.example.graph.directed.linked.basic.Main m) {
		if (appendix.equals("makeComplete")) {
			flushAndExecute("size + sqrt(size)");
		} else if (appendix.equals("printToDot")) {
			flushAndExecute("size");
		} else if (appendix.equals("mark")) {
			flushAndExecute("size");
		} else if (appendix.equals("recursivePrint")) {
			flushAndExecute("size*2 - sqrt(size)");
		} else if (appendix.equals("makeNodes")) {
			flushAndExecute("size");
		}
		
	}

	public void visit(Node n) {
		// TODO Auto-generated method stub
		
	}

	public void visit(
			ch.usi.inf.sape.algoprof.example.graph.directed.linked.basic.Graph g) {
	
		try {
			dataFile.write(g.getData());
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public void visit(
			ch.usi.inf.sape.algoprof.example.graph.directed.linked.basic.bidi.Graph g) {
		try {
			dataFile.write(g.getData());
		} catch (IOException e) {
			e.printStackTrace();
		}
		
	}

	public void visit(
			ch.usi.inf.sape.algoprof.example.graph.directed.linked.basic.bidi.Main m) {
		if (appendix.equals("makeComplete")) {
			flushAndExecute("size + sqrt(size)");
		} else if (appendix.equals("printToDot")) {
			flushAndExecute("size");
		} else if (appendix.equals("mark")) {
			flushAndExecute("size");
		} else if (appendix.equals("recursivePrint")) {
			flushAndExecute("size*2 - sqrt(size)");
		} else if (appendix.equals("makeNodes")) {
			flushAndExecute("size");
		}
		
	}

	public void visit(
			ch.usi.inf.sape.algoprof.example.graph.directed.linked.basic.bidi.Node n) {
		// TODO Auto-generated method stub
		
	}

	public void visit(
			ch.usi.inf.sape.algoprof.example.graph.undirected.linked.basic.Graph g) {
		try {
			dataFile.write(g.getData());
		} catch (IOException e) {
			e.printStackTrace();
		}
		
	}

	public void visit(
			ch.usi.inf.sape.algoprof.example.graph.undirected.linked.basic.Main m) {
		if (appendix.equals("makeComplete")) {
			flushAndExecute("0.5*size*size + 0.5*size");
		} else if (appendix.equals("printToDot")) {
			flushAndExecute("0.5*size*size + 0.5*size");
		} else if (appendix.equals("mark")) {
			flushAndExecute("size");
		} else if (appendix.equals("recursivePrint")) {
			flushAndExecute("size*size");
		}
		
	}

	public void visit(
			ch.usi.inf.sape.algoprof.example.graph.undirected.linked.basic.Node n) {
		// TODO Auto-generated method stub
		
	}

	public void visit(ArrayList al) {
		try {
			dataFile.write(al.getData());
		} catch (IOException e) {
			e.printStackTrace();
		}	
	}

	public void visit(
			ch.usi.inf.sape.algoprof.example.list.array.doubling.basic.Main m) {
		if (appendix.equals("make")) {
			flushAndExecute("2*size");
		} else if (appendix.equals("iterativePrint")) {
			flushAndExecute("size");
		} else if (appendix.equals("recursivePrint")) {
			flushAndExecute("size");
		} else if (appendix.equals("iterativeIndex")) {
			flushAndExecute("size");
		} else if (appendix.equals("recursiveIndex")) {
			flushAndExecute("size");
		}
		
	}

	public void visit(
			ch.usi.inf.sape.algoprof.example.list.array.naive.basic.ArrayList al) {
		try {
			dataFile.write(al.getData());
		} catch (IOException e) {
			e.printStackTrace();
		}	
		
	}

	public void visit(
			ch.usi.inf.sape.algoprof.example.list.array.naive.basic.Main m) {
		if (appendix.equals("make")) {
			flushAndExecute("0.5*size*size - 0.5*size");
		} else if (appendix.equals("iterativePrint")) {
			flushAndExecute("size");
		} else if (appendix.equals("recursivePrint")) {
			flushAndExecute("size");
		} else if (appendix.equals("iterativeIndex")) {
			flushAndExecute("size");
		} else if (appendix.equals("recursiveIndex")) {
			flushAndExecute("size");
		} else if (appendix.equals("iterativePrintForEach")) {
			flushAndExecute("size");
		}
		
	}

	public void visit(
			ch.usi.inf.sape.algoprof.example.list.array.naive.generic.ArrayList<?> al) {
		try {
			dataFile.write(al.getData());
		} catch (IOException e) {
			e.printStackTrace();
		}	
		
	}

	public void visit(
			ch.usi.inf.sape.algoprof.example.list.array.naive.generic.Main m) {
		if (appendix.equals("make")) {
			flushAndExecute("0.5*size*size - 0.5*size");
		} else if (appendix.equals("iterativePrint")) {
			flushAndExecute("size");
		} else if (appendix.equals("recursivePrint")) {
			flushAndExecute("size");
		} else if (appendix.equals("iterativeIndex")) {
			flushAndExecute("size");
		} else if (appendix.equals("recursiveIndex")) {
			flushAndExecute("size");
		} else if (appendix.equals("iterativePrintForEach")) {
			flushAndExecute("size");
		}
		
	}

	public void visit(
			ch.usi.inf.sape.algoprof.example.list.array.naive.inheritance2.ArrayList al) {
		try {
			dataFile.write(al.getData());
		} catch (IOException e) {
			e.printStackTrace();
		}	
		
	}

	public void visit(PingValue ping) {
		// TODO Auto-generated method stub
		
	}

	public void visit(PongValue pong) {
		// TODO Auto-generated method stub
		
	}

	public void visit(
			ch.usi.inf.sape.algoprof.example.list.array.naive.inheritance2.Main m) {
		if (appendix.equals("make")) {
			flushAndExecute("0.5*size*size - 0.5*size");
		} else if (appendix.equals("iterativePrint")) {
			flushAndExecute("size");
		} else if (appendix.equals("recursivePrint")) {
			flushAndExecute("size");
		} else if (appendix.equals("iterativeIndex")) {
			flushAndExecute("size");
		} else if (appendix.equals("recursiveIndex")) {
			flushAndExecute("size");
		} else if (appendix.equals("iterativePrintForEach")) {
			flushAndExecute("size");
		}
		
	}

	public void visit(LinkedList ll) {
		try {
			dataFile.write(ll.getData());
		} catch (IOException e) {
			e.printStackTrace();
		}	
		
	}

	public void visit(ch.usi.inf.sape.algoprof.example.list.linked.basic.Main m) {
		if (appendix.equals("make")) {
			flushAndExecute("size");
		} else if (appendix.equals("iterativePrint")) {
			flushAndExecute("size");
		} else if (appendix.equals("recursivePrint")) {
			flushAndExecute("size");
		} else if (appendix.equals("nodeRecursivePrint")) {
			flushAndExecute("size");
		} else if (appendix.equals("iterativeIndex")) {
			flushAndExecute("size");
		} else if (appendix.equals("recursiveIndex")) {
			flushAndExecute("size");
		} else if (appendix.equals("nodeRecursiveIndex")) {
			flushAndExecute("size");
		} else if (appendix.equals("iterativeLength")) {
			flushAndExecute("size");
		} else if (appendix.equals("recursiveLength")) {
			flushAndExecute("size");
		} else if (appendix.equals("nodeRecursiveLength")) {
			flushAndExecute("size");
		} 
	}

	public void visit(ch.usi.inf.sape.algoprof.example.list.linked.basic.Node n) {
		// TODO Auto-generated method stub
		
	}

	public void visit(
			ch.usi.inf.sape.algoprof.example.list.linked.generic.LinkedList ll) {
		try {
			dataFile.write(ll.getData());
		} catch (IOException e) {
			e.printStackTrace();
		}	
	}

	public void visit(
			ch.usi.inf.sape.algoprof.example.list.linked.generic.Main m) {
		if (appendix.equals("make")) {
			flushAndExecute("size");
		} else if (appendix.equals("iterativePrint")) {
			flushAndExecute("size");
		} else if (appendix.equals("recursivePrint")) {
			flushAndExecute("size");
		} else if (appendix.equals("nodeRecursivePrint")) {
			flushAndExecute("size");
		} else if (appendix.equals("iterativeIndex")) {
			flushAndExecute("size");
		} else if (appendix.equals("recursiveIndex")) {
			flushAndExecute("size");
		} else if (appendix.equals("nodeRecursiveIndex")) {
			flushAndExecute("size");
		} else if (appendix.equals("iterativeLength")) {
			flushAndExecute("size");
		} else if (appendix.equals("recursiveLength")) {
			flushAndExecute("size");
		} else if (appendix.equals("nodeRecursiveLength")) {
			flushAndExecute("size");
		} 
		
	}

	public void visit(
			ch.usi.inf.sape.algoprof.example.list.linked.generic.Node<?> n) {
		// TODO Auto-generated method stub
		
	}

	public void visit(
			ch.usi.inf.sape.algoprof.example.list.linked.inheritance.LinkedList ll) {
		try {
			dataFile.write(ll.getData());
		} catch (IOException e) {
			e.printStackTrace();
		}	
		
	}

	public void visit(
			ch.usi.inf.sape.algoprof.example.list.linked.inheritance.Main m) {
		if (appendix.equals("make")) {
			flushAndExecute("size");
		} else if (appendix.equals("iterativePrint")) {
			flushAndExecute("size");
		} else if (appendix.equals("recursivePrint")) {
			flushAndExecute("size");
		} else if (appendix.equals("nodeRecursivePrint")) {
			flushAndExecute("size");
		} else if (appendix.equals("iterativeIndex")) {
			flushAndExecute("size");
		} else if (appendix.equals("recursiveIndex")) {
			flushAndExecute("size");
		} else if (appendix.equals("nodeRecursiveIndex")) {
			flushAndExecute("size");
		} else if (appendix.equals("iterativeLength")) {
			flushAndExecute("size");
		} else if (appendix.equals("recursiveLength")) {
			flushAndExecute("size");
		} else if (appendix.equals("nodeRecursiveLength")) {
			flushAndExecute("size");
		} 
		
	}

	public void visit(
			ch.usi.inf.sape.algoprof.example.list.linked.inheritance.Node n) {
		// TODO Auto-generated method stub
		
	}

	public void visit(PingNode ping) {
		// TODO Auto-generated method stub
		
	}

	public void visit(PongNode pong) {
		// TODO Auto-generated method stub
		
	}

	public void visit(List l) {
		try {
			dataFile.write(l.getData());
		} catch (IOException e) {
			e.printStackTrace();
		}
		
	}

	public void visit(ch.usi.inf.sape.algoprof.example.sort.insertion.Main m) {
		//sort measure construct constructReversed constructRandom
			//0.25*size*size size*size
			if (appendix.equals("sort")) {
				flushAndExecute("0.25*size*size");
			} else if (appendix.equals("construct")) {
				flushAndExecute("size");
			} else if (appendix.equals("constructReversed")) {
				flushAndExecute("size");
			} else if (appendix.equals("constructRandom")) {
				flushAndExecute("size");
			}
		
	}

	public void visit(ch.usi.inf.sape.algoprof.example.sort.insertion.Node n) {
		// TODO Auto-generated method
		
	}

	public void visit(BinaryTree bt) {
		try {
			dataFile.write(bt.getData());
		} catch (IOException e) {
			e.printStackTrace();
		}	
		
	}

	public void visit(
			ch.usi.inf.sape.algoprof.example.tree.array.perfect.binary.Main m) {
		if (appendix.equals("make")) {
			flushAndExecute("size");
		} else if (appendix.equals("recursivePrint")) {
			flushAndExecute("size");
		}
		
		
	}

	public void visit(
			ch.usi.inf.sape.algoprof.example.tree.linked.perfect.binary.basic.BinaryTree bt) {
		try {
			dataFile.write(bt.getData());
		} catch (IOException e) {
			e.printStackTrace();
		}	
		
	}

	public void visit(
			ch.usi.inf.sape.algoprof.example.tree.linked.perfect.binary.basic.Main m) {
		if (appendix.equals("make")) {
			flushAndExecute("size");
		} else if (appendix.equals("recursivePrint")) {
			flushAndExecute("2*size");
		}  else if (appendix.equals("nodeRecursivePrint")) {
			flushAndExecute("size");
		}
		
	}

	public void visit(
			ch.usi.inf.sape.algoprof.example.tree.linked.perfect.binary.basic.Node n) {
		// TODO Auto-generated method stub
		
	}

	public void visit(
			ch.usi.inf.sape.algoprof.example.tree.linked.perfect.binary.basic.bidi.BinaryTree bt) {
		try {
			dataFile.write(bt.getData());
		} catch (IOException e) {
			e.printStackTrace();
		}
		
	}

	public void visit(
			ch.usi.inf.sape.algoprof.example.tree.linked.perfect.binary.basic.bidi.Main m) {
		if (appendix.equals("make")) {
			flushAndExecute("size");
		} else if (appendix.equals("recursivePrint")) {
			flushAndExecute("2*size");
		}  else if (appendix.equals("nodeRecursivePrint")) {
			flushAndExecute("size");
		}
		
	}

	public void visit(
			ch.usi.inf.sape.algoprof.example.tree.linked.perfect.binary.basic.bidi.Node n) {
		// TODO Auto-generated method stub
		
	}

	public void visit(
			ch.usi.inf.sape.algoprof.example.tree.linked.perfect.nary.basic.Main m) {
		if (appendix.equals("make")) {
			flushAndExecute("size");
		} else if (appendix.equals("recursivePrint")) {
			flushAndExecute("size");
		}  else if (appendix.equals("nodeRecursivePrint")) {
			flushAndExecute("size");
		}
		
	}

	public void visit(
			ch.usi.inf.sape.algoprof.example.tree.linked.perfect.nary.basic.Node n) {
		// TODO Auto-generated method stub
		
	}

	public void visit(NaryTree nt) {
		try {
			dataFile.write(nt.getData());
		} catch (IOException e) {
			e.printStackTrace();
		}
		
	}

	public void visit(
			ch.usi.inf.sape.algoprof.example.tree.linked.perfect.nary.basic.bidi.Main m) {
		if (appendix.equals("make")) {
			flushAndExecute("size");
		} else if (appendix.equals("recursivePrint")) {
			flushAndExecute("size");
		}  else if (appendix.equals("nodeRecursivePrint")) {
			flushAndExecute("size");
		}
		
	}

	public void visit(
			ch.usi.inf.sape.algoprof.example.tree.linked.perfect.nary.basic.bidi.NaryTree nt) {
		try {
			dataFile.write(nt.getData());
		} catch (IOException e) {
			e.printStackTrace();
		}
		
	}

	public void visit(
			ch.usi.inf.sape.algoprof.example.tree.linked.perfect.nary.basic.bidi.Node n) {
		// TODO Auto-generated method stub
		
	}



	public void visit(InsertionSortAlgorithm is) {
		try {
			dataFile.write(is.getData());
			if (is.isLast()) {
				if (appendix.equals("fillArray")) {
					flushAndExecute("size");
				} else if (appendix.equals("sort")) {
					flushAndExecute("0.25*size*size");
				}
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
		
	}



	public void visit(ch.usi.inf.sape.algoprof.example.sort.insertion3.Main is) {
		try {
			dataFile.write(is.getData());
			if (is.isLast()) {
				if (appendix.equals("fillArray")) {
					flushAndExecute("size");
				} else if (appendix.equals("sort")) {
					flushAndExecute("0.25*size*size");
				} 
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
		
	}

	


}
