package ch.usi.inf.sape.algoprof.statistics;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

public class DotWriter {

	
	
	private List<String> nodeContainer;
	private List<String> subgraphContainer;
	private List<String> edgeContainer;
	
	public DotWriter() {
		nodeContainer = new ArrayList<String>();
		edgeContainer = new ArrayList<String>();
		subgraphContainer = new ArrayList<String>();
	}
	
	public String createNode(final String className, final String methodName, final int lineNumber, final boolean Loop) {
		StringBuffer sb = new StringBuffer();
		sb.append(className);
		sb.append('_');
		sb.append(methodName);
		sb.append('_');
		sb.append(lineNumber);
		sb.append(" [shape=rectangle, style=filled, fillcolor=\"white\", label = ");
		sb.append('\"');
		sb.append(className);
		sb.append('.');
		sb.append(methodName);
		sb.append(' ');
		sb.append(isLoop(Loop));
		sb.append(" at line ");
		sb.append(lineNumber);
		sb.append("\"]");
		nodeContainer.add(sb.toString());
		return className + "_" + methodName + "_" + lineNumber;
	}
	
	public String createNode(final String prefix, final String className, final String methodName, final int lineNumber, final boolean Loop) {
		StringBuffer sb = new StringBuffer();
		sb.append(prefix);
		sb.append('_');
		sb.append(className);
		sb.append('_');
		sb.append(methodName);
		sb.append('_');
		sb.append(lineNumber);
		sb.append(" [shape=rectangle, style=filled, fillcolor=\"white\", label = ");
		sb.append('\"');
		sb.append(className);
		sb.append('.');
		sb.append(methodName);
		sb.append(' ');
		sb.append(isLoop(Loop));
		sb.append(" at line ");
		sb.append(lineNumber);
		sb.append("\"]");
		nodeContainer.add(sb.toString());
		return prefix + "_" + className + "_" + methodName + "_" + lineNumber;
	}
	
	private String isLoop(final boolean isLoop) {
		if (isLoop) 
			return "loop";
		return "recursion";
	}
	
	public void createSubgraph (String algorithmName, String costFunction, int id) {
		StringBuffer sb = new StringBuffer();
		sb.append("subgraph cluster_" + id + " {");
		sb.append("\n  [label=\"");
		sb.append(algorithmName);
		sb.append("\\nsteps = ");
		sb.append(costFunction);
		sb.append("\", style=filled, color=\"lightgrey\"]");
		for (String node: nodeContainer) {
			sb.append("\n  ");
			sb.append(node);
		}
		sb.append("\n}");
		nodeContainer.clear();
		//System.out.println(sb.toString());
		subgraphContainer.add(sb.toString());
		
	}
	
	public void connect (String nodeName1, String nodeName2) {
		edgeContainer.add(nodeName1 + " -> " + nodeName2);
	}
	
	private int giveID() {
		return 0;
	}
	
	public void writeToDot(final String path) {
		try {
			BufferedWriter bw = new BufferedWriter(new FileWriter(new File(path + "/repetition-tree-" + giveID() + ".dot")));
			bw.write("digraph G {\n");
			for (String subgraph: subgraphContainer) {
				bw.write(subgraph + "\n");
			}
			for (String edge: edgeContainer) {
				bw.write(edge + "\n");
			}
			bw.write("}");
			bw.flush();
			bw.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	public List<String> getEdgeContainer() {
		return edgeContainer;
	}
	
	
}
