#!/bin/bash

# This small script runs the harness with the prototype of algorithmic profiler that recreates the output  
# of the corresponding PLDI paper called Algorithmic Profiling.
# The harness includes: 
# 1. 17 examples of different data structures including lists, graphs, arrays, and trees; 
# 2. Three differently designed insertion sort algorithms yielding identical results under AlgoProf;
# For details, please refer to the readme file and the source code.
# To compare results, refer to the http://www.inf.usi.ch/phd/zaparanuks/Algorithmic_Profiling.html
# Note: this is a shell script that is meant to run on UNIX based systems.  

# compile example sources 
# find . -name '*.java' > sources
# cat sources | xargs javac -cp src  -d bin

CLASSPATH='examples/bin'
VM_FLAGS='-Xmx2500m  -noverify'
XBOOT='-Xbootclasspath/p:DynamicCounter-0.9.jar'
AGENT='-javaagent:DynamicCounter-0.9.jar'
VM_ARG='2 2'

examples=(
'ch.usi.inf.sape.algoprof.example.array.multidimensional.Main:array-multidimensional' 
'ch.usi.inf.sape.algoprof.example.array.multidimensional.local.Main:array-multidimensional-local' 
'ch.usi.inf.sape.algoprof.example.graph.directed.adjacencymatrix.basic.Main:graph-directed-adjacencymatrix-basic' 
'ch.usi.inf.sape.algoprof.example.graph.directed.linked.basic.bidi.Main:graph-directed-linked-basic-bidi' 
'ch.usi.inf.sape.algoprof.example.graph.directed.linked.basic.Main:graph-directed-linked-basic' 
'ch.usi.inf.sape.algoprof.example.graph.undirected.linked.basic.Main:graph-undirected-linked-basic' 
'ch.usi.inf.sape.algoprof.example.list.array.doubling.basic.Main:list-array-doubling-basic' 
'ch.usi.inf.sape.algoprof.example.list.array.naive.basic.Main:list-array-naive-basic' 
'ch.usi.inf.sape.algoprof.example.list.array.naive.generic.Main:list-array-naive-generic' 
'ch.usi.inf.sape.algoprof.example.list.array.naive.inheritance2.Main:list-array-naive-inheritance2' 
'ch.usi.inf.sape.algoprof.example.list.linked.basic.Main:list-linked-basic' 
'ch.usi.inf.sape.algoprof.example.list.linked.generic.Main:list-linked-generic' 
'ch.usi.inf.sape.algoprof.example.list.linked.inheritance.Main:list-linked-inheritance' 
'ch.usi.inf.sape.algoprof.example.tree.array.perfect.binary.Main:tree-array-perfect-binary' 
'ch.usi.inf.sape.algoprof.example.tree.linked.perfect.binary.basic.bidi.Main:tree-linked-perfect-binary-basic-bidi' 
'ch.usi.inf.sape.algoprof.example.tree.linked.perfect.binary.basic.Main:tree-linked-perfect-binary-basic' 
'ch.usi.inf.sape.algoprof.example.tree.linked.perfect.nary.basic.bidi.Main:tree-linked-perfect-nary-basic-bidi' 
'ch.usi.inf.sape.algoprof.example.tree.linked.perfect.nary.basic.Main:tree-linked-perfect-nary-basic'
)

sorts=(
'ch.usi.inf.sape.algoprof.example.sort.insertion.Main:sort-insertion'
'ch.usi.inf.sape.algoprof.example.sort.insertion2.InsertionSortAlgorithm:sort-insertion2'
'ch.usi.inf.sape.algoprof.example.sort.insertion3.Main:sort-insertion3'
)

#run data structures

for example in "${!examples[@]}"; do 
	MAP=${examples[$example]}
	KEY=${MAP%%:*} 
	VALUE=${MAP#*:}
    java -cp $CLASSPATH $VM_FLAGS $XBOOT $AGENT="$VALUE" "$KEY" $VM_ARG; 
done

# run sorts

for sort in "${!sorts[@]}"; do 
	MAP=${sorts[$sort]}
	KEY=${MAP%%:*} 
	VALUE=${MAP#*:}
    java -cp $CLASSPATH $VM_FLAGS $XBOOT $AGENT="$VALUE" "$KEY"; 
done