package ch.unisi.inf.sp.cfg;

import static org.objectweb.asm.Opcodes.AALOAD;
import static org.objectweb.asm.Opcodes.AASTORE;
import static org.objectweb.asm.Opcodes.ANEWARRAY;
import static org.objectweb.asm.Opcodes.ARETURN;
import static org.objectweb.asm.Opcodes.ARRAYLENGTH;
import static org.objectweb.asm.Opcodes.ATHROW;
import static org.objectweb.asm.Opcodes.BALOAD;
import static org.objectweb.asm.Opcodes.BASTORE;
import static org.objectweb.asm.Opcodes.CALOAD;
import static org.objectweb.asm.Opcodes.CASTORE;
import static org.objectweb.asm.Opcodes.CHECKCAST;
import static org.objectweb.asm.Opcodes.DALOAD;
import static org.objectweb.asm.Opcodes.DASTORE;
import static org.objectweb.asm.Opcodes.DRETURN;
import static org.objectweb.asm.Opcodes.FALOAD;
import static org.objectweb.asm.Opcodes.FASTORE;
import static org.objectweb.asm.Opcodes.FRETURN;
import static org.objectweb.asm.Opcodes.GETFIELD;
import static org.objectweb.asm.Opcodes.GETSTATIC;
import static org.objectweb.asm.Opcodes.IALOAD;
import static org.objectweb.asm.Opcodes.IASTORE;
import static org.objectweb.asm.Opcodes.IDIV;
import static org.objectweb.asm.Opcodes.INSTANCEOF;
import static org.objectweb.asm.Opcodes.INVOKEINTERFACE;
import static org.objectweb.asm.Opcodes.INVOKESPECIAL;
import static org.objectweb.asm.Opcodes.INVOKESTATIC;
import static org.objectweb.asm.Opcodes.INVOKEVIRTUAL;
import static org.objectweb.asm.Opcodes.IREM;
import static org.objectweb.asm.Opcodes.IRETURN;
import static org.objectweb.asm.Opcodes.LALOAD;
import static org.objectweb.asm.Opcodes.LASTORE;
import static org.objectweb.asm.Opcodes.LDIV;
import static org.objectweb.asm.Opcodes.LREM;
import static org.objectweb.asm.Opcodes.LRETURN;
import static org.objectweb.asm.Opcodes.MONITORENTER;
import static org.objectweb.asm.Opcodes.MONITOREXIT;
import static org.objectweb.asm.Opcodes.MULTIANEWARRAY;
import static org.objectweb.asm.Opcodes.NEW;
import static org.objectweb.asm.Opcodes.NEWARRAY;
import static org.objectweb.asm.Opcodes.PUTFIELD;
import static org.objectweb.asm.Opcodes.PUTSTATIC;
import static org.objectweb.asm.Opcodes.RETURN;
import static org.objectweb.asm.Opcodes.SALOAD;
import static org.objectweb.asm.Opcodes.SASTORE;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;

public class InstructionData {
	public static final HashSet<Integer> RETURN_OPCODES = new HashSet<Integer>(
			Arrays.asList(RETURN, IRETURN, LRETURN, FRETURN, DRETURN, ARETURN));

	public static final HashSet<Integer> PEI_OPCODES = new HashSet<Integer>(
			Arrays.asList(AALOAD, AASTORE, ANEWARRAY, ARETURN, ARRAYLENGTH,
					ATHROW, BALOAD, BASTORE, CALOAD, CASTORE, CHECKCAST,
					DALOAD, DASTORE, DRETURN, FALOAD, FASTORE, FRETURN,
					GETFIELD, GETSTATIC, IALOAD, IASTORE, IDIV, INSTANCEOF,
					INVOKEINTERFACE, INVOKESPECIAL, INVOKESTATIC,
					INVOKEVIRTUAL, IREM, IRETURN, LALOAD, LASTORE, LDIV, LREM,
					LRETURN, MONITORENTER, MONITOREXIT, MULTIANEWARRAY, NEW,
					NEWARRAY, PUTFIELD, PUTSTATIC, RETURN, SALOAD, SASTORE));
	
	public static final HashSet<Integer> JMP_OPCODES = new HashSet<Integer> ();

	private final String name;
	public final List<Edge> edges;
	public final int i;
	public final int opcode;
	public final int instructionNumber;


	public InstructionData(List<Edge> edges, String name, int i, int opcode, int instructionNumber) {
		this.edges = edges;
		this.name = name;
		this.i = i;
		this.opcode = opcode;
		this.instructionNumber = instructionNumber;

	}

	public InstructionData(int i, int opcode, int instructionNumber) {
		this(new ArrayList<Edge>(), null, i, opcode, instructionNumber);
	}

	public InstructionData(String name, int i, int opcode, int instructionNumber) {
		this(new ArrayList<Edge>(), name, i, opcode, instructionNumber);
	}

	public String getName() {
		return (name == null) ? null : name.replaceAll("<", "\\\\<")
				.replaceAll(">", "\\\\>");
	}

	public boolean isReturn() {
		return RETURN_OPCODES.contains(opcode);
	}

	public boolean isPEI() {
		return PEI_OPCODES.contains(opcode);
	}
}
