package ch.unisi.inf.sp.type;

import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.objectweb.asm.Opcodes;

import ch.unisi.inf.sp.tree.Snode;
import ch.unisi.inf.sp.tree.Snode.Kind;
import ch.usi.inf.sape.graph.example.RecursionTreater;

import usi.instrumentation.CCT;
import usi.instrumentation.stack.Node;
import usi.instrumentation.stack.Tree;
import usi.instrumentation.tree.TreeIR.VertexState;


public class CallRoutineDumper {
	
	private Map<String, Node<String>> database;
	final String filename; 
	
	public CallRoutineDumper(final String filename) {
		this.filename = filename;
		database = new HashMap<String, Node<String>>();
	}
	
	private String makeName (final String methodName, final String className, final String desc) {
		return className + "." + methodName + desc;
	}

	public void constructTree (ClassHierarchy hierarchy) {
		
		Tree<String> tree = new Tree<String>();
		Node<String> entryNode = new Snode<String>("ENTRY", Kind.NORMAL, 0);
		tree.setRootElement(entryNode);
		
		for (Type type: hierarchy.getTypes()) {
			
			if (type instanceof ClassType) {
				final ClassType classType = (ClassType)type;
				
				for (Subroutine subroutine: classType.getSubroutines()) {
				
					if (!subroutine.isAbstract()) {
						
						Node<String> node;
						String name = makeName(subroutine.getName(), subroutine.getDeclaringClassName(), subroutine.getDescriptor());
						if (database.containsKey(name)) {
							//System.out.println(subroutine.getName()+subroutine.getDeclaringClassName());
							node = database.get(name);
						} else {
							Kind kind;
							int numberOfLoops = 0;
							if (subroutine.isMethod()) {
								kind = Kind.NORMAL;
								numberOfLoops = ((Method)subroutine).numberOgLoops;
							} else {
								kind = Kind.LOOP;
							}
							node = new Snode<String>(name, kind, numberOfLoops);
							database.put(name, node);
						}
						
						for (CallSite site: subroutine.getCallSites()) {
							
							/*
							 * first we do this for declared class
							 * after for possible class targets
							 */
							
							Node<String> child;
						/*
							if (database.containsKey(site.getTargetMethodName()+site.getDeclaredTargetClassName())) {
							child = database.get(site.getTargetMethodName()+site.getDeclaredTargetClassName());
							} else {
								Kind kind;
								//bogous check, better would be to resolve the target class
								
								if (!site.getTargetMethodName().contains("-")) {
									
									kind = Kind.NORMAL;
								} else {
									kind = Kind.LOOP;
								}
								
								child = new Snode<String>(site.getDeclaredTargetClassName().replaceAll("/", "") + "" + site.getTargetMethodName(), kind);
								database.put(site.getTargetMethodName()+site.getDeclaredTargetClassName(), child);
							}
							child.father = node;
							node.addChild(child);
						*/	 
							/*
							 * for possible classes
							 */
							for (ClassType cType : site.getPossibleTargetClasses()) {
								child = null;
								name = makeName(site.getTargetMethodName(), cType.getInternalName(), site.getTargetMethodDescriptor());
								
								if (database.containsKey(name)) {
									child = database.get(name);
								} else {
									Kind kind;
									//bogous check, better would be to resolve the target class
									int numberOfLoops = 0;
									if (!site.getTargetMethodName().contains("-")) {
										numberOfLoops = ((Method)cType.getSubroutine(site.getTargetMethodName(), site.getTargetMethodDescriptor())).numberOgLoops;
										kind = Kind.NORMAL;
									} else {
										kind = Kind.LOOP;
									}
									
									child = new Snode<String>(name, kind, numberOfLoops);
									
									//System.out.println(name);
									database.put(name, child);
								}
								if ((node.children!=null && !node.children.contains(child)) ||
										node.children==null) {
									child.father = node;
									node.addChild(child);
									((Snode)node).setEdge(site.getOpcode(), child);									
								}
								//System.out.println("adding child to " + node.data + " and child is " + child.data);
								
							}
							
						}
					}
					
				}
			}
		}
		
		for (Node<String> node : database.values()) {
			if (node.father == null) {
				((Snode)entryNode).setEdge(Opcodes.INVOKESTATIC, node);
				entryNode.addChild(node);
				node.father = entryNode;
			}
		}
		database.put("ENTRY", entryNode);
		hierarchy = null;
		
		try {
			defineRecursion(filename);
		} catch (CloneNotSupportedException e1) {
			e1.printStackTrace();
		} catch (IOException e1) {
			e1.printStackTrace();
		}
		
		/*
		 * at this moment we can detect recursion, delete backedge and mark recursive node as "recursive"
		 */
		/*DFS(tree);
		for (Node<String> node: level.keySet()) {
			if (node.getNumberOfChildren()>0) {
				ArrayList<Node<String>> toRemove = new ArrayList<Node<String>>();
				for (int i=0; i<node.getNumberOfChildren(); i++) {
					Node<String> child = node.getChildren().get(i); 
					if (level.get(child)<level.get(node) || 
							child.equals(node)) {
						//node has to be recursive
						toRemove.add(child);
						((Snode) child).setKind(Kind.RECURSIVE);
						//System.out.println("happened " + node.data + " makes recursive " + child.data);
					}
				}
				if (!toRemove.isEmpty()) {
					//node.getChildren().removeAll(toRemove);					
				}
			}
		}
		
		try {
			dumpDot(tree, filename+".dot");
			dumpStatistics(tree, filename+".txt");
		} catch (IOException e) {
			System.out.println("wrong path...");
			e.printStackTrace();
		}
		*/
	}
	
	private HashMap<Node<String>, Integer> level = new HashMap<Node<String>, Integer>();
	
	private void defineRecursion(final String filename) throws CloneNotSupportedException, IOException {
		new RecursionTreater(filename).check(database.values());
	}
	
	
	private void DFS (Tree<String> tree) {
		Collection<Node<String>> treeList = database.values();
		int vertexCount = treeList.size();
		HashMap<Node<String>, VertexState> state = new HashMap<Node<String>, VertexState>();
          for (Node<String> node: treeList)
                state.put(node, VertexState.White);
        runDFS(tree.getRootElement(), state, 0);
       

    }


    private void runDFS (Node<String> node, HashMap<Node<String>, VertexState> state, int id) {
          state.put(node, VertexState.Gray);
          
          if (node.getNumberOfChildren()!=0) {
        	  for (Node<String> child: node.getChildren()) {
        		  if (state.get(child) == VertexState.White) {
        			  runDFS(child, state, id);	  
        		  }  
        	  }
          }
          state.put(node, VertexState.Black);
          level.put(node, id);
          //System.out.println(node.data + " " + id);
          id++;
    }


	private void dumpDot(final Tree<String> tree, final String fileName) throws IOException {
		final PrintWriter pw = new PrintWriter(new FileWriter(fileName));
		pw.println("digraph types {");
		//pw.println("  rankdir=\"BT\"");
		Collection<Node<String>> treeList = database.values();
		
		for (Node<String> node : treeList) {
			Snode type = (Snode) node;
			String style;
			if (type.getKind() == Kind.LOOP) {
				style = "dashed";
			} else if (type.getKind() == Kind.RECURSIVE) {
				style = "bold";
			} else {
				style = "solid";
			}
			pw.println(""+ node.hashCode() +" [shape = record"+", label = "+ node.data + ", style="+style+"]");
			if (node.getNumberOfChildren()>0) {
				for (Node<String> child: node.getChildren()) {
					pw.println(""+node.hashCode()+" -> "+child.hashCode());
				}
				
			}
		}
		pw.flush();
		pw.close();
	}
	
	private void dumpStatistics(final Tree<String> tree, final String fileName) throws IOException {
		final PrintWriter pw = new PrintWriter(new FileWriter(fileName));
		Collection<Node<String>> treeList = database.values();
		int loops = 0;
		int normal = 0;
		int recursive = 0;
		for (Node<String> node : treeList) {
			Snode type = (Snode) node;
			if (type.getKind() == Kind.LOOP) {
				loops++;
			} else if (type.getKind() == Kind.RECURSIVE) {
				recursive++;
			} else {
				normal++;
			}
		
		}
		pw.println("number of normal nodes is " + (normal-1));
		pw.println("number of loop nodes is " + loops);
		pw.println("number of recursive nodes is " + recursive);
		pw.flush();
		pw.close();
		System.out.println("number of normal nodes is " + (normal-1));
		System.out.println("number of loop nodes is " + loops);
		System.out.println("number of recursive nodes is " + recursive);
	}
}

