package ch.unisi.inf.sp.type;

import java.io.IOException;
import java.util.Collection;
import java.util.List;

import org.objectweb.asm.Opcodes;
import org.objectweb.asm.tree.AbstractInsnNode;
import org.objectweb.asm.tree.ClassNode;
import org.objectweb.asm.tree.InsnList;
import org.objectweb.asm.tree.MethodNode;

import usi.types.instrumentation.LoopFinder;


public final class CallSiteCounter implements ClassAnalyzer {

	private int dynamicCallSites;
	private int interfaceCallSites;
	private int specialCallSites;
	private int staticCallSites;
	private int virtualCallSites;
	private int loopCallSites;
	
	
	public void analyze(final String location, final ClassNode classNode) {
		final List<MethodNode> methods = (List<MethodNode>)classNode.methods;
		for (final MethodNode method : methods) {
			
			LoopFinder loopFinder = new LoopFinder();
			List<ch.unisi.inf.sp.type.LoopConstruct> loops = loopFinder.findLoops(classNode, method);
			loopCallSites = loopCallSites + loops.size();
			
			final InsnList instructions = method.instructions;
			for (int i=0; i<instructions.size(); i++) {
				final AbstractInsnNode ins = instructions.get(i);
				switch (ins.getOpcode()) {
				case Opcodes.INVOKEDYNAMIC:
					dynamicCallSites++;
					break;
				case Opcodes.INVOKEINTERFACE:
					interfaceCallSites++;
					break;
				case Opcodes.INVOKESPECIAL:
					specialCallSites++;
					break;
				case Opcodes.INVOKESTATIC:
					staticCallSites++;
					break;
				case Opcodes.INVOKEVIRTUAL:
					virtualCallSites++;
					break;
				}
			}
		}
	}
	
	

	
	public int getDynamicCallSiteCount() {
		return dynamicCallSites;
	}
	
	public int getInterfaceCallSiteCount() {
		return interfaceCallSites;
	}
	
	public int getVirtualCallSiteCount() {
		return virtualCallSites;
	}
	
	public int getSpecialCallSiteCount() {
		return specialCallSites;
	}
	
	public int getStaticCallSiteCount() {
		return staticCallSites;
	}
	
	
	

	public static void main(final String[] args) throws IOException {
		final ArchiveScanner scanner = new ArchiveScanner();
		final CallSiteCounter callCounter = new CallSiteCounter();
		scanner.addAnalyzer(callCounter);
		for (int i=0; i<args.length; i++) {
			scanner.scan(args[i]);
		}
		System.out.println("Static count of call sites: ");
		System.out.println(" dynamic:\t"+callCounter.getDynamicCallSiteCount());
		System.out.println(" interface:\t"+callCounter.getInterfaceCallSiteCount());
		System.out.println(" virtual:\t"+callCounter.getVirtualCallSiteCount());
		System.out.println(" special:\t"+callCounter.getSpecialCallSiteCount());
		System.out.println(" static:\t"+callCounter.getStaticCallSiteCount());
		final int poly = callCounter.getInterfaceCallSiteCount()+callCounter.getVirtualCallSiteCount();
		final int all = poly+callCounter.getSpecialCallSiteCount()+callCounter.getStaticCallSiteCount();
		System.out.println("potentially polymorphic call sites: "+poly+" ("+(poly*100/all)+"%)");
		
		System.out.println(" loops:\t"+callCounter.loopCallSites);
	}
}
