package ch.unisi.inf.sp.type;

import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;


public final class ClassHierarchyDumper {

	public void dump(final ClassHierarchy nameSpace) {
		for (final Type type : nameSpace.getTypes()) {
			System.out.println(type);
		}
	}
	
	public void dumpDot(final ClassHierarchy nameSpace) throws IOException {
		final PrintWriter pw = new PrintWriter(new FileWriter("types.dot"));
		pw.println("digraph types {");
		pw.println("  rankdir=\"BT\"");
		for (final Type type : nameSpace.getTypes()) {
			if (type instanceof ClassType) {
				final ClassType classType = (ClassType)type;
				final String lineStyle;
				final String shape;
				if (classType.isResolved()) {
					if (classType.isInterface()) {
						lineStyle = "dotted";
						shape = "record";
					} else if (classType.isAbstract()) {
						lineStyle = "dashed";
						shape = "record";
					} else if (classType.isFinal()) {
						lineStyle = "bold";
						shape = "record";
					} else {
						lineStyle = "solid";
						shape = "record";						
					}
				} else {
					lineStyle = "dotted";
					shape = "ellipse";
				}
				if (classType.getSuperClass()!=null) {
					pw.println("  "+n(type)+" -> "+n(classType.getSuperClass())+" [arrowhead=empty]");
				}
				for (final Type interfa : classType.getInterfaces()) {
					pw.println("  "+n(type)+" -> "+n(interfa)+" [style=dashed,arrowhead=empty]");				
				}
				pw.println("  "+n(type)+" [shape="+shape+","+l(type)+", style="+lineStyle+"]");
			}
		}
		pw.println("}");
		pw.close();
	}
	
	private String l(Type type) {
		if (type instanceof ClassType) {
			final ClassType classType = (ClassType)type;
			final StringBuilder sb = new StringBuilder("label=\"");
			if (classType.isResolved()) {
				sb.append("{");
				//sb.append(classType.getArchive()+"\\n");
			}
			if (type.getInternalName().contains("/")) {
				final int split = type.getInternalName().lastIndexOf("/");
				sb.append(type.getInternalName().substring(0, split).replaceAll("/", ".")+"\\n"+type.getInternalName().substring(split+1));
			} else {
				sb.append(type.getInternalName());
			}
			/* include fields?
			if (type instanceof ClassType && !((ClassType)type).isInterface()) {
				sb.append("|");
				sb.append("fields");
			}
			*/
			/* include methods?
			if (classType.isResolved()) {
				sb.append("|");
			}
			for (final Method method : classType.getMethods()) {
				if (!method.isPrivate() && !method.isStatic()) {
					sb.append(method.getName().replace('<', '[').replace('>', ']')+method.getDescriptor()+"\\n");
				}
			}
			*/
			if (classType.isResolved()) {
				sb.append("}");
			}
			return sb.append("\"").toString();
		} else {
			return "label=\""+type.getInternalName()+"\"";
		}
	}

	private String n(Type type) {
		return type.getInternalName().replaceAll("/", "_").replaceAll("\\$", "__");
	}
	
}
