package ch.unisi.inf.sp.type;

import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;

import org.objectweb.asm.Opcodes;


public final class ClassHierarchyWithCallEdgeDumper {

	public void dumpDot(final ClassHierarchy hierarchy, final String fileName, final boolean showDeclaredCallEdges, final boolean showPossibleCallEdges) throws IOException {
		final PrintWriter pw = new PrintWriter(new FileWriter(fileName));
		pw.println("digraph types {");
		pw.println("  rankdir=\"BT\"");
		for (final Type type : hierarchy.getTypes()) {
			if (type instanceof ClassType) {
				final ClassType classType = (ClassType)type;
				final String lineStyle;
				final String shape;
				if (classType.isResolved()) {
					if (classType.isInterface()) {
						lineStyle = "dotted";
						shape = "record";
					} else if (classType.isAbstract()) {
						lineStyle = "dashed";
						shape = "record";
					} else if (classType.isFinal()) {
						lineStyle = "bold";
						shape = "record";
					} else {
						lineStyle = "solid";
						shape = "record";						
					}
				} else {
					lineStyle = "dotted";
					shape = "ellipse";
				}
				if (classType.getSuperClass()!=null) {
					pw.println("  "+n(type)+" -> "+n(classType.getSuperClass())+" [arrowhead=empty]");
				}
				for (final Type interfa : classType.getInterfaces()) {
					pw.println("  "+n(type)+" -> "+n(interfa)+" [style=dashed,arrowhead=empty]");				
				}
				pw.println("  "+n(type)+" [shape="+shape+","+l(type)+", style="+lineStyle+"]");
				for (final Subroutine subroutine : classType.getSubroutines()) {
					for (final CallSite site : subroutine.getCallSites()) {
						final String s;
						switch (site.getOpcode()) {
						case Opcodes.INVOKEINTERFACE:
							s = "dotted";
							break;
						case Opcodes.INVOKEVIRTUAL:
							s = "dashed";
							break;
						case Opcodes.INVOKESPECIAL:
							s = "solid";
							break;
						case Opcodes.INVOKESTATIC:
							s = "bold";
							break;
						default:
							s = "solid";	
						}
						try {
							if (hierarchy.getOrCreateClass(site.getDeclaredTargetClassName()).isResolved()) {
								final String l = site.getTargetMethodName().replace('<', '[').replace('>', ']')+site.getTargetMethodDescriptor();
								pw.println("  "+n(type)+" -> "+n(site.getDeclaredTargetClassName())+" [style="+(showDeclaredCallEdges?s:"invis")+", color=blue,fontcolor=blue,label=\""+l+"\\n"+site.getPossibleTargetClasses().size()+"\"]");
								for (final ClassType targetType : site.getPossibleTargetClasses()) {
									pw.println("  "+n(type)+" -> "+n(targetType)+" [style="+(showPossibleCallEdges?s:"invis")+",color=red,fontcolor=red,label=\""+l+"\"]");
								}
							}
						} catch (final TypeInconsistencyException ex) {
							System.err.println(ex);
						}
					}
				}
			}
		}
		pw.println("}");
		pw.close();
	}

	private String l(Type type) {
		if (type instanceof ClassType) {
			final ClassType classType = (ClassType)type;
			final StringBuilder sb = new StringBuilder("label=\"");
			if (classType.isResolved()) {
				sb.append("{");
				//sb.append(classType.getArchive()+"\\n");
			}
			if (type.getInternalName().contains("/")) {
				final int split = type.getInternalName().lastIndexOf("/");
				sb.append(type.getInternalName().substring(0, split).replaceAll("/", ".")+"\\n"+type.getInternalName().substring(split+1));
			} else {
				sb.append(type.getInternalName());
			}
			if (classType.isResolved()) {
				sb.append("|");
				for (final Subroutine subroutine : classType.getSubroutines()) {
					final String mod = 
						(subroutine.isPublic()?"public ":(subroutine.isProtected()?"protected ":(subroutine.isPrivate()?"private ":"")))+
						(subroutine.isAbstract()?"abstract ":(subroutine.isFinal()?"final ":""))+
						(subroutine.isStatic()?"static ":"");
					sb.append(mod+subroutine.getName().replace('<', '[').replace('>', ']')+subroutine.getDescriptor()+"\\n");
				}
			}
			if (classType.isResolved()) {
				sb.append("}");
			}
			return sb.append("\"").toString();
		} else {
			return "label=\""+type.getInternalName()+"\"";
		}
	}

	private String n(Type type) {
		return n(type.getInternalName());
	}

	private String n(String typeName) {
		return typeName.replaceAll("/", "_").replaceAll("\\$", "__");
	}
	
}
