package ch.unisi.inf.sp.type;

import java.io.IOException;
import java.util.Enumeration;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

import org.objectweb.asm.ClassReader;
import org.objectweb.asm.tree.ClassNode;
import org.objectweb.asm.tree.MethodNode;


/**
 * @deprecated use ClassHierarchyBuilder
 */
public class TypeExtractor {

	private ClassHierarchy nameSpace;
	
	
	public TypeExtractor(final ClassHierarchy nameSpace) {
		this.nameSpace = nameSpace;
	}
	
	public void scan(final String archiveName) throws IOException {
		final ZipFile zip = new ZipFile(archiveName);
		final Enumeration<? extends ZipEntry> entries = zip.entries();
		while (entries.hasMoreElements()) {
			final ZipEntry entry = entries.nextElement();
			if (entry.getName().toLowerCase().endsWith(".class"))
				scanClass(zip, entry);
		}
	}

	private void scanClass(final ZipFile zip, final ZipEntry entry) throws IOException {
		final ClassReader cr = new ClassReader(zip.getInputStream(entry));
		// create an empty ClassNode (in-memory representation of a class)
		final ClassNode clazz = new ClassNode();
		// have the ClassReader read the class file and populate the ClassNode with the corresponding information
		cr.accept(clazz, 0);
		// extract type info
		try {
			final ClassType classType = nameSpace.getOrCreateClass(clazz.name);
			if (classType.isResolved()) {
				System.err.println("WARNING: Class "+classType.getInternalName()+" defined multiple times");
				return;
			}
			classType.setLocation(zip.getName());
			// modifiers
			classType.setModifiers(clazz.access);
			// super class
			final ClassType superClassType = nameSpace.getOrCreateClass(clazz.superName);
			classType.setSuperClass(superClassType);
			// interfaces
			final List<String> interfaces = (List<String>)clazz.interfaces;
			for (final String interfaceName : interfaces) {
				final ClassType interfaceType = nameSpace.getOrCreateClass(interfaceName);
				classType.addInterface(interfaceType);
			}
			// methods
			final List<MethodNode> methodNodes = (List<MethodNode>)clazz.methods;
			for (final MethodNode methodNode : methodNodes) {
				final Method method = new Method(clazz.name, methodNode.name, methodNode.desc, methodNode.access);
				classType.addSubroutine(method);
			}
			classType.setResolved();
		} catch (final TypeInconsistencyException ex) {
			System.err.println(ex);
		}
	}
	
	public static void main(final String[] args) throws IOException {
		final ClassHierarchy nameSpace = new ClassHierarchy();
		final TypeExtractor e = new TypeExtractor(nameSpace);
		for (int i=0; i<args.length; i++) {
			e.scan(args[i]);
		}
		final ClassHierarchyDumper d = new ClassHierarchyDumper();
		d.dump(nameSpace);
		d.dumpDot(nameSpace);
		final StatisticsDumper s = new StatisticsDumper();
		s.dump(nameSpace);
	}
	
}
