package ch.usi.inf.sape.graph;

import java.io.Serializable;


/**
 * Edges are immutable.
 * Don't create edges yourself, call Graph.createEdge() instead!
 * 
 * @author Matthias.Hauswirth@usi.ch
 */
public class Edge implements Cloneable, Serializable {

	private Node source;
	private Node target;
	
	
	public Edge(final Node source, final Node target) {
		this.source = source;
		this.target = target;
	}
	
	/**
	 * Create a clone of this Edge,
	 * but the clone does not refer to any source or target!
	 * 
	 * The FlowGraph, when it is being cloned, will establish
	 * source and target during its cloning process.
	 * 
	 * You need to override this method if you implement subclasses of Edge 
	 * and want to properly support cloning a FlowGraph.
	 */
	public Edge clone() throws CloneNotSupportedException {
		final Edge clone = (Edge)super.clone();
		clone.source = null;
		clone.target = null;
		return clone;
	}
	
	public Node getSource() {
		return source;
	}
	
	public Node getTarget() {
		return target;
	}
	
	//--- update (don't call these methods directly, go through Graph)
	//NOTE: HechtUllmanReducibilityTest calls these methods directly (and maybe should)
	public void setSource(final Node source) {
		this.source = source;
	}
	
	public void setTarget(final Node target) {
		this.target = target;
	}
	
}
