package ch.usi.inf.sape.graph;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;


/**
 * A hack to allow us to traverse an existing FlowGraph
 * while assuming that some of its edges have been removed.
 * 
 * It is a hack because algoritms HAVE to explicitly go through this EdgeFilter. 
 * If they don't (or forget to do so in some place), they will break!
 * 
 * In the future, we should refactor FlowGraph so we can create a FilteredFlowGraph,
 * which would be based on the original FlowGraph (same Edge and Node instances)
 * but would represent a subgraph (some Edge or Nodes excluded).
 * This would require us to provide getInEdges/getOutEdges in the FlowGraph class
 * (instead of in the Node class), so the FilteredFlowGraph could filter out "removed" edges.
 * 
 * @author Matthias.Hauswirth@usi.ch
 */
public class EdgeFilter {

	final HashSet<Edge> removed;
	
	
	public EdgeFilter() {
		removed = new HashSet<Edge>();
	}
	
	public void addRemovedEdge(final Edge edge) {
		removed.add(edge);
	}
	
	/**
	 * @return the number of removed edges
	 */
	public int size() {
		return removed.size();
	}
	
	public boolean passes(final Edge edge) {
		return !removed.contains(edge);
	}
	
	public ArrayList<Edge> filter(final Collection<Edge> original) {
		final ArrayList<Edge> filtered = new ArrayList<Edge>(original.size());
		for (final Edge e : original) {
			if (passes(e)) {
				filtered.add(e);
			}
		}
		return filtered;
	}
	
}
