package ch.usi.inf.sape.graph;


/**
 * Factory used by some graph algorithms to operate on graphs, node, or edges.
 * Specific graph implementations need to provide their own Factory implementation.
 * If they don't and a specific feature is needed by a specific graph algorithm,
 * this Factory base class will throw an UnsupportedOperationException.
 * 
 * @author Matthias.Hauswirth@usi.ch
 */
public class Factory {

	/**
	 * Merge the data (not the edges!) of the other node into the merged node.
	 * Used by the HechtUllman reducibility algorithm.
	 * @param merged parent node, and also the result of the merge
	 * @param other child node, from which the data is merged into the parent
	 */
	public void mergeNodeData(Node merged, Node other) {
		throw new UnsupportedOperationException();
	}

}
