package ch.usi.inf.sape.graph;

import ch.usi.inf.sape.graph.algorithms.HechtUllmanReducibilityTest;
import ch.usi.inf.sape.graph.example.ExampleFactory;
import ch.usi.inf.sape.graph.example.ExampleGraphBuilder;
import ch.usi.inf.sape.graph.example.ExampleNode;
import junit.framework.TestCase;


public class HechtUllmanReducibilityTestTest extends TestCase {

	public void testOneNodeNoEdges() throws CloneNotSupportedException {
		final FlowGraph g = new FlowGraph();
		final Node n = new ExampleNode("1");
		g.addNode(n);
		g.makeEntry(n);
		assertEquals(true, HechtUllmanReducibilityTest.isReducible(g, new ExampleFactory()));
	}
	
	public void testOneNodeNoEdgesWithTwoUnreachableNodes() throws CloneNotSupportedException {
		final FlowGraph g = new FlowGraph();
		final Node n = new ExampleNode("1");
		final Node u1 = new ExampleNode("u1");
		final Node u2 = new ExampleNode("u2");
		g.addNode(n);
		g.addNode(u1);
		g.addNode(u2);
		g.addEdge(new Edge(u1, u2));
		g.makeEntry(n);
		assertEquals(true, HechtUllmanReducibilityTest.isReducible(g, new ExampleFactory()));
	}
	
	public void testOneNodeWithSelfLoop() throws CloneNotSupportedException {
		final FlowGraph g = new FlowGraph();
		final Node n = new ExampleNode("1");
		g.addNode(n);
		final Edge e = new Edge(n, n);
		g.addEdge(e);
		g.makeEntry(n);
		assertEquals(true, HechtUllmanReducibilityTest.isReducible(g, new ExampleFactory()));
	}
	
	public void testTwoNodesWithEdge() throws CloneNotSupportedException {
		final FlowGraph g = new FlowGraph();
		final Node n1 = new ExampleNode("1");
		final Node n2 = new ExampleNode("2");
		g.addNode(n1);
		g.addNode(n2);
		final Edge e = new Edge(n1, n2);
		g.addEdge(e);
		g.makeEntry(n1);
		assertEquals(true, HechtUllmanReducibilityTest.isReducible(g, new ExampleFactory()));
	}
	
	//This test with a multigraph (parallel edges) breaks the algorithm unless it first eliminates parallel edges
	public void testTwoNodesWithParallelEdges() throws CloneNotSupportedException {
		final FlowGraph g = new FlowGraph();
		final Node n1 = new ExampleNode("1");
		final Node n2 = new ExampleNode("2");
		g.addNode(n1);
		g.addNode(n2);
		final Edge e1 = new Edge(n1, n2);
		final Edge e2 = new Edge(n1, n2);
		g.addEdge(e1);
		g.addEdge(e2);
		g.makeEntry(n1);
		assertEquals(true, HechtUllmanReducibilityTest.isReducible(g, new ExampleFactory()));
	}
	
	public void testTest() throws CloneNotSupportedException {
		assertEquals(true, HechtUllmanReducibilityTest.isReducible(ExampleGraphBuilder.makeTest(), new ExampleFactory()));
	}
	
	public void testExample() throws CloneNotSupportedException {
		assertEquals(false, HechtUllmanReducibilityTest.isReducible(ExampleGraphBuilder.makeExample(), new ExampleFactory()));
	}
	
	public void testHechtUllmanFigures() throws CloneNotSupportedException {
		assertEquals(false, HechtUllmanReducibilityTest.isReducible(ExampleGraphBuilder.makeHechtUllman_2_1_a(), new ExampleFactory()));
		assertEquals(false, HechtUllmanReducibilityTest.isReducible(ExampleGraphBuilder.makeHechtUllman_2_3_a(), new ExampleFactory()));
		assertEquals(true, HechtUllmanReducibilityTest.isReducible(ExampleGraphBuilder.makeHechtUllman_4_1(), new ExampleFactory()));
		assertEquals(true, HechtUllmanReducibilityTest.isReducible(ExampleGraphBuilder.makeHechtUllman_4_2(), new ExampleFactory()));
		assertEquals(false, HechtUllmanReducibilityTest.isReducible(ExampleGraphBuilder.makeHechtUllman_4_3(), new ExampleFactory()));
		assertEquals(false, HechtUllmanReducibilityTest.isReducible(ExampleGraphBuilder.makeHechtUllman_6_1_a(), new ExampleFactory()));
		assertEquals(false, HechtUllmanReducibilityTest.isReducible(ExampleGraphBuilder.makeHechtUllman_6_2(), new ExampleFactory()));
		assertEquals(false, HechtUllmanReducibilityTest.isReducible(ExampleGraphBuilder.makeHechtUllman_6_3_b(), new ExampleFactory()));
		assertEquals(false, HechtUllmanReducibilityTest.isReducible(ExampleGraphBuilder.makeHechtUllman_6_3_c(), new ExampleFactory()));
		assertEquals(false, HechtUllmanReducibilityTest.isReducible(ExampleGraphBuilder.makeHechtUllman_6_4(), new ExampleFactory()));
		assertEquals(true, HechtUllmanReducibilityTest.isReducible(ExampleGraphBuilder.makeHechtUllman_6_5(), new ExampleFactory()));
	}
	
	public void testHechtTarjanFigure() throws CloneNotSupportedException {
		assertEquals(true, HechtUllmanReducibilityTest.isReducible(ExampleGraphBuilder.makeTarjan_1(), new ExampleFactory()));
	}
	
}
