package ch.usi.inf.sape.graph;

import junit.framework.TestCase;
import ch.usi.inf.sape.graph.algorithms.LoopForestFinder;
import ch.usi.inf.sape.graph.example.ExampleNode;
import ch.usi.inf.sape.graph.loop.LoopForest;


public class LoopForestFinderTest extends TestCase {
	
	public void testEmpty() {
		final FlowGraph g = new FlowGraph();
		final LoopForestFinder finder = new LoopForestFinder();
		final LoopForest f = finder.find(g);
		assertEquals(0, f.getTreeRoots().size());
	}
	
	public void testSingleton() {
		final FlowGraph g = new FlowGraph();
		final Node n = new ExampleNode("1");
		g.addNode(n);
		g.makeEntry(n);
		final LoopForestFinder finder = new LoopForestFinder();
		final LoopForest f = finder.find(g);
		assertEquals(0, f.getTreeRoots().size());
	}
	
	public void testSingletonWithLoop() {
		final FlowGraph g = new FlowGraph();
		final Node n = new ExampleNode("1");
		g.addNode(n);
		g.addEdge(new Edge(n, n));
		g.makeEntry(n);
		final LoopForestFinder finder = new LoopForestFinder();
		final LoopForest f = finder.find(g);
		assertEquals(1, f.getTreeRoots().size());
		assertEquals(1, f.getTreeRoots().get(0).getHeaders().size());
		assertEquals(1, f.getTreeRoots().get(0).getBody().size());
		assertEquals(n, f.getTreeRoots().get(0).getHeaders().get(0));
		assertEquals(n, f.getTreeRoots().get(0).getBody().get(0));
	}
	
	public void testChain() {
		//System.out.println("testChain");
		final FlowGraph g = new FlowGraph();
		final Node n1 = new ExampleNode("1");
		final Node n2 = new ExampleNode("2");
		final Node n3 = new ExampleNode("3");
		g.addNode(n1);
		g.addNode(n2);
		g.addNode(n3);
		g.addEdge(new Edge(n1, n2));
		g.addEdge(new Edge(n2, n3));
		g.makeEntry(n1);
		final LoopForestFinder finder = new LoopForestFinder();
		final LoopForest f = finder.find(g);
		assertEquals(0, f.getTreeRoots().size());
	}
	
	public void testCycle() {
		//System.out.println("testCycle");
		final FlowGraph g = new FlowGraph();
		final Node n1 = new ExampleNode("1");
		final Node n2 = new ExampleNode("2");
		final Node n3 = new ExampleNode("3");
		g.addNode(n1);
		g.addNode(n2);
		g.addNode(n3);
		g.addEdge(new Edge(n1, n2));
		g.addEdge(new Edge(n2, n3));
		g.addEdge(new Edge(n3, n1));
		g.makeEntry(n1);
		final LoopForestFinder finder = new LoopForestFinder();
		final LoopForest f = finder.find(g);
		assertEquals(1, f.getTreeRoots().size());
		assertEquals(1, f.getTreeRoots().get(0).getHeaders().size());
		assertEquals(3, f.getTreeRoots().get(0).getBody().size());
		assertEquals(n1, f.getTreeRoots().get(0).getHeaders().get(0));
		//TODO more asserts
	}
	
	public void testCycleCycle() {
		//System.out.println("LoopForestFinderTest.testCycleCycle");
		final FlowGraph g = new FlowGraph();
		final Node n1 = new ExampleNode("1");
		final Node n2 = new ExampleNode("2");
		final Node n3 = new ExampleNode("3");
		g.addNode(n1);
		g.addNode(n2);
		g.addNode(n3);
		g.addEdge(new Edge(n1, n2));
		g.addEdge(new Edge(n2, n1));
		g.addEdge(new Edge(n2, n3));
		g.addEdge(new Edge(n3, n2));
		g.makeEntry(n1);
		final LoopForestFinder finder = new LoopForestFinder();
		final LoopForest f = finder.find(g);
		assertEquals(1, f.getTreeRoots().size());
		//TODO more assertions
	}

	public void testCycleEdgeCycle() {
		//System.out.println("LoopForestFinderTest.testCycleEdgeCycle");
		final FlowGraph g = new FlowGraph();
		final Node n1 = new ExampleNode("1");
		final Node n2 = new ExampleNode("2");
		final Node n3 = new ExampleNode("3");
		final Node n4 = new ExampleNode("4");
		g.addNode(n1);
		g.addNode(n2);
		g.addNode(n3);
		g.addNode(n4);
		g.addEdge(new Edge(n1, n2));
		g.addEdge(new Edge(n2, n1));
		g.addEdge(new Edge(n2, n3));
		g.addEdge(new Edge(n3, n4));
		g.addEdge(new Edge(n4, n3));
		g.makeEntry(n1);
		final LoopForestFinder finder = new LoopForestFinder();
		final LoopForest f = finder.find(g);
		assertEquals(2, f.getTreeRoots().size());
		//TODO more assertions
	}

	public void testNestedCycles() {
		//System.out.println("LoopForestFinderTest.testNestedCycles");
		final FlowGraph g = new FlowGraph();
		final Node n1 = new ExampleNode("1");
		final Node n2 = new ExampleNode("2");
		final Node n3 = new ExampleNode("3");
		final Node n4 = new ExampleNode("4");
		g.addNode(n1);
		g.addNode(n2);
		g.addNode(n3);
		g.addNode(n4);
		g.addEdge(new Edge(n1, n2));
		g.addEdge(new Edge(n2, n3));
		g.addEdge(new Edge(n3, n2));
		g.addEdge(new Edge(n3, n4));
		g.addEdge(new Edge(n4, n1));
		g.makeEntry(n1);
		final LoopForestFinder finder = new LoopForestFinder();
		final LoopForest f = finder.find(g);
		assertEquals(1, f.getTreeRoots().size());
		//TODO more assertions
	}

}
