package ch.usi.inf.sape.graph;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;

import ch.usi.inf.sape.graph.algorithms.LoopForestFinder;
import ch.usi.inf.sape.graph.algorithms.LoopForestFinder2;
import ch.usi.inf.sape.graph.dot.DefaultEdgeAttributeStringProvider;
import ch.usi.inf.sape.graph.dot.DotWriter;
import ch.usi.inf.sape.graph.dot.LoopNestingNodeAttributeStringProvider;
import ch.usi.inf.sape.graph.dot.LoopNestingNodeAttributeStringProvider2;
import ch.usi.inf.sape.graph.example.ExampleGraphBuilder;
import ch.usi.inf.sape.graph.example.ExampleGraphGenerator;
import ch.usi.inf.sape.graph.loop.LoopForest;
import ch.usi.inf.sape.graph.loop.LoopForest2;


/**
 * @author Matthias.Hauswirth@usi.ch
 */
public class Main {

	public static void main(final String[] args) throws IOException, CloneNotSupportedException {
		process("Getter", ExampleGraphBuilder.makeGetterExample());
		process("GetterWithNonPolymorphicEntry", ExampleGraphBuilder.makeGetterWithNonPolymorphicEntryExample());
		process("Visitor", ExampleGraphBuilder.makeVisitorExample());
		process("VisitorWithNonPolymorphicEntry", ExampleGraphBuilder.makeVisitorWithNonPolymorphicEntryExample());
		process("EntryTwoSelfLoops", ExampleGraphBuilder.makeEntryToTwoSelfLoops());
		process("Tarjan_1", ExampleGraphBuilder.makeTarjan_1());
		process("Example", ExampleGraphBuilder.makeExample());
		process("HechtUllman_2_1_a", ExampleGraphBuilder.makeHechtUllman_2_1_a());
		process("HechtUllman_2_3_a", ExampleGraphBuilder.makeHechtUllman_2_3_a());
		process("HechtUllman_4_1", ExampleGraphBuilder.makeHechtUllman_4_1());
		process("HechtUllman_4_2", ExampleGraphBuilder.makeHechtUllman_4_2());
		process("HechtUllman_4_3", ExampleGraphBuilder.makeHechtUllman_4_3());
		process("HechtUllman_6_1_a", ExampleGraphBuilder.makeHechtUllman_6_1_a());
		process("HechtUllman_6_2", ExampleGraphBuilder.makeHechtUllman_6_2());
		process("HechtUllman_6_3_b", ExampleGraphBuilder.makeHechtUllman_6_3_b());
		process("HechtUllman_6_3_c", ExampleGraphBuilder.makeHechtUllman_6_3_c());
		process("HechtUllman_6_4", ExampleGraphBuilder.makeHechtUllman_6_4());
		process("HechtUllman_6_5", ExampleGraphBuilder.makeHechtUllman_6_5());
		
		process("GenFull_1", ExampleGraphGenerator.createFullyConnectedGraph(1));
		process("GenFull_6", ExampleGraphGenerator.createFullyConnectedGraph(6));
		//process("GenFull_300", ExampleGraphGenerator.createFullyConnectedGraph(300));
		process("GenChain_30", ExampleGraphGenerator.createChain(30));
		process("GenDoubleChain_30", ExampleGraphGenerator.createDoubleChain(30));
		process("GenFan_30", ExampleGraphGenerator.createFan(30));
		process("GenFan2_5", ExampleGraphGenerator.createFan2(5));
		process("GenFan2_30", ExampleGraphGenerator.createFan2(30));
		//process("GenFan2_20000", ExampleGraphGenerator.createFan2(20000));
		process("GenLoopNestChain_30", ExampleGraphGenerator.createLoopNestChain(30));
		//process("GenLoopNestChain_3000", ExampleGraphGenerator.createLoopNestChain(3000));

		//process("GenFull_1000", ExampleGraphGenerator.createFullyConnectedGraph(1000));
	}
	
	/**
	 * Write flow graph and its loop forest into dot files.
	 * 
	 * @param name
	 * @param g
	 * @throws IOException
	 */
	private static void process(final String name, final FlowGraph g) throws IOException {
		g.check(false);
		g.checkReachability(false);
		
		long startTime, endTime;

		startTime = System.nanoTime();
		final LoopForest2 f2 = new LoopForestFinder2().find(g);
		endTime = System.nanoTime();
		System.out.println("LoopForestFinder2().find() took "+((endTime-startTime)/1000000)+" ms");
		
		g.check(false);
		g.checkReachability(false);
		
		startTime = System.nanoTime();
		final LoopForest f = new LoopForestFinder().find(g);
		endTime = System.nanoTime();
		System.out.println("LoopForestFinder().find() took "+((endTime-startTime)/1000000)+" ms");

		
		PrintWriter pw;
		final File dir = new File("examples");
		dir.mkdir();
		pw = new PrintWriter(new FileWriter(new File(dir, name+".dot")));
		pw.print(DotWriter.renderToDot(g, new LoopNestingNodeAttributeStringProvider(f), new DefaultEdgeAttributeStringProvider()));
		pw.close();
		pw = new PrintWriter(new FileWriter(new File(dir, name+"-forest.dot")));
		pw.print(DotWriter.renderLoopForest(g, f));
		pw.close();		
		pw = new PrintWriter(new FileWriter(new File(dir, name+"-forest2.dot")));
		pw.print(DotWriter.renderLoopForest2(g, f2, new LoopNestingNodeAttributeStringProvider2(f2)));
		pw.close();		
	}
	
}
