package ch.usi.inf.sape.graph;

import java.util.Set;

import junit.framework.TestCase;
import ch.usi.inf.sape.graph.algorithms.StronglyConnectedComponent;
import ch.usi.inf.sape.graph.algorithms.TarjanStronglyConnectedComponents;
import ch.usi.inf.sape.graph.example.ExampleNode;


public final class TarjanStronglyConnectedComponentsTest extends TestCase {

	public void testEmpty() {
		final FlowGraph g = new FlowGraph();
		new TarjanStronglyConnectedComponents().getStronglyConnectedComponents(g);
	}
	
	public void testSingleton() {
		final FlowGraph g = new FlowGraph();
		final Node n = new ExampleNode("1");
		g.addNode(n);
		g.makeEntry(n);
		final Set<StronglyConnectedComponent> sccs = new TarjanStronglyConnectedComponents().getStronglyConnectedComponents(g);
		assertEquals(1, sccs.size());
		assertEquals(1, sccs.iterator().next().getNodes().size());
		assertEquals(n, sccs.iterator().next().getNodes().get(0));
	}
	
	public void testSingletonWithLoop() {
		final FlowGraph g = new FlowGraph();
		final Node n = new ExampleNode("1");
		g.addNode(n);
		g.addEdge(new Edge(n, n));
		g.makeEntry(n);
		final Set<StronglyConnectedComponent> sccs = new TarjanStronglyConnectedComponents().getStronglyConnectedComponents(g);
		assertEquals(1, sccs.size());
		assertEquals(1, sccs.iterator().next().getNodes().size());
		assertEquals(n, sccs.iterator().next().getNodes().get(0));
	}
	
	public void testChain() {
		//System.out.println("testChain");
		final FlowGraph g = new FlowGraph();
		final Node n1 = new ExampleNode("1");
		final Node n2 = new ExampleNode("2");
		final Node n3 = new ExampleNode("3");
		g.addNode(n1);
		g.addNode(n2);
		g.addNode(n3);
		g.addEdge(new Edge(n1, n2));
		g.addEdge(new Edge(n2, n3));
		g.makeEntry(n1);
		final Set<StronglyConnectedComponent> sccs = new TarjanStronglyConnectedComponents().getStronglyConnectedComponents(g);
		assertEquals(3, sccs.size());
		//TODO write more assertions
	}
	
	public void testCycle() {
		//System.out.println("testCycle");
		final FlowGraph g = new FlowGraph();
		final Node n1 = new ExampleNode("1");
		final Node n2 = new ExampleNode("2");
		final Node n3 = new ExampleNode("3");
		g.addNode(n1);
		g.addNode(n2);
		g.addNode(n3);
		g.addEdge(new Edge(n1, n2));
		g.addEdge(new Edge(n2, n3));
		g.addEdge(new Edge(n3, n1));
		g.makeEntry(n1);
		final Set<StronglyConnectedComponent> sccs = new TarjanStronglyConnectedComponents().getStronglyConnectedComponents(g);
		assertEquals(1, sccs.size());
		assertEquals(3, sccs.iterator().next().getNodes().size());
		assertEquals(true, sccs.iterator().next().getNodes().contains(n1));
		assertEquals(true, sccs.iterator().next().getNodes().contains(n2));
		assertEquals(true, sccs.iterator().next().getNodes().contains(n3));
	}

	public void testCycleEdgeCycle() {
		//System.out.println("testCycleEdgeCycle");
		final FlowGraph g = new FlowGraph();
		final Node n1 = new ExampleNode("1");
		final Node n2 = new ExampleNode("2");
		final Node n3 = new ExampleNode("3");
		final Node n4 = new ExampleNode("4");
		g.addNode(n1);
		g.addNode(n2);
		g.addNode(n3);
		g.addNode(n4);
		g.addEdge(new Edge(n1, n2));
		g.addEdge(new Edge(n2, n1));
		g.addEdge(new Edge(n2, n3));
		g.addEdge(new Edge(n3, n4));
		g.addEdge(new Edge(n4, n3));
		g.makeEntry(n1);
		final Set<StronglyConnectedComponent> sccs = new TarjanStronglyConnectedComponents().getStronglyConnectedComponents(g);
		assertEquals(2, sccs.size());
		assertEquals(2, sccs.iterator().next().getNodes().size());
		//TODO more assertions
	}
	
	public void testCycleCycle() {
		//System.out.println("testCycleCycle");
		final FlowGraph g = new FlowGraph();
		final Node n1 = new ExampleNode("1");
		final Node n2 = new ExampleNode("2");
		final Node n3 = new ExampleNode("3");
		g.addNode(n1);
		g.addNode(n2);
		g.addNode(n3);
		g.addEdge(new Edge(n1, n2));
		g.addEdge(new Edge(n2, n1));
		g.addEdge(new Edge(n2, n3));
		g.addEdge(new Edge(n3, n2));
		g.makeEntry(n1);
		final Set<StronglyConnectedComponent> sccs = new TarjanStronglyConnectedComponents().getStronglyConnectedComponents(g);
		assertEquals(1, sccs.size());
		assertEquals(3, sccs.iterator().next().getNodes().size());
		assertEquals(true, sccs.iterator().next().getNodes().contains(n1));
		assertEquals(true, sccs.iterator().next().getNodes().contains(n2));
		assertEquals(true, sccs.iterator().next().getNodes().contains(n3));
	}
	
}
