package ch.usi.inf.sape.graph.algorithms;

import java.util.ArrayList;

import ch.usi.inf.sape.graph.Edge;
import ch.usi.inf.sape.graph.FlowGraph;


/**
 * Collapse parallel edges in a multigraph.
 * 
 * @author Matthias.Hauswirth@usi.ch
 */
public class ParallelEdgeCollapser {
	
	/**
	 * This method converts a multigraph into a graph (that still may contain self loops).
	 * 
	 * A graph can't have parallel edges (it would be a multigraph in that case).
	 * However, control flow graphs with exception edges may indeed have parallel edges
	 * (especially edges that go to the exit node).
	 * 
	 * @param g FlowGraph
	 */
	public static void eliminateParallelEdges(final FlowGraph g) {
		// clone, because we will modify the graph during traversal
		final ArrayList<Edge> edgesToRemove = new ArrayList<Edge>();
		
		for (int e1=0; e1<g.getEdges().size(); e1++) {
			final Edge edge1 = g.getEdges().get(e1);
			for (int e2=e1+1; e2<g.getEdges().size(); e2++) {
				final Edge edge2 = g.getEdges().get(e2);
				if (edge1.getSource()==edge2.getSource() && edge1.getTarget()==edge2.getTarget()) {
					edgesToRemove.add(edge2);
				}
			}
		}
		
		for (final Edge edge : edgesToRemove) {
			g.removeEdge(edge);
		}
	}

}
