package ch.usi.inf.sape.graph.example;

import ch.usi.inf.sape.graph.Edge;
import ch.usi.inf.sape.graph.FlowGraph;
import ch.usi.inf.sape.graph.Node;


/**
 * Create example graphs from papers (e.g. Hecht/Ullman, or Tarjan's reducibility papers).
 * Used for testing.
 * 
 * @see Hecht'72: http://doi.acm.org/10.1145/800152.804919
 * @see Tarjan'73: http://doi.acm.org/10.1145/800125.804040
 * 
 * @author Matthias.Hauswirth@usi.ch
 */
public final class ExampleGraphBuilder {

	public static FlowGraph makeVisitorExample() {
		final FlowGraph g = new FlowGraph();
		final Node mainCalc = new ExampleNode("Main.calc");
		final Node addAccept = new ExampleNode("Add.accept");
		final Node subtractAccept = new ExampleNode("Subtract.accept");
		final Node multiplyAccept = new ExampleNode("Multiply.accept");
		final Node literalAccept = new ExampleNode("Literal.accept");
		final Node evalVisitorGetResult = new ExampleNode("EvalVisitor.getResult");
		final Node evalVisitorVisitAdd = new ExampleNode("EvalVisitor.visitAdd");
		final Node evalVisitorVisitSubtract = new ExampleNode("EvalVisitor.visitSubtract");
		final Node evalVisitorVisitMultiply = new ExampleNode("EvalVisitor.visitMultiply");
		final Node evalVisitorVisitLiteral = new ExampleNode("EvalVisitor.visitLiteral");
		final Node binOpGetA = new ExampleNode("BinOp.getA");
		final Node binOpGetB = new ExampleNode("BinOp.getB");
		final Node literalGetValue = new ExampleNode("Literal.getValue");
		g.addNode(mainCalc);
		g.addNode(addAccept);
		g.addNode(subtractAccept);
		g.addNode(multiplyAccept);
		g.addNode(literalAccept);
		g.addNode(binOpGetA);
		g.addNode(binOpGetB);
		g.addNode(literalGetValue);
		g.addNode(evalVisitorGetResult);
		g.addNode(evalVisitorVisitAdd);
		g.addNode(evalVisitorVisitSubtract);
		g.addNode(evalVisitorVisitMultiply);
		g.addNode(evalVisitorVisitLiteral);
		g.addEdge(new Edge(mainCalc, addAccept));
		g.addEdge(new Edge(mainCalc, subtractAccept));
		g.addEdge(new Edge(mainCalc, multiplyAccept));
		g.addEdge(new Edge(mainCalc, literalAccept));
		g.addEdge(new Edge(mainCalc, evalVisitorGetResult));
		g.addEdge(new Edge(addAccept, evalVisitorVisitAdd));
		g.addEdge(new Edge(evalVisitorVisitAdd, addAccept));
		g.addEdge(new Edge(evalVisitorVisitAdd, subtractAccept));
		g.addEdge(new Edge(evalVisitorVisitAdd, multiplyAccept));
		g.addEdge(new Edge(evalVisitorVisitAdd, literalAccept));
		g.addEdge(new Edge(evalVisitorVisitAdd, binOpGetA));
		g.addEdge(new Edge(evalVisitorVisitAdd, binOpGetB));
		g.addEdge(new Edge(subtractAccept, evalVisitorVisitSubtract));
		g.addEdge(new Edge(evalVisitorVisitSubtract, addAccept));
		g.addEdge(new Edge(evalVisitorVisitSubtract, subtractAccept));
		g.addEdge(new Edge(evalVisitorVisitSubtract, multiplyAccept));
		g.addEdge(new Edge(evalVisitorVisitSubtract, literalAccept));
		g.addEdge(new Edge(evalVisitorVisitSubtract, binOpGetA));
		g.addEdge(new Edge(evalVisitorVisitSubtract, binOpGetB));
		g.addEdge(new Edge(multiplyAccept, evalVisitorVisitMultiply));
		g.addEdge(new Edge(evalVisitorVisitMultiply, addAccept));
		g.addEdge(new Edge(evalVisitorVisitMultiply, subtractAccept));
		g.addEdge(new Edge(evalVisitorVisitMultiply, multiplyAccept));
		g.addEdge(new Edge(evalVisitorVisitMultiply, literalAccept));
		g.addEdge(new Edge(evalVisitorVisitMultiply, binOpGetA));
		g.addEdge(new Edge(evalVisitorVisitMultiply, binOpGetB));
		g.addEdge(new Edge(literalAccept, evalVisitorVisitLiteral));
		g.addEdge(new Edge(evalVisitorVisitLiteral, literalGetValue));
		g.makeEntry(mainCalc);
		return g;
	}
	

	public static FlowGraph makeVisitorWithNonPolymorphicEntryExample() {
		final FlowGraph g = new FlowGraph();
		final Node mainCalc = new ExampleNode("Main.calc");
		final Node addAccept = new ExampleNode("Add.accept");
		final Node subtractAccept = new ExampleNode("Subtract.accept");
		final Node multiplyAccept = new ExampleNode("Multiply.accept");
		final Node literalAccept = new ExampleNode("Literal.accept");
		final Node evalVisitorGetResult = new ExampleNode("EvalVisitor.getResult");
		final Node evalVisitorVisitAdd = new ExampleNode("EvalVisitor.visitAdd");
		final Node evalVisitorVisitSubtract = new ExampleNode("EvalVisitor.visitSubtract");
		final Node evalVisitorVisitMultiply = new ExampleNode("EvalVisitor.visitMultiply");
		final Node evalVisitorVisitLiteral = new ExampleNode("EvalVisitor.visitLiteral");
		final Node binOpGetA = new ExampleNode("BinOp.getA");
		final Node binOpGetB = new ExampleNode("BinOp.getB");
		final Node literalGetValue = new ExampleNode("Literal.getValue");
		g.addNode(mainCalc);
		g.addNode(addAccept);
		g.addNode(subtractAccept);
		g.addNode(multiplyAccept);
		g.addNode(literalAccept);
		g.addNode(binOpGetA);
		g.addNode(binOpGetB);
		g.addNode(literalGetValue);
		g.addNode(evalVisitorGetResult);
		g.addNode(evalVisitorVisitAdd);
		g.addNode(evalVisitorVisitSubtract);
		g.addNode(evalVisitorVisitMultiply);
		g.addNode(evalVisitorVisitLiteral);
		g.addEdge(new Edge(mainCalc, addAccept));
		g.addEdge(new Edge(mainCalc, evalVisitorGetResult));
		g.addEdge(new Edge(addAccept, evalVisitorVisitAdd));
		g.addEdge(new Edge(evalVisitorVisitAdd, addAccept));
		g.addEdge(new Edge(evalVisitorVisitAdd, subtractAccept));
		g.addEdge(new Edge(evalVisitorVisitAdd, multiplyAccept));
		g.addEdge(new Edge(evalVisitorVisitAdd, literalAccept));
		g.addEdge(new Edge(evalVisitorVisitAdd, binOpGetA));
		g.addEdge(new Edge(evalVisitorVisitAdd, binOpGetB));
		g.addEdge(new Edge(subtractAccept, evalVisitorVisitSubtract));
		g.addEdge(new Edge(evalVisitorVisitSubtract, addAccept));
		g.addEdge(new Edge(evalVisitorVisitSubtract, subtractAccept));
		g.addEdge(new Edge(evalVisitorVisitSubtract, multiplyAccept));
		g.addEdge(new Edge(evalVisitorVisitSubtract, literalAccept));
		g.addEdge(new Edge(evalVisitorVisitSubtract, binOpGetA));
		g.addEdge(new Edge(evalVisitorVisitSubtract, binOpGetB));
		g.addEdge(new Edge(multiplyAccept, evalVisitorVisitMultiply));
		g.addEdge(new Edge(evalVisitorVisitMultiply, addAccept));
		g.addEdge(new Edge(evalVisitorVisitMultiply, subtractAccept));
		g.addEdge(new Edge(evalVisitorVisitMultiply, multiplyAccept));
		g.addEdge(new Edge(evalVisitorVisitMultiply, literalAccept));
		g.addEdge(new Edge(evalVisitorVisitMultiply, binOpGetA));
		g.addEdge(new Edge(evalVisitorVisitMultiply, binOpGetB));
		g.addEdge(new Edge(literalAccept, evalVisitorVisitLiteral));
		g.addEdge(new Edge(evalVisitorVisitLiteral, literalGetValue));
		g.makeEntry(mainCalc);
		return g;
	}
	
	public static FlowGraph makeGetterExample() {
		final FlowGraph g = new FlowGraph();
		final Node mainCalc = new ExampleNode("Main.calc");
		final Node addEval = new ExampleNode("Add.eval");
		final Node subtractEval = new ExampleNode("Subtract.eval");
		final Node multiplyEval = new ExampleNode("Multiply.eval");
		final Node literalEval = new ExampleNode("Literal.eval");
		final Node binOpGetA = new ExampleNode("BinOp.getA");
		final Node binOpGetB = new ExampleNode("BinOp.getB");
		g.addNode(mainCalc);
		g.addNode(addEval);
		g.addNode(subtractEval);
		g.addNode(multiplyEval);
		g.addNode(literalEval);
		g.addNode(binOpGetA);
		g.addNode(binOpGetB);
		g.addEdge(new Edge(mainCalc, addEval));
		g.addEdge(new Edge(mainCalc, subtractEval));
		g.addEdge(new Edge(mainCalc, multiplyEval));
		g.addEdge(new Edge(mainCalc, literalEval));
		g.addEdge(new Edge(addEval, addEval));
		g.addEdge(new Edge(addEval, subtractEval));
		g.addEdge(new Edge(addEval, multiplyEval));
		g.addEdge(new Edge(addEval, literalEval));
		g.addEdge(new Edge(addEval, binOpGetA));
		g.addEdge(new Edge(addEval, binOpGetB));
		g.addEdge(new Edge(subtractEval, addEval));
		g.addEdge(new Edge(subtractEval, subtractEval));
		g.addEdge(new Edge(subtractEval, multiplyEval));
		g.addEdge(new Edge(subtractEval, literalEval));
		g.addEdge(new Edge(subtractEval, binOpGetA));
		g.addEdge(new Edge(subtractEval, binOpGetB));
		g.addEdge(new Edge(multiplyEval, addEval));
		g.addEdge(new Edge(multiplyEval, subtractEval));
		g.addEdge(new Edge(multiplyEval, multiplyEval));
		g.addEdge(new Edge(multiplyEval, literalEval));
		g.addEdge(new Edge(multiplyEval, binOpGetA));
		g.addEdge(new Edge(multiplyEval, binOpGetB));
		g.makeEntry(mainCalc);
		return g;
	}
	
	public static FlowGraph makeGetterWithNonPolymorphicEntryExample() {
		final FlowGraph g = new FlowGraph();
		final Node mainCalc = new ExampleNode("Main.calc");
		final Node addEval = new ExampleNode("Add.eval");
		final Node subtractEval = new ExampleNode("Subtract.eval");
		final Node multiplyEval = new ExampleNode("Multiply.eval");
		final Node literalEval = new ExampleNode("Literal.eval");
		final Node binOpGetA = new ExampleNode("BinOp.getA");
		final Node binOpGetB = new ExampleNode("BinOp.getB");
		g.addNode(mainCalc);
		g.addNode(addEval);
		g.addNode(subtractEval);
		g.addNode(multiplyEval);
		g.addNode(literalEval);
		g.addNode(binOpGetA);
		g.addNode(binOpGetB);
		g.addEdge(new Edge(mainCalc, multiplyEval));
		g.addEdge(new Edge(addEval, addEval));
		g.addEdge(new Edge(addEval, subtractEval));
		g.addEdge(new Edge(addEval, multiplyEval));
		g.addEdge(new Edge(addEval, literalEval));
		g.addEdge(new Edge(addEval, binOpGetA));
		g.addEdge(new Edge(addEval, binOpGetB));
		g.addEdge(new Edge(subtractEval, addEval));
		g.addEdge(new Edge(subtractEval, subtractEval));
		g.addEdge(new Edge(subtractEval, multiplyEval));
		g.addEdge(new Edge(subtractEval, literalEval));
		g.addEdge(new Edge(subtractEval, binOpGetA));
		g.addEdge(new Edge(subtractEval, binOpGetB));
		g.addEdge(new Edge(multiplyEval, addEval));
		g.addEdge(new Edge(multiplyEval, subtractEval));
		g.addEdge(new Edge(multiplyEval, multiplyEval));
		g.addEdge(new Edge(multiplyEval, literalEval));
		g.addEdge(new Edge(multiplyEval, binOpGetA));
		g.addEdge(new Edge(multiplyEval, binOpGetB));
		g.makeEntry(mainCalc);
		return g;
	}

	public static FlowGraph makeEntryToTwoSelfLoops() {
		final FlowGraph g = new FlowGraph();
		final Node n1 = new ExampleNode("1");
		final Node n2 = new ExampleNode("2");
		final Node n3 = new ExampleNode("3");
		g.addNode(n1);
		g.addNode(n2);
		g.addNode(n3);
		g.addEdge(new Edge(n1, n2));
		g.addEdge(new Edge(n2, n2));
		g.addEdge(new Edge(n1, n3));
		g.addEdge(new Edge(n3, n3));
		g.makeEntry(n1);
		return g;
	}
	
	public static FlowGraph makeTest() {
		final FlowGraph g = new FlowGraph();
		final Node e = new ExampleNode("e");
		final Node x = new ExampleNode("x");
		final Node a = new ExampleNode("a");
		final Node b = new ExampleNode("b");
		g.addNode(e);
		g.addNode(a);
		g.addNode(b);
		g.addNode(x);
		g.addEdge(new Edge(e, a));
		g.addEdge(new Edge(a, b));
		g.addEdge(new Edge(b, x));
		g.addEdge(new Edge(b, a));
		g.makeEntry(e);
		return g;
	}

	public static FlowGraph makeExample() {
		final FlowGraph g = new FlowGraph();
		final Node entry = new ExampleNode("entry");
		final Node u = new ExampleNode("u");
		final Node v = new ExampleNode("v");
		final Node w = new ExampleNode("w");
		final Node x = new ExampleNode("x");
		final Node exit = new ExampleNode("exit");
		g.addNode(entry);
		g.addNode(u);
		g.addNode(v);
		g.addNode(w);
		g.addNode(x);
		g.addNode(exit);
		g.addEdge(new Edge(entry, u));
		g.addEdge(new Edge(entry, v));
		g.addEdge(new Edge(u, w));
		g.addEdge(new Edge(w, u));
		g.addEdge(new Edge(v, x));
		g.addEdge(new Edge(x, v));
		g.addEdge(new Edge(w, x));
		g.addEdge(new Edge(x, w));
		g.addEdge(new Edge(x, exit));
		g.addEdge(new Edge(w, exit));
		g.makeEntry(entry);
		return g;
	}
	
	public static FlowGraph makeHechtUllman_2_1_a() {
		final FlowGraph g = new FlowGraph();
		final Node one = new ExampleNode("1");
		final Node two = new ExampleNode("2");
		final Node three = new ExampleNode("3");
		final Node four = new ExampleNode("4");
		g.addNode(one);
		g.addNode(two);
		g.addNode(three);
		g.addNode(four);
		g.addEdge(new Edge(one, one));
		g.addEdge(new Edge(one, two));
		g.addEdge(new Edge(two, three));
		g.addEdge(new Edge(two, four));
		g.addEdge(new Edge(three, four));
		g.addEdge(new Edge(four, one));
		g.addEdge(new Edge(four, three));
		g.makeEntry(one);
		return g;
	}

	public static FlowGraph makeHechtUllman_2_3_a() {
		final FlowGraph g = new FlowGraph();
		final Node n1 = new ExampleNode("n1");
		final Node n2 = new ExampleNode("n2");
		final Node n3 = new ExampleNode("n3");
		final Node n4 = new ExampleNode("n4");
		final Node n5 = new ExampleNode("n5");
		final Node n6 = new ExampleNode("n6");
		final Node n7 = new ExampleNode("n7");
		final Node n8 = new ExampleNode("n8");
		g.addNode(n1);
		g.addNode(n2);
		g.addNode(n3);
		g.addNode(n4);
		g.addNode(n5);
		g.addNode(n6);
		g.addNode(n7);
		g.addNode(n8);
		g.addEdge(new Edge(n1, n2));
		g.addEdge(new Edge(n2, n3));
		g.addEdge(new Edge(n3, n4));
		g.addEdge(new Edge(n3, n5));
		g.addEdge(new Edge(n4, n6));
		g.addEdge(new Edge(n5, n6));
		g.addEdge(new Edge(n5, n7));
		g.addEdge(new Edge(n6, n5));
		g.addEdge(new Edge(n6, n8));
		g.addEdge(new Edge(n8, n1));
		g.makeEntry(n1);
		return g;
	}
	
	public static FlowGraph makeHechtUllman_4_1() {
		final FlowGraph g = new FlowGraph();
		final Node n1 = new ExampleNode("1");
		final Node n2 = new ExampleNode("2");
		final Node n3 = new ExampleNode("3");
		final Node n4 = new ExampleNode("4");
		final Node n5 = new ExampleNode("5");
		final Node n6 = new ExampleNode("6");
		g.addNode(n1);
		g.addNode(n2);
		g.addNode(n3);
		g.addNode(n4);
		g.addNode(n5);
		g.addNode(n6);
		// forward
		g.addEdge(new Edge(n1, n2));
		g.addEdge(new Edge(n2, n3));
		g.addEdge(new Edge(n3, n4));
		g.addEdge(new Edge(n4, n5));
		g.addEdge(new Edge(n5, n6));
		// back
		g.addEdge(new Edge(n6, n1));
		g.addEdge(new Edge(n5, n2));
		g.addEdge(new Edge(n4, n3));
		g.makeEntry(n1);
		return g;
	}

	public static FlowGraph makeHechtUllman_4_2() {
		final FlowGraph g = new FlowGraph();
		final Node one = new ExampleNode("1");
		final Node two = new ExampleNode("2");
		final Node three = new ExampleNode("3");
		g.addNode(one);
		g.addNode(two);
		g.addNode(three);
		g.addEdge(new Edge(one, two));
		g.addEdge(new Edge(two, two));
		g.addEdge(new Edge(two, three));
		g.addEdge(new Edge(three, one));
		g.makeEntry(one);
		return g;
	}

	public static FlowGraph makeHechtUllman_4_3() {
		final FlowGraph g = new FlowGraph();
		final Node one = new ExampleNode("1");
		final Node two = new ExampleNode("2");
		final Node three = new ExampleNode("3");
		g.addNode(one);
		g.addNode(two);
		g.addNode(three);
		g.addEdge(new Edge(one, two));
		g.addEdge(new Edge(one, three));
		g.addEdge(new Edge(two, three));
		g.addEdge(new Edge(three, two));
		g.makeEntry(one);
		return g;
	}

	public static FlowGraph makeHechtUllman_6_1_a() {
		final FlowGraph g = new FlowGraph();
		final Node n1 = new ExampleNode("1");
		final Node n2 = new ExampleNode("2");
		final Node n3 = new ExampleNode("3");
		final Node n4 = new ExampleNode("4");
		final Node n5 = new ExampleNode("5");
		g.addNode(n1);
		g.addNode(n2);
		g.addNode(n3);
		g.addNode(n4);
		g.addNode(n5);
		g.addEdge(new Edge(n1, n2));
		g.addEdge(new Edge(n1, n3));
		g.addEdge(new Edge(n2, n3));
		g.addEdge(new Edge(n3, n2));
		g.addEdge(new Edge(n3, n4));
		g.addEdge(new Edge(n3, n5));
		g.addEdge(new Edge(n4, n2));
		g.addEdge(new Edge(n5, n4));
		g.makeEntry(n1);
		return g;
	}

	public static FlowGraph makeHechtUllman_6_2() {
		final FlowGraph g = new FlowGraph();
		final Node i = new ExampleNode("i");
		final Node a = new ExampleNode("a");
		final Node b = new ExampleNode("b");
		final Node c = new ExampleNode("c");
		g.addNode(i);
		g.addNode(a);
		g.addNode(b);
		g.addNode(c);
		g.addEdge(new Edge(i, a));
		g.addEdge(new Edge(a, b));
		g.addEdge(new Edge(a, c));
		g.addEdge(new Edge(b, c));
		g.addEdge(new Edge(c, b));
		g.makeEntry(i);
		return g;
	}

	public static FlowGraph makeHechtUllman_6_3_b() {
		final FlowGraph g = new FlowGraph();
		final Node i = new ExampleNode("i");
		final Node a = new ExampleNode("a");
		final Node b = new ExampleNode("b");
		final Node c = new ExampleNode("c");
		g.addNode(i);
		g.addNode(a);
		g.addNode(b);
		g.addNode(c);
		g.addEdge(new Edge(i, a));
		g.addEdge(new Edge(a, b));
		g.addEdge(new Edge(a, c));
		g.addEdge(new Edge(b, c));
		g.addEdge(new Edge(c, b));
		g.addEdge(new Edge(c, a)); // diff to 6.2
		g.makeEntry(i);
		return g;
	}

	public static FlowGraph makeHechtUllman_6_3_c() {
		final FlowGraph g = new FlowGraph();
		final Node i = new ExampleNode("i");
		final Node a = new ExampleNode("a");
		final Node b = new ExampleNode("b");
		final Node c = new ExampleNode("c");
		g.addNode(i);
		g.addNode(a);
		g.addNode(b);
		g.addNode(c);
		g.addEdge(new Edge(i, a));
		g.addEdge(new Edge(a, b));
		g.addEdge(new Edge(a, c));
		g.addEdge(new Edge(b, c));
		g.addEdge(new Edge(c, b));
		g.addEdge(new Edge(b, a)); // diff to 6.2
		g.addEdge(new Edge(c, a)); // diff to 6.2
		g.makeEntry(i);
		return g;
	}

	public static FlowGraph makeHechtUllman_6_4() {
		final FlowGraph g = new FlowGraph();
		final Node i = new ExampleNode("i");
		final Node a = new ExampleNode("a");
		final Node b = new ExampleNode("b");
		final Node c = new ExampleNode("c");
		final Node d = new ExampleNode("d");
		g.addNode(i);
		g.addNode(a);
		g.addNode(b);
		g.addNode(c);
		g.addNode(d);
		g.addEdge(new Edge(i, a));
		g.addEdge(new Edge(a, b));
		g.addEdge(new Edge(a, c));
		g.addEdge(new Edge(b, c));
		g.addEdge(new Edge(c, d));
		g.addEdge(new Edge(d, b));
		g.makeEntry(i);
		return g;
	}

	public static FlowGraph makeHechtUllman_6_5() {
		final FlowGraph g = new FlowGraph();
		final Node n1 = new ExampleNode("1");
		final Node n2 = new ExampleNode("2");
		final Node n3 = new ExampleNode("3");
		final Node n4 = new ExampleNode("4");
		g.addNode(n1);
		g.addNode(n2);
		g.addNode(n3);
		g.addNode(n4);
		g.addEdge(new Edge(n1, n2));
		g.addEdge(new Edge(n2, n3));
		g.addEdge(new Edge(n3, n2));
		g.addEdge(new Edge(n3, n4));
		g.addEdge(new Edge(n4, n3));
		g.makeEntry(n1);
		return g;
	}	

	public static FlowGraph makeTarjan_1() {
		final FlowGraph graph = new FlowGraph();
		final Node s = new ExampleNode("s");
		final Node a = new ExampleNode("a");
		final Node b = new ExampleNode("b");
		final Node c = new ExampleNode("c");
		final Node d = new ExampleNode("d");
		final Node e = new ExampleNode("e");
		final Node f = new ExampleNode("f");
		final Node g = new ExampleNode("g");
		final Node h = new ExampleNode("h");
		final Node i = new ExampleNode("i");
		final Node j = new ExampleNode("j");
		final Node k = new ExampleNode("k");
		graph.addNode(s);
		graph.addNode(a);
		graph.addNode(b);
		graph.addNode(c);
		graph.addNode(d);
		graph.addNode(e);
		graph.addNode(f);
		graph.addNode(g);
		graph.addNode(h);
		graph.addNode(i);
		graph.addNode(j);
		graph.addNode(k);
		graph.addEdge(new Edge(s, b));
		graph.addEdge(new Edge(s, c));
		graph.addEdge(new Edge(b, a));
		graph.addEdge(new Edge(b, d));
		graph.addEdge(new Edge(b, e));
		graph.addEdge(new Edge(a, d));
		graph.addEdge(new Edge(d, h));
		graph.addEdge(new Edge(e, h));
		graph.addEdge(new Edge(h, b));
		graph.addEdge(new Edge(h, k));
		graph.addEdge(new Edge(k, s));
		graph.addEdge(new Edge(c, f));
		graph.addEdge(new Edge(c, g));
		graph.addEdge(new Edge(f, i));
		graph.addEdge(new Edge(g, i));
		graph.addEdge(new Edge(g, j));
		graph.addEdge(new Edge(j, i));
		graph.addEdge(new Edge(i, k));
		graph.addEdge(new Edge(i, s));
		graph.makeEntry(s);
		return graph;
	}

}
