package ch.usi.inf.sape.graph.example;

import java.util.List;

import ch.usi.inf.sape.graph.Edge;
import ch.usi.inf.sape.graph.FlowGraph;
import ch.usi.inf.sape.graph.Node;

public class ExampleGraphGenerator {

	public static FlowGraph createFullyConnectedGraph(final int nodeCount) {
		FlowGraph g = new FlowGraph();
		for (int i=0; i<nodeCount; i++) {
			g.addNode(new ExampleNode(""+i));
		}
		final List<Node> nodes = g.getNodes();
		for (Node n1 : nodes) {
			for (Node n2 : nodes) {
				g.addEdge(new Edge(n1, n2));
			}
		}
		g.makeEntry(g.getNodes().get(0));
		return g;
	}

	public static FlowGraph createChain(final int nodeCount) {
		FlowGraph g = new FlowGraph();
		for (int i=0; i<nodeCount; i++) {
			g.addNode(new ExampleNode(""+i));
		}
		final List<Node> nodes = g.getNodes();
		for (int i=1; i<nodes.size(); i++) {
			g.addEdge(new Edge(nodes.get(i-1), nodes.get(i)));
		}
		g.makeEntry(nodes.get(0));
		return g;
	}

	public static FlowGraph createDoubleChain(final int nodeCount) {
		FlowGraph g = new FlowGraph();
		for (int i=0; i<nodeCount; i++) {
			g.addNode(new ExampleNode(""+i));
		}
		final List<Node> nodes = g.getNodes();
		for (int i=1; i<nodes.size(); i++) {
			g.addEdge(new Edge(nodes.get(i-1), nodes.get(i)));
			g.addEdge(new Edge(nodes.get(i), nodes.get(i-1)));
		}
		g.makeEntry(nodes.get(0));
		return g;
	}

	/**
	 * This is a pathological case for loop forest finding!
	 * Each iteration will remove just one loopback edge,
	 * and there are half as many loopback edges as there are nodes.
	 * Moreover, the LoopForest uses memory quadratic in the number of nodes
	 * (every level=loop of the tree stores a list of on average half of the graphs node).
	 */
	public static FlowGraph createLoopNestChain(final int nodeCount) {
		FlowGraph g = new FlowGraph();
		for (int i=0; i<nodeCount/2*2; i++) {
			g.addNode(new ExampleNode(""+i));
		}
		final List<Node> nodes = g.getNodes();
		for (int i=1; i<nodes.size(); i++) {
			g.addEdge(new Edge(nodes.get(i-1), nodes.get(i)));
		}
		for (int i=0; i<nodes.size()/2; i++) {
			g.addEdge(new Edge(nodes.get(nodes.size()-i-1), nodes.get(i)));
		}
		g.makeEntry(nodes.get(0));
		return g;
	}

	public static FlowGraph createFan(final int nodeCount) {
		FlowGraph g = new FlowGraph();
		for (int i=0; i<nodeCount; i++) {
			g.addNode(new ExampleNode(""+i));
		}
		final List<Node> nodes = g.getNodes();
		for (int i=1; i<nodes.size(); i++) {
			g.addEdge(new Edge(nodes.get(0), nodes.get(i)));
			g.addEdge(new Edge(nodes.get(i), nodes.get(0)));
		}
		g.makeEntry(nodes.get(0));
		return g;
	}

	public static FlowGraph createFan2(final int nodeCount) {
		FlowGraph g = new FlowGraph();
		for (int i=0; i<nodeCount/2*2+1; i++) {
			g.addNode(new ExampleNode(""+i));
		}
		final List<Node> nodes = g.getNodes();
		for (int i=1; i<nodes.size()/2+1; i++) {
			g.addEdge(new Edge(nodes.get(0), nodes.get(i)));
			g.addEdge(new Edge(nodes.get(i), nodes.get(nodes.size()/2+i)));
			g.addEdge(new Edge(nodes.get(nodes.size()/2+i), nodes.get(i)));
		}
		g.makeEntry(nodes.get(0));
		return g;
	}
	
}
