package ch.usi.inf.sape.graph.example;

import java.io.PrintWriter;

import ch.unisi.inf.sp.tree.Snode;
import ch.unisi.inf.sp.tree.Snode.Kind;
import ch.usi.inf.sape.graph.Edge;
import ch.usi.inf.sape.graph.FlowGraph;
import ch.usi.inf.sape.graph.Node;
import ch.usi.inf.sape.graph.dot.DefaultEdgeAttributeStringProvider;
import ch.usi.inf.sape.graph.dot.DefaultNodeAttributeStringProvider;
import ch.usi.inf.sape.graph.dot.EdgeAttributeStringProvider;
import ch.usi.inf.sape.graph.dot.NodeAttributeStringProvider;
import ch.usi.inf.sape.graph.dot.NormalLoopNodeAttributeStringProvider;
import ch.usi.inf.sape.graph.loop.Loop;
import ch.usi.inf.sape.graph.loop.LoopForest;


/**
 * Write a given FlowGraph or LoopForest to a dot file.
 *  
 * @author Matthias.Hauswirth@usi.ch
 */
public class FinalDotWriter {

	public static String renderToDot(final FlowGraph g) {
		return renderToDot(g, new DefaultNodeAttributeStringProvider(), new DefaultEdgeAttributeStringProvider());
	}

	public static String renderToDot(final FlowGraph g, final NodeAttributeStringProvider nodeAttributeStringProvider, final EdgeAttributeStringProvider edgeAttributeStringProvider) {
		final StringBuffer sb = new StringBuffer("digraph ");
		sb.append('\"');
		sb.append(g.toString());
		sb.append('\"');
		sb.append(" {\n");
		for (final Node node : g.getNodes()) {
			sb.append("  ");
			sb.append('\"');
			sb.append(node.toString());
			sb.append('\"');
			sb.append(nodeAttributeStringProvider.getAttributeString(g, node));
			sb.append("\n");
		}
		for (final Edge edge : g.getEdges()) {
			sb.append("  ");
			sb.append('\"');
			sb.append(edge.getSource().toString());
			sb.append('\"');
			sb.append(" -> ");
			sb.append('\"');
			sb.append(edge.getTarget().toString());
			sb.append('\"');
			sb.append(edgeAttributeStringProvider.getAttributeString(g, edge));
			sb.append("\n");
		}
		sb.append("}\n");
		return sb.toString();
	}
	
	public static String renderToDot(final FlowGraph g, final NodeAttributeStringProvider nodeAttributeStringProvider, final NormalLoopNodeAttributeStringProvider normalNodeStringProvider, final EdgeAttributeStringProvider edgeAttributeStringProvider) {
		final StringBuffer sb = new StringBuffer("digraph ");
		sb.append('\"');
		sb.append(g.toString());
		sb.append('\"');
		sb.append(" {\n");
		for (final Node node : g.getNodes()) {
			sb.append("  ");
			sb.append('\"');
			sb.append(node.toString());
			sb.append('\"');
			sb.append(nodeAttributeStringProvider.getAttributeString(g, node));
			
			sb.append(normalNodeStringProvider.getAttributeString(((FullNode)node).getKind() == Kind.LOOP, node));
			sb.append("\n");
		}
		for (final Edge edge : g.getEdges()) {
			sb.append("  ");
			sb.append('\"');
			sb.append(edge.getSource().toString());
			sb.append('\"');
			sb.append(" -> ");
			sb.append('\"');
			sb.append(edge.getTarget().toString());
			sb.append('\"');
			sb.append(edgeAttributeStringProvider.getAttributeString(g, edge));
			sb.append("\n");
		}
		sb.append("}\n");
		return sb.toString();
	}
	
	public static void renderToDotFully(final PrintWriter pw, final FlowGraph g, final NodeAttributeStringProvider nodeAttributeStringProvider, final NormalLoopNodeAttributeStringProvider normalNodeStringProvider, final EdgeAttributeStringProvider edgeAttributeStringProvider) {
		pw.write("digraph ");
		pw.write('\"');
		pw.write(g.toString());
		pw.write('\"');
		pw.write(" {\n");
		int count = 0;
		for (final Node node : g.getNodes()) {
			count++;
			pw.write("  ");
			pw.write('\"');
			pw.write(node.toString());
			pw.write('\"');
			pw.write(nodeAttributeStringProvider.getAttributeString(g, node));
			
			pw.write(normalNodeStringProvider.getAttributeString(((FullNode)node).getKind() == Kind.LOOP, node));
			pw.write("\n");
			if (count == 1000) {
				pw.flush();
				count = 0;
			}
		}
		count = 0;
		for (final Edge edge : g.getEdges()) {
			count++;
			pw.write("  ");
			pw.write('\"');
			pw.write(edge.getSource().toString());
			pw.write('\"');
			pw.write(" -> ");
			pw.write('\"');
			pw.write(edge.getTarget().toString());
			pw.write('\"');
			pw.write(edgeAttributeStringProvider.getAttributeString(g, edge));
			pw.write("\n");
			if (count == 1000) {
				pw.flush();
				count = 0;
			}
		}
		pw.write("}\n");
		
	}
	
	public static void renderLoopForest(final PrintWriter pw, final FlowGraph g, final LoopForest f) {
		pw.write("digraph ");
		pw.write('\"');
		pw.write(g.toString());
		pw.write(" Loop Forest\"");
		pw.write(" {\n");
		for (final Loop l : f.getTreeRoots()) {
			renderLoopTree(pw, l, 0);
		}
		pw.write("}\n");
		
	}
	
	private static void renderLoopTree(final PrintWriter pw, final Loop l, int i) {
		if (i==200) {
			pw.flush();
			i = 0;
		}
		i++;
		pw.write(" ");
		pw.write(getLoopId(l));
		pw.write(" [label=\"head: ");
		for (final Node h : l.getHeaders()) {
			pw.write(h+" ");
		}
		pw.write("\\nbody: ");
		for (final Node b : l.getBody()) {
			pw.write(b+" ");
		}
		pw.write("\"]\n");
		for (final Loop c : l.getChildren()) {
			pw.write(" ");
			pw.write(getLoopId(l));
			pw.write(" -> ");
			pw.write(getLoopId(c));
			pw.write("\n");
			renderLoopTree(pw, c, i);
		}
	}
	
	private static String getLoopId(final Loop l) {
		StringBuffer sb = new StringBuffer("\"");
		for (final Node h : l.getHeaders()) {
			sb.append(h+" ");
		}
		sb.append("\"");
		return sb.toString();
	}
	
}
