package ch.usi.inf.sape.graph.example;

import java.beans.XMLEncoder;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.ObjectOutput;
import java.io.ObjectOutputStream;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import org.objectweb.asm.Opcodes;

import usi.instrumentation.stack.Node;
import ch.unisi.inf.sp.tree.Snode;
import ch.unisi.inf.sp.tree.Snode.Kind;
import ch.usi.inf.sape.graph.Edge;
import ch.usi.inf.sape.graph.FlowGraph;
import ch.usi.inf.sape.graph.algorithms.HechtUllmanReducibilityTest;
import ch.usi.inf.sape.graph.algorithms.LoopForestFinder;
import ch.usi.inf.sape.graph.algorithms.LoopForestFinder2;
import ch.usi.inf.sape.graph.algorithms.ParallelEdgeCollapser;
import ch.usi.inf.sape.graph.dot.DefaultEdgeAttributeStringProvider;
import ch.usi.inf.sape.graph.dot.DotWriter;
import ch.usi.inf.sape.graph.dot.LoopNestingNodeAttributeStringProvider;
import ch.usi.inf.sape.graph.dot.NormalLoopNodeAttributeStringProvider;
import ch.usi.inf.sape.graph.loop.LoopForest;
import ch.usi.inf.sape.graph.loop.LoopForest2;
import ch.usi.inf.sape.statistics.Counter;
import ch.usi.inf.sape.statistics.StatisticsWriter;

public class RecursionTreater  {
	
	private String filename;
	
	public RecursionTreater(final String filename) {
		this.filename = filename;
	}
	
	public void makeReachability(final ch.usi.inf.sape.graph.Node entryNode, final FlowGraph g) {
		if (entryNode!=null) {
			dfsMark(entryNode);			
		}
		for (final ch.usi.inf.sape.graph.Node n : g.getNodes()) {
			if (!n.isMarked()) {
				if (((FullNode)n).getKind() != Kind.LOOP) {
					dfsMark(n);
					FullEdge.Boundary boundary;
					FullEdge.Type type = null;
					String nodeClassName, childClassName, nodePackage, childPackage;
					nodeClassName = getClassName(((FullNode)entryNode).getLabel());
					childClassName = getClassName(((FullNode)n).getLabel());
					nodePackage = getPackageName(((FullNode)entryNode).getLabel());
					childPackage = getPackageName(((FullNode)n).getLabel());
					//System.out.println("father package " + nodeClassName + " " + node.data);
					//System.out.println("child package " + childClassName + " " + child.data);
					
					if (nodeClassName.equals(childClassName)) {
						boundary = FullEdge.Boundary.CLASSBOUNDARY;
					} else if (nodePackage.equals(childPackage)) {
						boundary = FullEdge.Boundary.PACKAGEBOUNDARY;
					} else {
						boundary = FullEdge.Boundary.NOBOUNDARY;
					}
					g.addEdge(new FullEdge(entryNode, n, type, boundary));
				} 
			}
		}
	}
		
	
	private void dfsMark(final ch.usi.inf.sape.graph.Node node) {
		if (node.isMarked()) {
			return;
		} else {
			node.mark();
			for (final Edge edge : node.getOutEdges()) {
				dfsMark(edge.getTarget());
			}
		}
	}
	
	
	private FlowGraph makeGraph (Collection<Node<String>> tree) {
		final Set<String> dupl = new HashSet<String>();
		final FlowGraph g = new FlowGraph();
		Map<Node<String>, ch.usi.inf.sape.graph.Node> list = new HashMap<Node<String>, ch.usi.inf.sape.graph.Node>();
		for (Node<String> node: tree) {
			ch.usi.inf.sape.graph.Node n;
			if (!list.containsKey(node)) {
				
				n = new FullNode(node.data, ((Snode)node).getKind(), ((Snode)node).numberOfLoops);
				g.addNode(n);
				if (node.data.equals("ENTRY"))
					g.makeEntry(n);
				list.put(node, n);
			} else {
				n = list.get(node);
			}
			
			if (node.getNumberOfChildren()>0) {
				ch.usi.inf.sape.graph.Node c;
				for (Node<String> child : node.getChildren()) {
					if (!list.containsKey(child)) {
						c = new FullNode(child.data, ((Snode)child).getKind(), ((Snode)child).numberOfLoops);
						g.addNode(c);
						list.put(child, c);
					} else {
						c = list.get(child);
					}
					if (!dupl.contains(((FullNode)n).getLabel()+((FullNode)c).getLabel())) {
						FullEdge.Type type = null;
						switch (((Snode)node).getEdge(child)) {
						case Opcodes.INVOKEDYNAMIC:
							type = FullEdge.Type.INVOKEVIRTUAL;
							break;
						case Opcodes.INVOKEINTERFACE:
							type = FullEdge.Type.INVOKEINTERFACE;
							break;
						case Opcodes.INVOKESPECIAL:
							type = FullEdge.Type.INVOKESPECIAL;
							break;
						case Opcodes.INVOKESTATIC:
							type = FullEdge.Type.INVOKESTATIC;
							break;
						case Opcodes.INVOKEVIRTUAL:
							type = FullEdge.Type.INVOKEVIRTUAL;
							break;
						default:
							
							break;
						}
						
						FullEdge.Boundary boundary;
						
						String nodeClassName, childClassName, nodePackage, childPackage;
						nodeClassName = getClassName(node.data);
						childClassName = getClassName(child.data);
						nodePackage = getPackageName(node.data);
						childPackage = getPackageName(child.data);
						//System.out.println("father package " + nodeClassName + " " + node.data);
						//System.out.println("child package " + childClassName + " " + child.data);
						
						if (nodeClassName.equals(childClassName)) {
							boundary = FullEdge.Boundary.CLASSBOUNDARY;
						} else if (nodePackage.equals(childPackage)) {
							boundary = FullEdge.Boundary.PACKAGEBOUNDARY;
						} else {
							boundary = FullEdge.Boundary.NOBOUNDARY;
						}
						
						
						Edge edge = new FullEdge(n, c, type, boundary);
						
						g.addEdge(edge);
						dupl.add(((FullNode)n).getLabel()+((FullNode)c).getLabel());
					}
				}
			}
			
		}
		return g;
	}
	
	private String getClassName (String fullMethodName) {
		if (fullMethodName.equals("ENTRY"))
			return fullMethodName;
		
		return fullMethodName.substring(0, fullMethodName.indexOf("."));
	}
	
	private String getPackageName(String fullMethodName) {
		if (fullMethodName.equals("ENTRY"))
			return fullMethodName;
		
		String className = fullMethodName.substring(0, fullMethodName.indexOf("."));
		if (className.contains("/")) {
			return className.substring(0, className.lastIndexOf("/"));
			
		}
		return "";
	}
	
	private void serialize (final LoopForest forest, final String fileName) {
		
		
		/*JSONSerializer serializer = new JSONSerializer(); 
		try {
			serializer.deepSerialize(forest, new FileWriter(new File("recursion-" + fileName +".obj")));
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}*/
		File f = new File("recursion-" + fileName +".obj");
		/*try {
		    // Serialize object into XML
		    XMLEncoder encoder = new XMLEncoder(new BufferedOutputStream(
		        new FileOutputStream("recursion-" + fileName +".xml")));
		    encoder.writeObject(forest);
		    encoder.close();
		} catch (FileNotFoundException e) {
			System.err.println(e.getMessage());
			e.printStackTrace();
		}*/

		
		try{
			   ObjectOutput ObjOut = new ObjectOutputStream(new FileOutputStream(f));
			   
			
			   ObjOut.writeObject(forest);
			   ObjOut.close();
			 
			   
		} catch(IOException e){
			System.err.println(e.getMessage());
			e.printStackTrace();
		}
	}
	
	
	public void check(Collection<Node<String>> tree) throws CloneNotSupportedException, IOException {
		
		
		final FlowGraph g = makeGraph(tree);
		//System.out.println(tree.size());
		makeReachability(g.getEntryNode(), g);
		tree = null;
		System.gc();
		g.clearAllMarks();
		//g.check(true);
		//g.checkReachability(true);
		
		//System.out.println("first check had passed");
		
		//ParallelEdgeCollapser.eliminateParallelEdges(g);
		//final FlowGraph gCloned = g.clone();
		//boolean reducible = HechtUllmanReducibilityTest.isReducible(g, new ExampleFactory());
		//System.out.println(reducible);
		
		//LoopForestFinder2 forestFinder = new LoopForestFinder2();
		//LoopForest2 forest = forestFinder.find(g);
		//System.out.println(forest.getRoot().getChildren().isEmpty());
		//g.check(true);
		//g.checkReachability(true);
		//System.out.println("second check had passed");
		LoopForestFinder forestFinderN = new LoopForestFinder();
		LoopForest forestN = forestFinderN.find(g);
		serialize(forestN, filename);
		
		/*
		PrintWriter pw1 = new PrintWriter(new FileWriter(filename + "-forest.dot"));
		FinalDotWriter.renderLoopForest(pw1, g, forest);
		pw1.flush();
		pw1.close();
		*/
		
		//overall statistic
		//final String stat = StatisticsWriter.dump(g, forest);
		
		//PrintWriter pw1 = new PrintWriter(new FileWriter(filename + "-loop.dot"));
		
		//pw1.print(FinalDotWriter.renderToDot(g));
		
		
		//pw1.flush();
		//pw1.close();
		
		/*final HashMap<String, Counter> stat = StatisticsWriter.dumpMap(g, forest);
		PrintWriter pw2 = new PrintWriter(new FileWriter(filename + "-stat.txt"));
		for (String str: stat.keySet()) {
			Counter counter = stat.get(str);
			pw2.print(str + "\t" + counter.loops + "\t" + counter.recursions + "\t" + counter.normal + "\r\n");
			
		}
		pw2.close();*/
		
		//g.check(true);
		//g.checkReachability(true);
		
		LoopForestFinder forestFinder2 = new LoopForestFinder();
		LoopForest forest2 = forestFinder2.find(g);
		//System.out.println("third check had passed");
		/*final String s = FinalDotWriter.renderToDot(g, 
				new LoopNestingNodeAttributeStringProvider(forest2),
				new NormalLoopNodeAttributeStringProvider(),
				new DefaultEdgeAttributeStringProvider());
		*/
		
		//final String s = DotWriter.renderToDot(g);
		//final String s = DotWriter.renderToDot(g);
		PrintWriter pw = new PrintWriter(new FileWriter(filename + "-loopFinder.dot"));
		FinalDotWriter.renderToDotFully(pw, g, 
				new LoopNestingNodeAttributeStringProvider(forest2),
				new NormalLoopNodeAttributeStringProvider(),
				new DefaultEdgeAttributeStringProvider());
		//pw.print(s);
		pw.flush();
		pw.close();
	}
	

}
