package ch.usi.inf.sape.graph.loop;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import ch.usi.inf.sape.graph.Node;


/**
 * A loop in a FlowGraph.
 * Given that FlowGraphs can be irreducible, a loop may have multiple header nodes.
 * 
 * Loops are nested (@see LoopForest).
 * 
 * @author Matthias.Hauswirth@usi.ch
 */
public final class Loop implements Serializable {

	private final Loop parent;
	private final ArrayList<Loop> children;
	private final ArrayList<Node> body;
	private final ArrayList<Node> headers;
	
	
	public Loop(final Loop parent) {
		this.parent = parent;
		if (parent!=null) {
			parent.addChild(this);
		}
		children = new ArrayList<Loop>();
		body = new ArrayList<Node>();
		headers = new ArrayList<Node>();
	}
	
	public Loop getParent() {
		return parent;
	}
	
	public void addNode(final Node node) {
		body.add(node);
	}
	
	public List<Node> getBody() {
		return body;
	}
	
	/**
	 * A header node must also be a body node!
	 * @param node
	 */
	public void addHeader(final Node node) {
		assert body.contains(node);
		headers.add(node);
	}
	
	public List<Node> getHeaders() {
		return headers;
	}
	
	public void addChild(final Loop child) {
		children.add(child);
	}
	
	public List<Loop> getChildren() {
		return children;
	}
	
	public void dump(final StringBuffer sb, final String indent) {
		sb.append(indent+"Loop\n");
		sb.append(indent+" headers: ");
		for (final Node h : headers) {
			sb.append(h+" ");
		}
		sb.append("\n");
		sb.append(indent+" body: ");
		for (final Node b : body) {
			sb.append(b+" ");
		}
		sb.append("\n");
		for (final Loop c : children) {
			c.dump(sb, indent+" ");
		}
	}
	
}
