package ch.usi.inf.sape.graph.loop;

import java.util.ArrayList;
import java.util.List;

import ch.usi.inf.sape.graph.Node;


/**
 * A loop in a (possibly irreducible) FlowGraph.
 * 
 * Use this class instead of Loop.
 * 
 * @author Matthias.Hauswirth@usi.ch
 */
public final class Loop2 {

	private final Loop2 parent;
	private final String id;
	private final ArrayList<Node> headers;
	private final ArrayList<Loop2> children;
	public boolean marked = false;
	
	public Loop2(final Loop2 parent) {
		this.parent = parent;
		if (parent!=null) {
			parent.children.add(this);
		}
		if (parent==null) {
			id = "FOREST";
		} else {
			id = parent.getId()+"."+parent.children.size();
		}
		headers = new ArrayList<Node>();
		children = new ArrayList<Loop2>();
	}
	
	/**
	 * @return A String uniquely identifying this loop in the LoopForest2.
	 */
	public String getId() {
		return id;
	}
	
	/**
	 * @return The immediate outer loop of this loop.
	 * If this loop is an outermost loop in the graph,
	 * getParent() returns the root "loop".
	 * If this loop is the root "loop",
	 * getParent() returns null.
	 */
	public Loop2 getParent() {
		return parent;
	}
	
	/**
	 * @return The loops directly nested in this loop.
	 */
	public List<Loop2> getChildren() {
		return children;
	}
	
	public void addHeader(final Node header) {
		headers.add(header);
	}
	
	public List<Node> getHeaders() {
		return headers;
	}
	
}
