package ch.usi.inf.sape.graph.loop;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import ch.usi.inf.sape.graph.Node;


/**
 * A LoopForest represents the entire loop nesting structure in a (possibly non-reducible) FlowGraph.
 * 
 * @author Matthias.Hauswirth@usi.ch
 */
public final class LoopForest implements Serializable {

	private ArrayList<Loop> treeRoots;
	
	
	public LoopForest() {
		treeRoots = new ArrayList<Loop>();
	}
	
	public void addRoot(final Loop root) {
		treeRoots.add(root);
	}
	
	public List<Loop> getTreeRoots() {
		return treeRoots;
	}
	
	/**
	 * Is the given Node a loop header (in any tree, at any nesting depth)?
	 * @param n A Node
	 * @return
	 */
	public boolean isHeader(final Node n) {
		for (final Loop l : treeRoots) {
			if (isHeader(n, l)) {
				return true;
			}
		}
		return false;
	}
	
	private boolean isHeader(final Node n, final Loop parent) {
		if (parent.getHeaders().contains(n)) {
			return true;
		}
		for (final Loop l : parent.getChildren()) {
			if (isHeader(n, l)) {
				return true;
			}
		}
		return false;
	}
	
	public void dump(final StringBuffer sb) {
		sb.append("LoopForest\n");
		for (final Loop l : treeRoots) {
			l.dump(sb, " ");
		}
	}
	
}
