package ch.usi.inf.sape.statistics;

import java.util.HashMap;

import ch.unisi.inf.sp.tree.Snode.Kind;
import ch.usi.inf.sape.graph.Edge;
import ch.usi.inf.sape.graph.FlowGraph;
import ch.usi.inf.sape.graph.Node;
import ch.usi.inf.sape.graph.example.FullEdge;
import ch.usi.inf.sape.graph.example.FullNode;
import ch.usi.inf.sape.graph.example.LoopKind2;

import ch.usi.inf.sape.graph.example.LoopKind2.Type;
import ch.usi.inf.sape.graph.loop.Loop2;
import ch.usi.inf.sape.graph.loop.LoopForest;
import ch.usi.inf.sape.graph.loop.LoopForest2;

public class StatisticsWriter {

	/*
	- number of polymorphic recursive cycles (recursive cycles that involve at least one INVOKE_INTERFACE or INVOKE_VIRTUAL)
	- number of constructive recursive cycles (recursive cycles that involve at least one constructor) + loop constractive including transitive
	- number of in-package recursive cycles (recursive cycles that do NOT cross package-boundaries*)
	- number of in-class recursive cycles (recursive cycles that do NOT cross class-boundaries*)
	- number of header-only recursive cycles (recursive cycles where the header consists of the entire body)
	- number of single-header recursive cycles (recursive cycles where the header consists of a single node)
	- number of multi-header recursive cycles (recursive cycles where the header consists of multiple nodes)
	- recursive cycle body size distribution (number of a cycle's body nodes)
	- recursive cycle header size distribution (number of a cycle's header nodes)
	- nesting depth information:
	 - number of loop per method distribution
	 - number of top/second/third/...-level recursive cycles (I expect to see mostly top) dfs on forrest

	*/
	
	private static HashMap<Loop2, LoopKind2> map;
	private static int[] numberOfnodesPerLevelRecursiveCycles = new int[10]; //dfs on forest
	private static int numberOfConstructiveLoopNodes = 0; //dfs on tree
	
	public static String dump(final FlowGraph g, LoopForest2 f) {
		int numberOfPolyMorphicRecursiveCycles = 0; //ok
		int numberOfConstructiveRecursiveCycles = 0;//ok
		int numberOfInPackageRecursiveCycles = 0;//ok
		int numberOfInClassRecursiveCycles = 0;//ok
		int numberOfOtherRecursiveCycles = 0;//ok
		
		int numberOfHeaderOnlyRecursiveCycles = 0; //all headers are body
		int numberOfSingleHeaderRecursiveCycles = 0; //header single node
		int numberOfMultiheaderRecursiveCycles = 0; //many headers but there are other nodes also
		
		int[] numberOfLoopsPerMethod = new int[10]; //ok
		
		
		
		map = new HashMap<Loop2, LoopKind2>();
		String str;
		int normal = 0;
		int loop = 0;
		int recursive = 0;
		LoopKind2 lk;
		for (final Node node : g.getNodes()) {
			
			Loop2 l = f.getLoop(node);
			if (map.containsKey(l)) {
				lk = map.get(l); 
			} else {
				lk = new LoopKind2();
				map.put(l, lk);
			}
			
			if (!lk.marked) {
				if (!l.getId().equals("FOREST")) {
					if (l.getHeaders().size()>1) {
						if (!l.getHeaders().contains(node) && ((FullNode)node).getKind()!=Kind.LOOP) {
							lk.type = Type.MANYHEADERS;
							lk.marked = true;
						} else {
							lk.type = Type.MANYHEADERSONLY;
						}
					} else {
						if (!l.getHeaders().contains(node) && ((FullNode)node).getKind()!=Kind.LOOP) {
							lk.type = Type.HEADER;
							lk.marked = true;
						} else {
							lk.type = Type.HEADERONLY;
						}
					}
				}				
			}
						
			
			if (((FullNode)node).getKind() == Kind.LOOP) {
				loop++;
			
			} else if (f.isHeader(node)) {
				if (((FullNode)node).numberOfLoops>8) {
					numberOfLoopsPerMethod[9]++;
				} else {
					numberOfLoopsPerMethod[((FullNode)node).numberOfLoops]++;
				}
				recursive++;
			} else {
				if (((FullNode)node).numberOfLoops>8) {
					numberOfLoopsPerMethod[9]++;
				} else {
					numberOfLoopsPerMethod[((FullNode)node).numberOfLoops]++;
				}
				normal++;
			}
			
			
			
		}
		
		System.out.println("\t" + recursive + "\t" + (normal-1) + "\t" + loop);
		return null;
		/*for (Edge edgeT: g.getEdges()) {
			FullEdge edge = (FullEdge) edgeT;
			
			Node source = edge.getSource();
			Node target = edge.getTarget();
			
			if (!f.getLoop(source).equals(f.getRoot())) {
				if (f.getLoop(source).equals(f.getLoop(target))) {
					
					if (edge.type != FullEdge.Type.INVOKEINTERFACE && 
							edge.type != FullEdge.Type.INVOKEVIRTUAL) {
						map.get(f.getLoop(source)).isPolymorphic = false;
					} 
					if (edge.type == FullEdge.Type.INVOKESPECIAL) {
						
						map.get(f.getLoop(source)).isConstractive = true;
					}
					
					if (edge.boundary == FullEdge.Boundary.NOBOUNDARY) {
						map.get(f.getLoop(source)).boundary = FullEdge.Boundary.NOBOUNDARY;
					} else if (edge.boundary == FullEdge.Boundary.PACKAGEBOUNDARY &&
							map.get(f.getLoop(source)).boundary != FullEdge.Boundary.NOBOUNDARY) {
						map.get(f.getLoop(source)).boundary = FullEdge.Boundary.PACKAGEBOUNDARY;
					}
				}
				
			}	
		}
		
		for (Loop2 l: map.keySet()) {
			if (!f.getRoot().equals(l)) {
				lk = map.get(l);
				if (lk.isPolymorphic) {
					numberOfPolyMorphicRecursiveCycles++;
				}
				if (lk.isConstractive) {
					numberOfConstructiveRecursiveCycles++;
				}
				if (lk.boundary == FullEdge.Boundary.CLASSBOUNDARY) {
					numberOfInClassRecursiveCycles++;
				} else if (lk.boundary == FullEdge.Boundary.PACKAGEBOUNDARY) {
					numberOfInPackageRecursiveCycles++;
				} else {
					numberOfOtherRecursiveCycles++;
				}
				if (lk.type == LoopKind2.Type.HEADER) {
					numberOfHeaderOnlyRecursiveCycles++;
				} else if (lk.type == LoopKind2.Type.HEADERONLY) {
					numberOfSingleHeaderRecursiveCycles++;
				} else if (lk.type == LoopKind2.Type.MANYHEADERSONLY) {
					numberOfMultiheaderRecursiveCycles++;
				}
			}
		}
		
		//dfs of forest
		dfs(f.getRoot(), 0);
		
		//dfs on graph
		g.clearAllMarks();
		dfsGraph(g.getEntryNode());
		
		//normal loop recursive edges
		str = "" + (normal-1) +  " ";
		str = str + loop  +  " ";
		str = str + recursive + " ";
		str = str + g.getEdges().size() + " ";
		str = str + numberOfPolyMorphicRecursiveCycles + " ";
		str = str + numberOfConstructiveRecursiveCycles + " ";
		str = str + numberOfConstructiveLoopNodes + " ";
		str = str + numberOfInPackageRecursiveCycles + " ";
		str = str + numberOfInClassRecursiveCycles + " ";
		str = str + numberOfOtherRecursiveCycles + " ";
		str = str + numberOfHeaderOnlyRecursiveCycles + " ";
		str = str + numberOfSingleHeaderRecursiveCycles + " ";
		str = str + numberOfMultiheaderRecursiveCycles + " ";
		
		for (int i = 0; i<10; i++) {
			str = str + numberOfLoopsPerMethod[i] + " ";
			
		}
		for (int i = 0; i<10; i++) {
			str = str + numberOfnodesPerLevelRecursiveCycles[i] + " ";
		}
		System.out.println("normal loop recursive edges polyCycles constrCycles " +
				"constrLoops inPackage inClass other header headeronly multiheader" +
				" 0 1 2 3 4 5 6 7 8 9" +
				" 0 1 2 3 4 5 6 7 8 9");
		System.out.println(str);
		return str;*/
	}
	
	
	public static HashMap<String, Counter> dumpMap(final FlowGraph g, LoopForest2 f) {
			
		HashMap<String, Counter> stat = new HashMap<String, Counter>();
		
		Counter counter  = null;
		String className = null;
		for (final Node n : g.getNodes()) {
			FullNode node = (FullNode)n;
		//	System.out.println(node.getLabel());
			if (node.getLabel().contains(".")) {
				className = node.getLabel().substring(0, node.getLabel().indexOf("."));		
				
			} else {
				continue;
			}
			
			if (stat.containsKey(className)) {
				counter = stat.get(className);
			} else {
				counter = new Counter();
				stat.put(className, counter);
			}
			if (node.getKind() == Kind.LOOP) {
				counter.loops++;
			
			} else if (f.isHeader(node)) {
				
				counter.recursions++;
			} else {
		
				counter.normal++;
			}
			
			
			
		}
		
		return stat;
	}
	
	private static void dfs (Loop2 loop, int depth) {
		if (loop.marked) {
			depth--;
			return;
		} else {
			if (depth>8) {
				numberOfnodesPerLevelRecursiveCycles[9]++;
			} else {
				numberOfnodesPerLevelRecursiveCycles[depth]++;
			}
			depth++;
			loop.marked = true;
			for (Loop2 childLoop: loop.getChildren()) {
				dfs(childLoop, depth);
			}
		}		
	}

	private static void dfsGraph (Node node) {
		if (node.isMarked()) {
			return;
		} else {
			
			
			node.mark();
			for (final Edge edge : node.getOutEdges()) {
				dfsGraph(edge.getTarget());
			}
			
			if (((FullNode)node).getLabel().contains("<init>") &&
					((FullNode)node).getKind()!=Kind.LOOP) {
				((FullNode)node).Constructive_transitive = true;
			}
			
			if (((FullNode)node).Constructive_transitive) {
				for (final Edge edge : node.getInEdges()) {
					if (!((FullNode)edge.getSource()).Constructive_transitive) {
						((FullNode)edge.getSource()).Constructive_transitive = true;
						
						if ((((FullNode) edge.getSource())).getKind() == Kind.LOOP) {
							numberOfConstructiveLoopNodes++;
						}
					}
					
				}
			}
		}
	}
}
