/*
 * Created on Nov 22, 2005 at 5:49:20 PM.
 * 
 * Copyright (c) 2005 Robert Virkus / Enough Software
 *
 * This file is part of J2ME Polish.
 *
 * J2ME Polish is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * J2ME Polish is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with J2ME Polish; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Commercial licenses are also available, please
 * refer to the accompanying LICENSE.txt or visit
 * http://www.j2mepolish.org for details.
 */
package de.enough.polish.util;

import java.text.MessageFormat;
import java.util.ResourceBundle;

/**
 * <p>Translations can be used for easier maintainance of internationalized/localized applications.</p>
 *
 * <p>Copyright Enough Software 2005</p>
 * <pre>
 * history
 *        Nov 22, 2005 - rob creation
 * </pre>
 * @author Robert Virkus, j2mepolish@enough.de
 */
public class Translations {

	private final ResourceBundle bundle;

	/**
	 * Creates a new translations object.
	 * 
	 * @param bundle the resource bundle with translations. 
	 */
	public Translations( ResourceBundle bundle ) {
		super();
		this.bundle = bundle;
	}
	
	public ResourceBundle getBundle() {
		return this.bundle;
	}
	
	public String getString( String name ) {
		return this.bundle.getString(name);
	}

	public String getString( String name, Object value ) {
		String message = this.bundle.getString(name);
		return MessageFormat.format( message, new Object[]{ value} );
	}
	
	public String getString( String name, Object[] values ) {
		String message = this.bundle.getString(name);
		return MessageFormat.format( message, values );
	}

}
