package org.objectweb.asm.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.RandomAccessFile;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import org.objectweb.asm.ClassReader;
import org.objectweb.asm.Opcodes;
import org.objectweb.asm.tree.ClassNode;
import org.objectweb.asm.tree.MethodNode;

public class CountClasses implements Opcodes {
	private static final String javaClassPath = "C:\\CLASSES";
	private boolean isEventListener = false;
	private boolean debug = false;
	private int numEventInterfaceListeners = 0;
	private int numEventClassListeners = 0;
	private int numEventAbstractListeners = 0;
	private int numClasses = 0;
	private int numInterfaces = 0;
	private int numAbstract = 0;
	private int numObserverInterfaces = 0;
	private int numObserverClasses = 0;
	
	public void printStatistics () {
		System.out.println("Number of class event listeners: " + numEventClassListeners);
		System.out.println("Number of abstract event listeners: " + numEventAbstractListeners);
		
		System.out.println("Number of interface event listeners: " + numEventInterfaceListeners);
		System.out.println("Number of class observers: " + numObserverClasses);
		System.out.println("Number of interface observers: " + numObserverInterfaces);
		System.out.println("Number of abstract classes: " + numAbstract);
		
		System.out.println("Number of normal classes: " + numClasses);
		System.out.println("Number of interfaces: " + numInterfaces);
	}
	private void traverseAndInstrument(File file) {
		if (file.isDirectory()) {		
			File[] fileArray = file.listFiles();
			for (File currentFile : fileArray ) {
				traverseAndInstrument(currentFile);
			}
		} else {
			if (file.getName().endsWith(".class")) {
				instrumentClass(file.getAbsolutePath());
				//countObserverables(file.getAbsolutePath());
			}
		}
	}

	public boolean visitInterface (String currentInterface) {
		String windowsLikeInterfacePathAndName = currentInterface;
		try {
			if (currentInterface.equals("java/util/EventListener")) {
				isEventListener=true;
				return true;
			}
			ClassNode cn = new ClassNode ();
			InputStream in;
			windowsLikeInterfacePathAndName = javaClassPath + "\\" + windowsLikeInterfacePathAndName.replace("/", "\\") + ".class";
			in = new FileInputStream (windowsLikeInterfacePathAndName);
			ClassReader cr = new ClassReader (in);
			cr.accept(cn, 0);
			for (int cnt=0; (cnt<cn.interfaces.size()); cnt++) {
				if (debug) System.out.println("Found interface: " + cn.interfaces.get(cnt).toString());
				visitInterface (cn.interfaces.get(cnt).toString());
			}						
			if (debug) System.out.println("Parsed interface: " + windowsLikeInterfacePathAndName);	
			in.close();
		} catch (Exception ex) {
			System.out.println("INTERFACE NOT FOUND: " + windowsLikeInterfacePathAndName);
		}
		return false;
	}

	public boolean visitClass (String className) {
		String windowsLikeClassPathAndName = className;
		try {
			boolean passThrough=false;
			ClassNode cn = new ClassNode ();
			InputStream in;
			windowsLikeClassPathAndName = javaClassPath + "\\" + windowsLikeClassPathAndName.replace("/", "\\") + ".class";
			in = new FileInputStream (windowsLikeClassPathAndName);
			ClassReader cr = new ClassReader (in);
			cr.accept(cn, 0);
			if (debug) System.out.println("Looking for a file: " + windowsLikeClassPathAndName);
			if ((cn.access & ACC_INTERFACE) > 0){
				visitInterface(cn.name);
				passThrough = true;
			} else
				for (int cnt=0; (cnt<cn.interfaces.size()); cnt++) {
					if (debug)System.out.println("Interface" + cnt + " " + cn.interfaces.get(cnt).toString());
					visitInterface (cn.interfaces.get(cnt).toString());
				}
			if (cn.name.equals("java/lang/Object") || passThrough) {
				if (isEventListener) {
					in.close();
					return true;
				} else {
					in.close();
					return false;
				}
			} else {
				if (cn.superName.startsWith("["))
					System.out.println("Super class: " + cn.superName);
				boolean results = visitClass (cn.superName);
				in.close();
				return results;
			}
			
		} catch (Exception ex) {
			System.out.println("CLASS NOT FOUND: " + windowsLikeClassPathAndName);
			return false;
		}
	}

	public void instrumentClass (String currentClass) {
		try {
			ClassNode cn = new ClassNode ();
			InputStream in = new FileInputStream (currentClass);
			ClassReader cr = new ClassReader(in);
			cr.accept(cn, 0);
			if (visitClass (cr.getClassName())) {
				System.out.println("Class " + cr.getClassName() + " belongs to EventListeners");
				if ((cn.access & ACC_INTERFACE) > 0) 
					numEventInterfaceListeners++;
				else
					if ((cn.access & ACC_ABSTRACT) > 0) 
						numEventAbstractListeners++;
					else
						numEventClassListeners++;
				isEventListener = false;
			}
			if ((cn.access & ACC_INTERFACE) > 0) 
				numInterfaces++;
			else
				if ((cn.access & ACC_ABSTRACT) > 0)
					numAbstract++;
				else
					numClasses++;
			in.close();
		} catch (Exception e) {
			System.out.println("ERROR IN: " + currentClass);
		};	
	}
	
	
	
	
	public void countObserverables (String currentClass) {
		try {
			ClassNode cn = new ClassNode ();
			InputStream in = new FileInputStream (currentClass);
			ClassReader cr = new ClassReader(in);
			cr.accept(cn, 0);
			Iterator methodList = cn.methods.iterator();
			boolean classIsObserver = false;
			while (methodList.hasNext()) { 
				MethodNode mn = (MethodNode) methodList.next();
				if (mn.name.endsWith("Listener") && (mn.name.startsWith("add"))) {
					System.out.println("Class " + cn.name + " is an observer");
					classIsObserver = true;
					continue;
				}
			}
			if (classIsObserver) {
				if ((cn.access & ACC_INTERFACE) > 0)
					numObserverInterfaces++;
				else
					numObserverClasses++;
			}
			in.close();
		} catch (Exception ex) {
			System.out.println("ERROR IN: " + currentClass);
		}
	}

	public void CountClassesFromTraceFiles (String source) {
		try {
			HashMap <String, String> classes = new HashMap <String, String> ();
			RandomAccessFile ra = new RandomAccessFile ("c:\\aaa.bbb", "r");
			String currentLine, className ="";
			while ((currentLine = ra.readLine()) != null) {
				if (currentLine.startsWith("[Load")) {
					currentLine = currentLine.substring(8);
					//if (currentLine.contains("$"))
					//	className = currentLine.substring(0, currentLine.indexOf('$'));
					//else 
					if (currentLine.contains(" from")) {
						className = currentLine.substring(0, currentLine.indexOf(" from"));
					//System.out.println(className);
					classes.put(className, "");
					}
				}
						
			}
			System.out.println(classes.size());
		} catch (Exception ex){
			
		}
		
	}
	
	public static void main(String[] args) {
		CountClasses cc = new CountClasses ();
		cc.CountClassesFromTraceFiles("C:\\aaa.bbb");
		//cc.traverseAndInstrument(new File ("C:\\unjared_netbeans_third"));
		//cc.printStatistics();
		//CountClasses cc2 = new CountClasses ();
		//System.out.println("-----------------------------------");
		//cc2.traverseAndInstrument(new File ("C:\\source\\rt"));
		//cc2.printStatistics();
	}
}
