package usi.instrumentation;

import static org.objectweb.asm.Opcodes.ATHROW;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.objectweb.asm.Opcodes;
import org.objectweb.asm.Type;
import org.objectweb.asm.tree.AbstractInsnNode;
import org.objectweb.asm.tree.ClassNode;
import org.objectweb.asm.tree.FieldInsnNode;
import org.objectweb.asm.tree.IincInsnNode;
import org.objectweb.asm.tree.InsnList;
import org.objectweb.asm.tree.IntInsnNode;
import org.objectweb.asm.tree.JumpInsnNode;
import org.objectweb.asm.tree.LabelNode;
import org.objectweb.asm.tree.LdcInsnNode;
import org.objectweb.asm.tree.LookupSwitchInsnNode;
import org.objectweb.asm.tree.MethodInsnNode;
import org.objectweb.asm.tree.MethodNode;
import org.objectweb.asm.tree.MultiANewArrayInsnNode;
import org.objectweb.asm.tree.TableSwitchInsnNode;
import org.objectweb.asm.tree.TryCatchBlockNode;
import org.objectweb.asm.tree.TypeInsnNode;
import org.objectweb.asm.tree.VarInsnNode;
import org.objectweb.asm.util.AbstractVisitor;

import ch.unisi.inf.sp.cfg.Block;
import ch.unisi.inf.sp.cfg.Edge;
import ch.unisi.inf.sp.cfg.ExceptionData;
import ch.unisi.inf.sp.cfg.ExceptionEdge;
import ch.unisi.inf.sp.cfg.Exceptions;
import ch.unisi.inf.sp.cfg.InstType;
import ch.unisi.inf.sp.cfg.InstructionData;
import ch.unisi.inf.sp.cfg.NormalEdge;
import ch.unisi.inf.sp.cfg.Pair;

public class BBlockExtractor {

	public Map <Block, List <Block>> domTree;
	
	private MethodNode mn;
	private ClassNode cn;
	
	public List <Block> produceBlocksForMethod(final MethodNode method, ClassNode cn) {
		this.cn = cn;
		mn = method;
		Exceptions exceptions = new Exceptions();
		for (final Object o : method.tryCatchBlocks) {
			final TryCatchBlockNode node = (TryCatchBlockNode)o;
			InsnList instrs = method.instructions;
			exceptions.add(new ExceptionData(
					instrs.indexOf(node.start),
					instrs.indexOf(node.end),
					instrs.indexOf(node.handler),
					node.type));
		}
		
		final InsnList instructions = method.instructions;
		final List<InstructionData> instData = getInstructions(instructions);
		final InstType[] jumps = makeJumps(instData, exceptions);
		List<Block> blocks = makeBlocks(instData, jumps);
		int i =0;
		for (Block b: blocks) {
			b.setBlockIndex(i++);
		}
		return blocks;
		
	}
	
	public Map <Block, Block> disassembleMethod(final MethodNode method, ClassNode cn) {
		this.cn = cn;
		mn = method;
		Exceptions exceptions = new Exceptions();
		for (final Object o : method.tryCatchBlocks) {
			final TryCatchBlockNode node = (TryCatchBlockNode)o;
			InsnList instrs = method.instructions;
			exceptions.add(new ExceptionData(
					instrs.indexOf(node.start),
					instrs.indexOf(node.end),
					instrs.indexOf(node.handler),
					node.type));
		}
		
		final InsnList instructions = method.instructions;
		
		final List<InstructionData> instData = getInstructions(instructions);
		final InstType[] jumps = makeJumps(instData, exceptions);
		List<Block> blocks = makeBlocks(instData, jumps);
		
		int i =0;
		for (Block b: blocks) {
			b.setBlockIndex(i++);
		}
		domTree = buildDominatorTree(blocks);
		Map <Block, Block> loops = identifyLoops(domTree, blocks);
				
		return loops;
		
	}
	
	private Map<Block, Block> identifyLoops (Map<Block, List<Block>> dominator, List<Block> blocks) {
		Map<Block, Block> loop = new HashMap<Block, Block>();
		for (Block block: blocks) {
			List<Edge> edges = block.getEdges();
			List<Block> dominators = dominator.get(block);
			for (Edge edge: edges) {
				if (!edge.v.contains("ex")) {
					if (dominators!=null) {
						for (Block dom: dominators) {
							if (edge.t.intValue()==dom.getBlockNumber()) {
								loop.put(block, dom);
							}
						}						
					}
					
				}
			}
		}
		
		
		
		return loop;
	}
	
	private Map<Block, List<Block>> buildPredecessorTree(List<Block> blocks) {
		Map<Block, List<Block>> predMap = new HashMap<Block, List<Block>>();
		List<Block> predList;
		Block blockZero = null;
		for (Block block: blocks) {
				
			if (block.getBlockNumber() == 0) {
				blockZero = block;
			}
			List<Edge> edges = block.getEdges();
			
			for (Edge edge: edges) {
				
				for (Block nextBlock: blocks) {
					if (edge.t.intValue()==nextBlock.getBlockNumber()) {
						if (!predMap.containsKey(nextBlock)) {
							predList = new ArrayList<Block>();
							
						} else {
							predList = predMap.get(nextBlock);
						}
						predList.add(block);
						predMap.put(nextBlock, predList);
					}
				}
				
			}	
		}
		boolean is = true;
		Block entry = new Block ();
		entry.setBlockIndex(-1);
		
		for (Block block: predMap.keySet()) {
			if (block.getBlockNumber()==0) {
				is = false;
				List <Block> al = predMap.get(block);
				al.add(entry);
				predMap.put(block, al);
				break;
			}
		}
		if (is) {
			ArrayList <Block> zeroList = new ArrayList <Block> ();
			zeroList.add(entry);
			predMap.put(blockZero, zeroList);
		}
		return predMap;
	}
	
		
	private Map <Block, List <Block>> dom (List <Block> nodes, Map <Block, List <Block>> predMap) {
		HashMap <Block, List <Block>> dom = new HashMap <Block, List <Block>> ();
		ArrayList <Block> al;
		for (Block b: predMap.keySet()) {
			if (b != null)
			if (b.getBlockIndex() == -1) {
				al = new ArrayList <Block> ();
				al.add (b);
				dom.put(b, al);
			} else {
				al = new ArrayList <Block> ();
				for (Block key: predMap.keySet()) {
					al.add(key);
				}
				dom.put(b, al);
			}
		}

		HashMap <Block, List <Block>> old = new HashMap <Block, List <Block>> ();
		boolean change = true;
		while (change) {
			change = false;
			
			Iterator <Block> itOld = dom.keySet().iterator();
			while (itOld.hasNext()) {
				Block curBlock = itOld.next();
				List <Block> curList = dom.get(curBlock);
				List <Block> oldList = new ArrayList <Block> (); 
				for (Block b : curList) {
					oldList.add(b);
				}
				old.put(curBlock, oldList);
			}
			 
			for (Block b: nodes) {
				ArrayList <Block> intersect = new ArrayList <Block> ();
				
				if (!predMap.containsKey(b)) {
					continue;
				}
				List <Block> curPred  = predMap.get(b);
				for (Block block: curPred) {
					if (dom.get(block) != null) {
						intersect.addAll(dom.get(block));						
					}
				}
				
				int size = curPred.size();
				HashMap <Block, Integer> countHash = new HashMap <Block, Integer> ();
				for (Block cur: intersect) {
					if (countHash.containsKey(cur)) {
						Integer num = countHash.get(cur);
						num++;
						countHash.put(cur, num);
					} else {
						countHash.put(cur, 1);
					}
				}
				
				Set <Block> intersection = new HashSet <Block> ();
				Iterator <Block> it = countHash.keySet().iterator();
				while (it.hasNext()) {
					Block block = it.next ();
					Integer num = countHash.get(block);
					if (num == size) {
						intersection.add(block);
					}
				}

				intersection.add(b);
				dom.put(b, new ArrayList <Block>(intersection));
			}
			
			Iterator <Block> itCompNew = dom.keySet().iterator();
			while (itCompNew.hasNext() && !change) {
				Block blockCompNew = itCompNew.next();
				
				Iterator <Block> itCompOld = old.keySet().iterator();
				while (itCompOld.hasNext() && !change) {
					Block blockCompOld = itCompOld.next();
					if (blockCompOld.getBlockIndex() == blockCompNew.getBlockIndex()) {
						List <Block> oldList = old.get(blockCompOld);
						List <Block> newList = dom.get(blockCompNew);
						for (Block oldBlock : oldList) {
							boolean found = false;
							for (Block newBlock : newList) {
								if (oldBlock.getBlockIndex() == newBlock.getBlockIndex()) {
									found = true;
									break;
								}
							}
							if (!found) {
								change = true;
								break;
							}
						}
					} 
				}
			}
			
		}
		
		return dom;
	}
	
	public Map<Block, List<Block>> predescessors;
		
	private Map <Block, List<Block>> buildDominatorTree (List<Block> blocks) {
		Map<Block, List<Block>> predMap = buildPredecessorTree(blocks);
		predescessors = new HashMap<Block, List<Block>>(predMap);
		
		Block toPutIn = null;
		for (Block b : predMap.keySet()) {
		
			if (b != null) 
			if (b.getBlockNumber() == 0) {
				for (Block block : predMap.get(b)) {
					//System.out.println("aaa");
					if (block.getBlockNumber() == -1) {
						//System.out.println("bb");
						blocks.add(block);
						toPutIn = block;
					}
				}
			}
		}
		predMap.put(toPutIn, new ArrayList <Block> ());
		predMap = dom (blocks, predMap);
		
		return predMap;
	}
	
	private List<Block> makeBlocks(final List<InstructionData> instData, final InstType[] jumps) {
		int n = instData.size();
		List<Block> blocks = new ArrayList<Block>(n);

		Block current = new Block();
		for (int i = 0; i < n; i++) {
			switch (jumps[i]) {
				case NORMAL:
					current.addInstr(instData.get(i));
					break;
				case IN:
					if (!current.isEmpty()) {
						blocks.add(current);
						if (current.getEdges().isEmpty()) {
							current.getEdges().add(new NormalEdge(i, ""));
						}
					}
					current = new Block();
					current.addInstr(instData.get(i));
					break;
				case OUT:
					current.addInstr(instData.get(i));
					current.setEdges(instData.get(i).edges);
					blocks.add(current);
					current = new Block();
					break;
				case RETURN:
					current.addInstr(instData.get(i));
					blocks.add(current);
					current = new Block();
					break;
			}
		}
		if (!current.isEmpty()) {
			blocks.add(current);
		}
		return blocks;
	}

	private InstType[] makeJumps(List<InstructionData> instData, Exceptions exceptions) {
		int n = instData.size();
		final InstType[] jumps = new InstType[n];
		for (int i = 0; i < n; i++) {
			jumps[i] = InstType.NORMAL;
		}

		for (int i = 0; i < n; i++) {
			InstructionData data = instData.get(i);
			if (data.isReturn()) {
				jumps[i] = InstType.OUT;
				data.edges.add(new NormalEdge(n + 1,""));
			} else if (data.edges.size() > 0) {
				jumps[i] = InstType.OUT;
				for (Edge edge : data.edges) {
					jumps[edge.t] = InstType.IN;
				}
			}
			
			if (data.isPEI()) {
				jumps[i] = (jumps[i] == InstType.RETURN) ? InstType.RETURN : InstType.OUT;
				Pair<List<Integer>, Boolean> pair = exceptions.getPossibleJumps(data.i);
				List<Integer> exJumps = pair.t;
				for (int j: exJumps) {
					jumps[j] = InstType.IN;
					data.edges.add(new ExceptionEdge(j, "ex"));
				}
				if (!pair.v) {
					data.edges.add(new ExceptionEdge(n + 1, "ex"));
				}
				if ((data.opcode != ATHROW) && (!data.isReturn()) && (i+1<n)) {
					jumps[i+1] = InstType.IN;
					data.edges.add(new NormalEdge(i+1, ""));
				}
			}
		}
		return jumps;
	}

	
	private List<InstructionData> getInstructions(InsnList instructions) {
		final int n = instructions.size();
	
		final List<InstructionData> instData = new ArrayList<InstructionData>(n);
		int nextInstructionNumber = 0;
		int permanent = 0;
		for (int i = 0; i < n; i++) {
			
			final AbstractInsnNode instruction = instructions.get(i);
			if (instruction.getOpcode() == 168 || instruction.getOpcode() == 169) {
				System.out.println(cn.name + "." + mn.name + " " + OpcodeReader.giveName(instruction));
			}
			
			
			InstructionData data = disassembleInstruction(instruction, i, instructions, instruction.byteCodeNumber);
			if (mn.name.equals("runBenchmark") && cn.name.contains("TestHarness")) {
				if (data.instructionNumber==0) permanent = instruction.byteCodeNumber;
				System.out.println((instruction.byteCodeNumber)+": " + data.getName());
			}
			
			if (data == null) {
				throw new IllegalStateException("Unknown instruction - " + instruction.getOpcode());
			}
			instData.add(data);
		}
		return instData;
	}
	
	public InstructionData disassembleInstruction(
			final AbstractInsnNode instruction, 
			final int i,
			final InsnList instructions, int instructionNumber) {
		final int opcode = instruction.getOpcode();
		final String mnemonic = opcode == -1 ? ""
				: AbstractVisitor.OPCODES[instruction.getOpcode()] + " ";
	
		switch (instruction.getType()) {
		case AbstractInsnNode.LABEL:
			// pseudo-instruction (branch or exception target)
			return new InstructionData(i, opcode, instructionNumber);
		case AbstractInsnNode.FRAME:
			// pseudo-instruction (stack frame map)
			return new InstructionData(i, opcode, instructionNumber);
		case AbstractInsnNode.LINE:
			// pseudo-instruction (line number information)
			return new InstructionData(i, opcode, instructionNumber);
		case AbstractInsnNode.INSN:
			// Opcodes: NOP, ACONST_NULL, ICONST_M1, ICONST_0, ICONST_1,
			// ICONST_2,
			// ICONST_3, ICONST_4, ICONST_5, LCONST_0, LCONST_1, FCONST_0,
			// FCONST_1, FCONST_2, DCONST_0, DCONST_1, IALOAD, LALOAD, FALOAD,
			// DALOAD, AALOAD, BALOAD, CALOAD, SALOAD, IASTORE, LASTORE,
			// FASTORE,
			// DASTORE, AASTORE, BASTORE, CASTORE, SASTORE, POP, POP2, DUP,
			// DUP_X1, DUP_X2, DUP2, DUP2_X1, DUP2_X2, SWAP, IADD, LADD, FADD,
			// DADD, ISUB, LSUB, FSUB, DSUB, IMUL, LMUL, FMUL, DMUL, IDIV, LDIV,
			// FDIV, DDIV, IREM, LREM, FREM, DREM, INEG, LNEG, FNEG, DNEG, ISHL,
			// LSHL, ISHR, LSHR, IUSHR, LUSHR, IAND, LAND, IOR, LOR, IXOR, LXOR,
			// I2L, I2F, I2D, L2I, L2F, L2D, F2I, F2L, F2D, D2I, D2L, D2F, I2B,
			// I2C, I2S, LCMP, FCMPL, FCMPG, DCMPL, DCMPG, IRETURN, LRETURN,
			// FRETURN, DRETURN, ARETURN, RETURN, ARRAYLENGTH, ATHROW,
			// MONITORENTER, or MONITOREXIT.
			// zero operands, nothing to print
			return new InstructionData(mnemonic, i, opcode, instructionNumber);
		case AbstractInsnNode.INT_INSN:
			// Opcodes: NEWARRAY, BIPUSH, SIPUSH.
			if (instruction.getOpcode() == Opcodes.NEWARRAY) {
				// NEWARRAY
				return new InstructionData(mnemonic + AbstractVisitor.TYPES[((IntInsnNode) instruction).operand], i, opcode, instructionNumber);
			} else {
				// BIPUSH or SIPUSH
				return new InstructionData(mnemonic + ((IntInsnNode) instruction).operand, i, opcode, instructionNumber);
			}
		case AbstractInsnNode.JUMP_INSN:
			// Opcodes: IFEQ, IFNE, IFLT, IFGE, IFGT, IFLE, IF_ICMPEQ,
			// IF_ICMPNE, IF_ICMPLT, IF_ICMPGE, IF_ICMPGT, IF_ICMPLE, IF_ACMPEQ,
			// IF_ACMPNE, GOTO, JSR, IFNULL or IFNONNULL.
		{
			final LabelNode targetInstruction = ((JumpInsnNode) instruction).label;
			final int targetId = instructions.indexOf(targetInstruction);
			
			List<Edge> edges = new ArrayList<Edge>();
			
			if (!mnemonic.equals("GOTO ")) {
				edges.add(new NormalEdge(i+1, "F"));
				edges.add(new NormalEdge(targetId, "T"));
			} else {
				edges.add(new NormalEdge(targetId, ""));
			}
			return new InstructionData(edges, mnemonic + targetId, i, opcode, instructionNumber);
		}
		case AbstractInsnNode.LDC_INSN:
			// Opcodes: LDC.
			//bug here as ldc_w and ldc are merged, but ldc_w is wider
			
			
			return new InstructionData(mnemonic + ((LdcInsnNode) instruction).cst, i, opcode, instructionNumber);
		case AbstractInsnNode.IINC_INSN:
			// Opcodes: IINC.
			return new InstructionData(mnemonic + ((IincInsnNode) instruction).var + " " 
					+ ((IincInsnNode) instruction).incr, i, opcode, instructionNumber);
		case AbstractInsnNode.TYPE_INSN:
			// Opcodes: NEW, ANEWARRAY, CHECKCAST or INSTANCEOF.
			return new InstructionData(mnemonic + ((TypeInsnNode) instruction).desc, i, opcode, instructionNumber);
		case AbstractInsnNode.VAR_INSN:
			// Opcodes: ILOAD, LLOAD, FLOAD, DLOAD, ALOAD, ISTORE,
			// LSTORE, FSTORE, DSTORE, ASTORE or RET.
			//this can take either two or one

			return new InstructionData(mnemonic + ((VarInsnNode) instruction).var, i, opcode, instructionNumber);
		case AbstractInsnNode.FIELD_INSN:
			// Opcodes: GETSTATIC, PUTSTATIC, GETFIELD or PUTFIELD.
			return new InstructionData(mnemonic + ((FieldInsnNode) instruction).owner + "." 
					+ ((FieldInsnNode) instruction).name + " " + ((FieldInsnNode) instruction).desc, i, opcode, instructionNumber);
		case AbstractInsnNode.METHOD_INSN:
			// Opcodes: INVOKEVIRTUAL, INVOKESPECIAL, INVOKESTATIC,
			// INVOKEINTERFACE or INVOKEDYNAMIC.

			return new InstructionData(mnemonic + ((MethodInsnNode) instruction).owner + "."
					+ ((MethodInsnNode) instruction).name + " " + ((MethodInsnNode) instruction).desc, i, opcode, instructionNumber);
		case AbstractInsnNode.MULTIANEWARRAY_INSN:
			// Opcodes: MULTIANEWARRAY.
			return new InstructionData(mnemonic + ((MultiANewArrayInsnNode) instruction).desc + " " +
					((MultiANewArrayInsnNode) instruction).dims, i, opcode, instructionNumber);
		case AbstractInsnNode.LOOKUPSWITCH_INSN:
			// Opcodes: LOOKUPSWITCH.
		{
			List<Edge> edges = new ArrayList<Edge>();
			final List keys = ((LookupSwitchInsnNode) instruction).keys;
			final List labels = ((LookupSwitchInsnNode) instruction).labels;
			for (int t = 0; t < keys.size(); t++) {
				final int key = (Integer) keys.get(t);
				final LabelNode targetInstruction = (LabelNode) labels.get(t);
				final int targetId = instructions.indexOf(targetInstruction);
				edges.add(new NormalEdge(targetId, "K:" + key));
			}
			final LabelNode defaultTargetInstruction = ((LookupSwitchInsnNode) instruction).dflt;
			final int defaultTargetId = instructions
					.indexOf(defaultTargetInstruction);
			edges.add(new NormalEdge(defaultTargetId, "default:"));
			
			
			return new InstructionData(edges, mnemonic, i, opcode, instructionNumber);
		}
		case AbstractInsnNode.TABLESWITCH_INSN:
			// Opcodes: TABLESWITCH.
			/*
			 * initial tableswitch allocates 19 position (1 value and 1 default)
			 */
		{
			List<Edge> edges = new ArrayList<Edge>();
			final int minKey = ((TableSwitchInsnNode) instruction).min;
			final List labels = ((TableSwitchInsnNode) instruction).labels;
			
			for (int t = 0; t < labels.size(); t++) {
				final int key = minKey + t;
				final LabelNode targetInstruction = (LabelNode) labels.get(t);
				final int targetId = instructions.indexOf(targetInstruction);
				edges.add(new NormalEdge(targetId, "K:" + key));
			}
			final LabelNode defaultTargetInstruction = ((TableSwitchInsnNode) instruction).dflt;
			final int defaultTargetId = instructions
					.indexOf(defaultTargetInstruction);
			edges.add(new NormalEdge(defaultTargetId, "default:"));
			return new InstructionData(edges, mnemonic, i, opcode, instructionNumber);
		}
		}
		return null;
	}

	
}
