package usi.instrumentation;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.concurrent.ConcurrentHashMap;

import usi.instrumentation.deserializer.RecursionForestDeserializer;
import usi.instrumentation.stack.ArrayStack;
import usi.instrumentation.stack.EdgeAnnotation;
import usi.instrumentation.stack.Node;
import usi.instrumentation.stack.RecursionStorage;
import usi.instrumentation.stack.Stack;
import usi.instrumentation.stack.Tree;
import usi.instrumentation.stack.Node.RepetitionEntrance;

public class CCT {

	public static volatile HashMap<Long, CCT> threadMap;
	static {
		threadMap = new HashMap<Long, CCT>();
	}
	
	public static ThreadLocal <CCT> curCCT = new ThreadLocal <CCT> () {
		protected synchronized CCT initialValue() {
			CCT cct = new CCT();
			threadMap.put(Thread.currentThread().getId(), cct);
			return cct;
		}
		
	};
	
	public Stack stack = new ArrayStack();
	public Tree<String> tree = new Tree<String>();
	public Node<String> rootElement;
	public Node<String> curElement;
	public HashMap<Node<String>, Integer> methodCount = new HashMap<Node<String>, Integer>();
	public HashSet<RecursionStorage> recSet = new HashSet<RecursionStorage>();
	
	private static void updateMethodCounter(Node<String> curElement, CCT cur) {
		
					
		if (!cur.methodCount.containsKey(curElement)) {
			if (Configuration.COUNT_COST)
				cur.methodCount.put(curElement, new Integer(1));
		} else {
			Integer i = cur.methodCount.get(curElement);
			i++;
			cur.methodCount.put(curElement, i);
		}
		
	}
	
	private static void addToEmptyCCT (Object obj, CCT cur, long id) {
				
		cur.rootElement = new Node<String>("ENTRY");
		cur.tree.setRootElement(cur.rootElement);
		cur.curElement = cur.rootElement;
		cur.stack.push(cur.rootElement);
		cur.curElement = new Node<String>((String)obj, id, cur.rootElement);
		cur.rootElement.addChild(cur.curElement);
		cur.stack.push(cur.curElement);

		if (Configuration.DEBUG || Configuration.DEBUG_ENTRIES_CCT) {
			if (cur.rootElement.data.contains("/")) {
				System.out.println("adding " + cur.rootElement.data);											
			} else {
				if (!cur.rootElement.data.equals("ENTRY"))
				System.out.println("adding " + Table.table.get(Integer.parseInt(cur.rootElement.data)));
			}
		}
		updateMethodCounter(cur.curElement, cur);
	}
	
	private static void detectRecursion (CCT cur, boolean iterates) {

		boolean isThere = false;
		for (RecursionStorage rs : cur.recSet) {
			if (rs.node.equals(cur.curElement)) { //&& rs.pointer.equals((Node<String>)cur.stack.top())) {
				rs.addPointer((Node<String>)cur.stack.top());
				rs.extra.totalNumberOfIterations++;
				if (iterates) {
					int i = rs.counters.get(rs.counters.size()-1);
					rs.counters.remove(rs.counters.size()-1);
					i++;
					rs.counters.add(i);
					cur.curElement.enteredRepetition = RepetitionEntrance.ITERATING;
					
				} else {
					rs.extra.numberOfEntrancies++;
					if (rs.counters.get(rs.counters.size()-1)<rs.extra.minNumberOfIterations) {
						rs.extra.minNumberOfIterations = rs.counters.get(rs.counters.size()-1);
					}
					if (rs.counters.get(rs.counters.size()-1)>rs.extra.maxNumberOfIterations) {
						rs.extra.maxNumberOfIterations = rs.counters.get(rs.counters.size()-1);
					}
					rs.counters.add(1);
					cur.curElement.enteredRepetition = RepetitionEntrance.ENTERED;
					rs.descendants.put(rs.counters.size()-1, findRelatedIteration(rs.node));
					
					
					
				}
				
				isThere = true;
				break;
			}
		}
		if (!isThere) {
			
			ArrayList<Integer> al = new ArrayList<Integer>();
			al.add(new Integer(1));
			EdgeAnnotation ea = new EdgeAnnotation();
			ea.numberOfEntrancies++;
			RecursionStorage rs = new RecursionStorage(cur.curElement, al, ea);
			cur.recSet.add(rs);	
			if (!cur.stack.isEmpty()) {
				rs.addPointer((Node<String>)cur.stack.top());				
			} else {
				rs.addPointer(cur.curElement);
				
			}
			cur.curElement.enteredRepetition = RepetitionEntrance.ENTERED;
				
		}
		
		
		if (Configuration.DEBUG || Configuration.DEBUG_ENTRIES_CCT) {
			if (cur.curElement.data.contains("/")) {
				System.out.println("adding " + cur.curElement.data);											
			} else {
				System.out.println("adding " + Table.table.get(Integer.parseInt(cur.curElement.data)));
			}
		}
		
	}
	
	private static Object findPotentiallyRecursiveNode (String name, CCT cur) {
		String intName = name;
		Object element = null;
		if (!name.contains("/")) {
			name = Table.table.get(Integer.parseInt(name));
		}
		if (RecursionForestDeserializer.dataStructure.containsKey(name)) {	
			for (String str: RecursionForestDeserializer.dataStructure.get(name)) {
				String number;
				if (str.contains("-")) {
					element = cur.stack.containsData(str);
				} else {
					
					for (Integer i: Table.table.keySet()) {
						String value = Table.table.get(i);
						
						if (value.equals(str)) {
							number = ""+i;
							element = cur.stack.containsData(number);
							break;
						}
					}	
				}
				
				if (element!=null) {	
					return element;
				}
			}
			
			if (cur.curElement.children!=null)
			for (Node<String> n: cur.curElement.children) {
				if (n.data.equals(intName)) {
					updateMethodCounter(n, cur);
					return n;
				}
			}
			
			long id = Thread.currentThread().getId();
			Node<String> nextNode = new Node<String>(intName, id, cur.curElement);
			cur.curElement.addChild(nextNode);
			updateMethodCounter(nextNode, cur);
			return nextNode; 
		}
		
		return null;
	}
	
	private static boolean addToCCTwithRecursion (Object obj, CCT cur) {
		String name = (String)obj;
		
		Object element = null;
		if (Configuration.STATIC_RECURSIONS) {
			element = findPotentiallyRecursiveNode(name, cur);
		} else {
			element = cur.stack.containsData((String)obj);			
		}
		if (element!=null) {
			
			boolean iterates = false;
			cur.curElement = (Node<String>)element;
			
			if (Configuration.STATIC_RECURSIONS) {
				if (cur.stack.containsData(cur.curElement.data)!=null) {
					iterates = true;
					
				}
				detectRecursion(cur, iterates);
			} else if (((Node<String>)cur.stack.top()).children==null ||
					!((Node<String>)cur.stack.top()).children.contains(element)) {
				if (cur.stack.containsMoreThanOne(cur.curElement.data)) {
					iterates = true;
					 					
				}
				detectRecursion(cur, iterates);
			}
			cur.stack.push(cur.curElement);
			updateMethodCounter(cur.curElement, cur);
			return true;
		
		} else {
			
			for (Node<String> node: ((Node<String>)cur.stack.top()).getChildren()) {
				if (node.data.equals((String)obj)) {
					cur.curElement = node;
					if (Configuration.DEBUG || Configuration.DEBUG_ENTRIES_CCT) {
						if (node.data.contains("/")) {
							System.out.println("adding " + node.data);											
						} else {
							System.out.println("adding " + Table.table.get(Integer.parseInt(node.data)));
						}
					}
					cur.stack.push(cur.curElement);
					updateMethodCounter(cur.curElement, cur);
					return true;
				}
			}
			
		}
		
		
		return false;
	}
	
	private static void addToCCTwithoutRecursion (Object obj, CCT cur) {
		for (Node<String> node: ((Node<String>)cur.stack.top()).getChildren()) {
			if (node.data.equals((String)obj)) {
				cur.curElement = node;
				
				if (Configuration.DEBUG || Configuration.DEBUG_ENTRIES_CCT) {
					if (node.data.contains("/")) {
						System.out.println("adding " + node.data);											
					} else {
						System.out.println("adding " + Table.table.get(Integer.parseInt(node.data)));
					}
				}
				cur.stack.push(cur.curElement);
				updateMethodCounter(cur.curElement, cur);
				return;
			}
		}
	}
	
	public static void addToCCT (Object obj) {
		if (Configuration.LIMIT_TO_SPECIFIC_THREAD) {
			if (!Thread.currentThread().getName().equals(Configuration.THREAD_NAME)) {
				return;
			}
		}
		CCT cur = curCCT.get();
		long id = Thread.currentThread().getId();
		if (cur.stack.isEmpty()) {
			addToEmptyCCT(obj, cur, id);
			return;
		}
		
		boolean done = false;
		if (Configuration.MAINTAIN_RECURSION) {
			done = addToCCTwithRecursion(obj, cur);
		} else {
			addToCCTwithoutRecursion(obj, cur);	
		}
		
		
		if (!done) {
			if (((String)obj).contains("/")) {
			
				if (cur.curElement.data.contains("/")) {
					if (!((String)obj).contains(cur.curElement.data.substring(0, cur.curElement.data.lastIndexOf("/")))) {
						cleanStack(((String)obj), cur);
					}
																					
				} else {
					if (!((String)obj).contains(Table.table.get(Integer.parseInt(cur.curElement.data))
							.substring(0, Table.table.get(Integer.parseInt(cur.curElement.data)).indexOf("(")))) {
						cleanStack(((String)obj), cur);
					}
				}
			}
			
			Node<String> nextNode = new Node<String>((String)obj, id, cur.curElement);
			cur.curElement.addChild(nextNode);
			if (Configuration.DEBUG || Configuration.DEBUG_ENTRIES_CCT) {
				if (nextNode.data.contains("/")) {
					System.err.println("adding " + nextNode.data);											
				} else {					
					System.err.println("adding " + Table.table.get(Integer.parseInt(nextNode.data)));
					System.err.println("to " + Table.table.get(Integer.parseInt(cur.curElement.data)));
				}
			}
			cur.curElement = nextNode;
			updateMethodCounter(cur.curElement, cur);
			cur.stack.push(nextNode);
		}
		
	}
	private static void cleanStack (String str, CCT cur) {
		if (((Node<String>)cur.stack.top()).data.contains("/")) {
			if (!str.contains(((Node<String>)cur.stack.top()).data.substring(0, ((Node<String>)cur.stack.top()).data.lastIndexOf("/")))) {
				cur.stack.pop();
				cleanStack(str, cur);
			} else {
				cur.curElement = ((Node<String>)cur.stack.top());
			}
		} else {
			if (!str.contains(Table.table.get(Integer.parseInt(((Node<String>)cur.stack.top()).data)).substring(0, Table.table.get(Integer.parseInt(((Node<String>)cur.stack.top()).data)).indexOf("(")))) {
				cur.stack.pop();
				cleanStack(str, cur);
			} else {
				cur.curElement = ((Node<String>)cur.stack.top());
			}
		}
	}
	
	
	private static ArrayList<String> exceptionStorage = new ArrayList<String>();
	private static void trackExceptions (String name) {
		exceptionStorage.add(name);
	}
	
	private static void printExceptions () {
		for (String str: exceptionStorage) {
			System.out.println(Table.table.get(Integer.parseInt(str)));
		}
	}
	
	public static void removeFromCCT (String name) {
		if (Configuration.LIMIT_TO_SPECIFIC_THREAD) {
			if (!Thread.currentThread().getName().equals(Configuration.THREAD_NAME)) {
				return;
			}
		}
		CCT cur =  curCCT.get();
		if (name.startsWith("ex-")) {
			trackExceptions(name.substring(3));
			name = name.substring(3);
			
		}
		if (((Node<String>)cur.stack.top()).data.equals(name) || Configuration.TRACK_INPUTS) {
			String rn = name;
			if (!name.contains("/")) {
				rn = Table.table.get(Integer.parseInt(name));
			} 
			
			if (RecursionForestDeserializer.dataStructure.containsKey(rn)) {
				for (String n: RecursionForestDeserializer.dataStructure.get(rn)) {
					for (Integer key: Table.table.keySet()) {
						if (rn.equals(Table.table.get(key))) {
							if (cur.stack.containsData(""+key)!=null && !cur.stack.containsMoreThanOne(""+key)) {
								FieldProfiler.recountReferences();
							}
						}
					}
				}
			}
					
			
			removeFromCCT();
		} else {
			
			if (cur.stack.contains(name)) {
				System.out.println("on top of stack is " + ((Node<String>)cur.stack.top()).data + 
				"instead of " + name);
				while (!((Node<String>)cur.stack.top()).data.equals(name)) {
					cur.stack.pop();
				}
				removeFromCCT();
			}
			
			if (cur.stack.contains(name)) {
				System.out.println("on top of stack is " + ((Node<String>)cur.stack.top()).data + 
				" and not on stack at all " + name);
			}
		}
		
	}
	
	public static void removeFromCCT () {
		if (Configuration.LIMIT_TO_SPECIFIC_THREAD) {
			if (!Thread.currentThread().getName().equals(Configuration.THREAD_NAME)) {
				return;
			}
		}
		CCT cur =  curCCT.get();			
		
		
		if (Configuration.DEBUG || Configuration.DEBUG_ENTRIES_CCT) {
			if (((Node<String>)cur.stack.top()).data.contains("/")) {
				System.out.println("removing " + ((Node<String>)cur.stack.top()).data);											
			} else {
				System.out.println("removing " + Table.table.get(Integer.parseInt(((Node<String>)cur.stack.top()).data)));
			}
		}
		if (cur.stack.isEmpty()) {
			System.out.println("stack is empty, leaving the method first");
			Exception e = new Exception();
			e.printStackTrace();
			System.exit(-1);
			return;
		}
		Node<String> temp = (Node<String>)cur.stack.top();
		cur.stack.pop();
		if (cur.stack.isEmpty()) {
			System.out.println("stack is empty, leaving the method second " + temp);
			Exception e = new Exception();
			e.printStackTrace();
			printStack();
			System.exit(-1);
			return;
		} else {
			cur.curElement = (Node<String>)cur.stack.top();
		}
	}
	
	private static void printStack() {
		CCT cur = curCCT.get();			
		while (!cur.stack.isEmpty()) {
			if (((Node<String>)cur.stack.top()).data.contains("/") ||
					((Node<String>)cur.stack.top()).data.contains("ENTRY"))  {
						System.out.println(((Node<String>)cur.stack.top()).data);					
			} else {
						System.out.println(Table.table.get(Integer.parseInt(((Node<String>)cur.stack.top()).data)));
			}
					cur.stack.pop();
		}
	}
	
	public static Node<String> getNode (String name) {
		CCT cur = curCCT.get();			
		
		if (!cur.curElement.data.equals(name)) {
			if (cur.stack.containsData(name)!=null) {
				cur.curElement = (Node<String>) cur.stack.containsData(name);
				while (!cur.stack.top().equals(cur.curElement)) {
					cur.stack.pop();
				}
				return cur.curElement;
			} else {
				return null;
			}
		}
		return cur.curElement;
	}
	
	public static Integer findRelatedIteration (Node<String> node) {
		CCT cur = curCCT.get();
		Node<String> curNode = node.father;
		boolean found = false;
		
		while (!found) {
			
			if (curNode.data.contains("/")) {
				if (Configuration.COUNT_COST)
					return LoopProfiler.map.get(curNode).findAlivePosition();
			} else {
				for (RecursionStorage rs : cur.recSet) {
					if (rs.node.equals(cur.curElement)) {
						found = true;
						return rs.counters.size()-1;
					}
				}	
			}
			
			if (!found) {
				curNode = curNode.father;
			}
			
			if (curNode == null) {
				return null;
			}
		}
		
		return 0;
	}
}

 
