package usi.instrumentation;

	import java.io.PrintWriter;
	import java.util.Iterator;
	import org.objectweb.asm.*;
	import org.objectweb.asm.commons.JSRInlinerAdapter;
import org.objectweb.asm.tree.AbstractInsnNode;
	import org.objectweb.asm.tree.ClassNode;
import org.objectweb.asm.tree.JumpInsnNode;
	import org.objectweb.asm.tree.MethodNode;
import org.objectweb.asm.util.*;

	/**
	 * Class that instruments all classes with ASM library
	 * @author zaparand
	 *
	 */
	public class InlineJSR implements Opcodes {
		
		//field from ASM that gives a class node
		private ClassNode cn;
		//field from ASM that gives a method node
		private MethodNode mn;
		
		
		/*
		 * singelton class
		 */
		public static InlineJSR ic = new InlineJSR();
		private InlineJSR () {}
		
		
		synchronized public byte [] instrument (String className, final byte [] classBytes, ClassLoader loader) {
			boolean somethingIsChanged = true;
			
			cn = new ClassNode ();
			
			if ((cn.access & ACC_INTERFACE)>0) {
				return classBytes;
			}
			
			try {
				ClassReader cr = new ClassReader(classBytes);
				
				cr.accept(cn, 0);
			
				//cn.visitMethod(access, name, desc, signature, exceptions);
				
				/*
				 * let's make a pass and inline all JSR/RET instructions first
				 */
				
				return (inline(cn, classBytes));
				
				
			} catch (Exception e) {
				System.err.println("JSRInliner failed");
				System.err.println("should never happen, instrument class fails");
				e.printStackTrace();
			};	
			return classBytes;
		
		}
		
		public static byte[] inline(ClassNode cn, final byte [] classBytes) {
			Iterator <MethodNode> methods = cn.methods.iterator();
			//iterate over a methods
			MethodNode mn;
			while (methods.hasNext()) {
				mn = (MethodNode) methods.next();
				
				//MethodVisitor mv = cn.visitMethod(mn.access, mn.name, mn.desc, mn.signature, exceptions);
				
				
				JSRInliner jsrInliner = new JSRInliner(mn);
				AbstractInsnNode ain;
				Iterator<AbstractInsnNode> ains = mn.instructions.iterator(); 
				
				while (ains.hasNext()) {
					ain = ains.next();
					if (ain instanceof JumpInsnNode) {
						jsrInliner.visitAndMark((JumpInsnNode)ain);
					}
				}
				
				if (jsrInliner.subroutineHeads.isEmpty()) {
					//System.out.println("inlined classes for " + cn.name);
					return classBytes;
				}
				
				jsrInliner.visitEnd();
				System.out.println("inlined classes for " + cn.name);
			}
				ClassWriter cw = new ClassWriter (0);
				cn.accept (cw);	
				return cw.toByteArray();
			
		}
		
}
