package usi.instrumentation;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;

//import soot.util.IdentityHashSet;


import ch.usi.inf.sape.graph.Node;
import ch.usi.inf.sape.graph.loop.Loop;

public class InputCounter {

	enum BehavioralType {CONSTRUCTION, MODIFICATION, TRAVERSAL, INPUT, OUTPUT};
	public HashSet<Object> elements;
	public int number;
	public Loop loop;
	public String name;
	public HashMap<String, Integer> numberPerType;
	public int accessed;
	public ArrayList<Object> accessedElements;
	public int edgeNumber; 
	public BehavioralType behavioralType = BehavioralType.MODIFICATION; //default value
	private boolean finalized = false;
	
	public InputCounter(HashSet<Object> elements, int number, ArrayList<Object> accessedElements, int accessed, Loop loop, String name) {
		this.elements = elements;
		this.number = number;
		this.loop = loop;
		this.name = name;
		this.numberPerType = new HashMap<String, Integer>();
		this.accessed = accessed;
		this.accessedElements = accessedElements;
	}
	
	public boolean merges(Object counter) {
		if (counter!= null && counter instanceof InputCounter) {
			InputCounter c = (InputCounter) counter;
			for (Object obj1: c.elements) {
				for (Object obj2: this.elements) {
					if (obj1 == obj2)
						return true;
				}
			}
			
			for (Object obj1: c.accessedElements) {
				for (Object obj2: this.accessedElements) {
					if (obj1 == obj2)
						return true;
				}
			}
		}
		return false;
	}
	public void merge (InputCounter counter) {
		merge(counter, false);
	}
	public void merge (InputCounter counter, boolean atEnd) {
		if (atEnd) {
			if (this.elements.size()>counter.elements.size()) {
				if (!finalized) {
					behavioralType = BehavioralType.MODIFICATION;					
				}
			} else if (this.elements.size()<counter.elements.size()) {
				behavioralType = BehavioralType.CONSTRUCTION;
				finalized = true;
			} else {
				behavioralType = BehavioralType.MODIFICATION;
				finalized = true;
			}
		}
		
		
		this.elements.addAll(counter.elements);
			
		for (Object obj: counter.accessedElements) {
			if (!this.accessedElements.contains(obj)) {
				this.accessedElements.add(obj);
				this.accessed++;
			}
		}
		this.number = this.elements.size();
		this.numberPerType.clear();
		for (Object ref: this.elements) {
			if (this.numberPerType.containsKey(ref.getClass().getName())) {
				this.numberPerType.put(ref.getClass().getName(), 
						this.numberPerType.get(ref.getClass().getName())+1);
			} else {
				this.numberPerType.put(ref.getClass().getName(), 1);
			}
		}
		
		
		
		
	}
	
	
	@Override
	public int hashCode() {
		int hashcode = 1;
		for (Object node: elements) {
			hashcode = hashcode ^ node.hashCode();
		}
		return hashcode;
	}
	
	
	
}