package usi.instrumentation;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;

import org.objectweb.asm.*;
import org.objectweb.asm.commons.JSRInlinerAdapter;
import org.objectweb.asm.tree.AbstractInsnNode;
import org.objectweb.asm.tree.ClassNode;
import org.objectweb.asm.tree.FieldInsnNode;
import org.objectweb.asm.tree.InsnList;
import org.objectweb.asm.tree.InsnNode;
import org.objectweb.asm.tree.JumpInsnNode;
import org.objectweb.asm.tree.LdcInsnNode;
import org.objectweb.asm.tree.MethodInsnNode;
import org.objectweb.asm.tree.MethodNode;
import org.objectweb.asm.tree.LabelNode;
import org.objectweb.asm.tree.TypeInsnNode;
import org.objectweb.asm.tree.VarInsnNode;
import org.objectweb.asm.util.*;

import usi.instrumentation.deserializer.FieldForestDeserializer;
import usi.instrumentation.deserializer.RecursionForestDeserializer;
import usi.instrumentation.tracker.BytecodeInstructionTracker;

import ch.unisi.inf.sp.cfg.Block;
import ch.unisi.inf.sp.cfg.ControlFlowGraphExtractor;
import ch.unisi.inf.sp.cfg.Edge;
import ch.unisi.inf.sp.cfg.InstructionData;
import ch.unisi.inf.sp.cfg.InstructionPointer;
import ch.unisi.inf.sp.cfg.Loop;
import ch.unisi.inf.sp.collections.CollectionFilter;

/**
 * Class that instruments all classes with ASM library
 * @author zaparand
 *
 */
public class InstrumentClass implements Opcodes {
	
	private ClassNode cn;
	private MethodNode mn;
	
	
	private static int counter = 0;
	
	public static InstrumentClass ic = new InstrumentClass();
	private InstrumentClass () {}
	
	
	synchronized public byte [] instrument (String className, final byte [] classBytes, ClassLoader loader) {
		boolean somethingIsChanged = true;
		
		cn = new ClassNode ();
		
		if ((cn.access & ACC_INTERFACE)>0) {
			return classBytes;
		}
		Map<Block, Block> loops;
		try {
			ClassReader cr = new ClassReader(classBytes);
			
			cr.accept(cn, 0);
						
			if (Configuration.DEBUG || Configuration.BUILD_UNINSTRUMENTED_CFG) {
				if (Configuration.classToSee.equals(cn.name)) {
						try {
								
								ClassNode cni = new ClassNode();
								ClassReader cri = new ClassReader(classBytes);
								cri.accept(cni, 0);
								BufferedWriter bf = new BufferedWriter(new FileWriter(new File("/Users/zaparanuks/agent/"
										+cni.name.replaceAll("/", "_") + "_uninstrumented_graph.dot")));
								new ControlFlowGraphExtractor().build(cni, bf);								
								System.out.println("thread cfg uninstrumented finished " + cni.name);
								
							} catch (IOException e) {
								System.err.println("Couldn't build CFG due to IO exception");
								e.printStackTrace();
							}				
				}
			}
			Iterator <MethodNode> methodList = cn.methods.iterator();
			while (methodList.hasNext()) {
				
				mn = (MethodNode) methodList.next();
				if (((mn.access & ACC_ABSTRACT)>0) || ((mn.access & ACC_NATIVE)>0)) {
					continue;
				} 
				if (mn.name.contains("clinit")) continue;
				
				
				
				
				if (Configuration.DEBUG) {
					System.out.println(cn.name + "/" + mn.name + "/" +mn.desc + " " + counter);					
				}
			
				if (!Configuration.LOOP_NESTS) {
					Table.table.put(new Integer(counter), cn.name + "/" + mn.name + "/" +mn.desc);
					collectCallingContext();
					
					counter++;
					collectEndCallingContext();
					if (Configuration.TRACK_INPUTS) {
						instrumentGetfields();
						instrumentPutfields();
						instrumentNews();
					}
				}
				
				BBlockExtractor bb = new BBlockExtractor();
				
				if (Configuration.TRACK_INDEXES) {
					if (RecursionForestDeserializer.dataStructure.containsKey(cn.name + "/" + mn.name + "/" +mn.desc)) {	
						
						BytecodeInstructionTracker.writeIndexes(cn.name + "/" + mn.name + "/" +mn.desc, bb.produceBlocksForMethod(mn, cn));
					}
				}
				
				loops = bb.disassembleMethod(mn, cn);
				List<Block>[] allB = getAllBlocksOfLoop(bb, loops);
				List<Loop> iterations = new LoopBuilder(loops, mn, cn, bb, allB).build();
				
				if (Configuration.DEBUG || Configuration.DEBUG_LOOP_FINDER) {
					for (Loop loop: iterations) {
						String instrToHeader = "";
						String instrOutLoop = "";
						
						for (AbstractInsnNode node: loop.instructionsPointingToHeader) {
							instrToHeader = instrToHeader + " " + OpcodeReader.giveName(node); 
						}
						for (AbstractInsnNode node: loop.instructionsPointingOutOfTheLoop.keySet()) {
							instrOutLoop = instrOutLoop + " " + OpcodeReader.giveName(node); 
						}
						System.out.println("method has following loops: " + cn.name + "." + mn.name + mn.desc +
								" header is " + OpcodeReader.giveName(loop.header) + 
								loop.numbers[0] + "-" + loop.numbers[1] + 
								" instruction pointing to header " + instrToHeader + 
								" instructions pointing out of the loop " + instrOutLoop);
					}
					
				}
				if (iterations.size()>0) {
					if (!mn.name.contains("harnesss"))
					new LoopInstrumenter(cn, mn, iterations).instrumentLoop();
				}
				
			
					
			}
			
			if (Configuration.DEBUG || Configuration.BUILD_INSTRUMENTED_CFG) {
				if (Configuration.classToSee.equals(cn.name)) {
					final byte[] nb;
					if (somethingIsChanged) {
						ClassWriter cw = new ClassWriter (0);
						cn.accept (cw);	
						nb =  cw.toByteArray();
					} else {
					System.out.println("passed " + cn.name);
					nb =  classBytes;
					}
				
					try {
						
						ClassNode cni = new ClassNode();
						ClassReader cri = new ClassReader(nb);
						cri.accept(cni, 0);
						BufferedWriter bf = new BufferedWriter(new FileWriter(new File("/Users/zaparanuks/agent/"
								+cni.name.replaceAll("/", "_") + "_graph.dot")));
						new ControlFlowGraphExtractor().build(cni, bf);								
						System.out.println("thread cfg instrumented finished " + cni.name);
					
					} catch (IOException e) {
						System.err.println("Couldn't build CFG due to IO exception");
						e.printStackTrace();
					}	
				}
			}
			if (somethingIsChanged) {
				ClassWriter cw = new ClassWriter (0);
				
				if (Configuration.Verify) {
					ClassVisitor cv = new CheckClassAdapter(cw);
					CheckClassAdapter.verify(new ClassReader(cw.toByteArray()), false, new PrintWriter(System.out));
					cn.accept (cv);	
					
				} else {
					cn.accept (cw);
				}
				
				
				return cw.toByteArray();
			} else {
				return classBytes;
			}
		} catch (Exception e) {
			System.err.println("InstrumentClass failed");
			System.err.println("should never happen, instrument class fails");
			e.printStackTrace();
		};	
		return classBytes;

			
	}
	
	private Block addBlockToLoop (Block currentBlock, Block header, Map<Block, List<Block>> pred) {
		if (currentBlock.equals(header)) {
			allBlocksOfTheLoop.add(header);
			return header;
		}
		for (Block b: pred.get(currentBlock)) {
			if (!allBlocksOfTheLoop.contains(b)) {
				allBlocksOfTheLoop.add(b);
				addBlockToLoop(b, header, pred);
			}
		}
		
		return currentBlock;
	}
	
	
	private Set<Block> allBlocksOfTheLoop;
	private List<Block>[] getAllBlocksOfLoop(BBlockExtractor bb, Map<Block, Block> loops) {
		List<Block> all[] = new List[loops.values().size()];
		
		Map<Block, List<Block>> pred = bb.predescessors;
		
		
		int i = 0;
		for (Block tail: loops.keySet()) {
			Block header = loops.get(tail);
			allBlocksOfTheLoop = new HashSet<Block>();
			allBlocksOfTheLoop.add(tail);
			addBlockToLoop(tail, header, pred);
			all[i] = new ArrayList<Block>(allBlocksOfTheLoop);
			i++;	
		}
		
		if (Configuration.DEBUG || Configuration.DEBUG_TRAVERSE_PREDESCESSORS) {
			int count = 0;
			for (Block block: loops.keySet()) {
				System.out.print("For tail " + block.getInstructions("") + " predescesssors are: ");
				for (Block pr: all[count]) {
					System.out.print(pr.getInstructions("") + ", ");
				}
				System.out.println();
				count++;
			}
		}
		
		return all;
	}
	
	
	private void collectEndCallingContext() {
		
		AbstractInsnNode currentInstr;
		for (int i=0; i<mn.instructions.size(); i++) {
			currentInstr = mn.instructions.get(i);
			/*
			int IRETURN = 172; // visitInsn 
		    int LRETURN = 173; // -
		    int FRETURN = 174; // -
		    int DRETURN = 175; // -
		    int ARETURN = 176; // -
		    int RETURN = 177; 
		    */
			
			if (currentInstr instanceof InsnNode) {
				if (((((InsnNode) currentInstr).getOpcode() > 171) &&
						(((InsnNode) currentInstr).getOpcode() < 178))) {
					
					mn.instructions.insertBefore(currentInstr, new LdcInsnNode (""+(counter-1), -1));
					mn.maxStack++;
					i++;
					mn.instructions.insertBefore(currentInstr, new MethodInsnNode (INVOKESTATIC, "usi/instrumentation/CCT", "removeFromCCT", "(Ljava/lang/String;)V",-1));
					mn.maxStack++;
					i++;
				} else if (((InsnNode) currentInstr).getOpcode() == 191) {
					mn.instructions.insertBefore(currentInstr, new LdcInsnNode ("ex-"+(counter-1),-1));
					mn.maxStack++;
					i++;
					mn.instructions.insertBefore(currentInstr, new MethodInsnNode (INVOKESTATIC, "usi/instrumentation/CCT", "removeFromCCT", "(Ljava/lang/String;)V",-1));
					mn.maxStack++;
					i++;
				}
				
			
			}
		}
		
		if (Configuration.DEBUG || Configuration.DEBUG_REMOVE_METHOD_CCT) {
			System.out.println("DEBUG exit " + 
					cn.name + "." + mn.name);
		}
	}
	
	private void instrumentCallSites() {
		
		List<MethodInsnNode> list = new ArrayList<MethodInsnNode>();
		AbstractInsnNode currentInstr;
		for (int i=0; i<mn.instructions.size(); i++) {
			currentInstr = mn.instructions.get(i);
			
			if (currentInstr instanceof MethodInsnNode) {
				if (Filter.eligible(((MethodInsnNode) currentInstr).owner) || 
						(((MethodInsnNode) currentInstr).owner.contains("Class") &&
								((MethodInsnNode) currentInstr).name.contains("newInstance"))) {
					
					mn.instructions.insert(currentInstr, new MethodInsnNode (INVOKESTATIC, "usi/instrumentation/CCT", "removeFromCCT", "()V",-1));
					mn.maxStack++;
					i++;
					list.add((MethodInsnNode) currentInstr);
					
					if (Configuration.DEBUG || Configuration.DEBUG_REMOVE_METHOD_CCT) {
						System.out.println("DEBUG " + 
								cn.name + "." + mn.name + 
								" instr "+ ((MethodInsnNode)currentInstr).owner + "." + 
								((MethodInsnNode)currentInstr).name);
					}
				}
				
			}
			
		}
		
	}
	
	private String formTypeName (String line) {
		return line.substring(line.indexOf("L")+1, line.indexOf(";"));
	}
	
	private void instrumentGetfields () {
		final InsnList instructions = mn.instructions;
		for (int i=0; i<instructions.size(); i++) {
			final AbstractInsnNode ins = instructions.get(i);
			switch (ins.getOpcode()) {
			case Opcodes.GETFIELD:
				FieldInsnNode field = (FieldInsnNode)ins;
				if (field.desc.contains("[") && !FieldForestDeserializer.dataStructure.containsKey(field.owner+"->"+field.name)) {
					
				} else if (CollectionFilter.contains(field.desc) && !FieldForestDeserializer.dataStructure.containsKey(field.owner+"->"+field.name)) {
					final AbstractInsnNode dup = new InsnNode(Opcodes.DUP, -1);
					mn.instructions.insert(ins, dup);
					mn.maxStack++;
					mn.maxLocals++;
					i++;
					LdcInsnNode fieldName = new LdcInsnNode (field.owner+"->"+field.name, -1);
					mn.instructions.insert(dup, fieldName);
					mn.maxStack++;
					mn.maxLocals++;
					i++;
					MethodInsnNode min = new MethodInsnNode (INVOKESTATIC, "usi/instrumentation/FieldProfiler", "trackCollection", "(Ljava/lang/Object;Ljava/lang/Object;)V",-1);
					mn.instructions.insert(fieldName, min);
					mn.maxStack++;
					mn.maxLocals++;
					i++;
				} else if (field.desc.contains("L")) {
				
					if (FieldForestDeserializer.dataStructure.containsKey(field.owner+"->"+field.name)) {
							final AbstractInsnNode dup = new InsnNode(Opcodes.DUP2, -1);
							mn.instructions.insertBefore(ins, new InsnNode(Opcodes.DUP, -1));
							i++;
							mn.instructions.insert(ins, dup); 
							mn.maxStack++;
							mn.maxLocals++;
							i++;
							LdcInsnNode fieldName = new LdcInsnNode (field.owner+"->"+field.name, -1);
							mn.instructions.insert(dup, fieldName);
							mn.maxStack++;
							mn.maxLocals++;
							i++;
							MethodInsnNode min = new MethodInsnNode (INVOKESTATIC, "usi/instrumentation/FieldProfiler", "trackReference", "(Ljava/lang/Object;Ljava/lang/Object;Ljava/lang/Object;)V",-1);
							mn.instructions.insert(fieldName, min);
							mn.maxStack++;
							mn.maxLocals++;
							i++;
							InsnNode sw = new InsnNode(Opcodes.SWAP, -1);
							mn.instructions.insert(min, sw);
							mn.maxLocals++;
							i++;
							mn.instructions.insert(sw, new InsnNode(Opcodes.POP, -1));							
					}					
				}
				break;
			case Opcodes.AALOAD:
				//pass reference to some method, to check if objects pointed inside are members of a recursive data structure
			case Opcodes.BALOAD: 
			case Opcodes.DALOAD:
			case Opcodes.IALOAD:
			case Opcodes.LALOAD:
			case Opcodes.CALOAD:
			case Opcodes.FALOAD:
			case Opcodes.SALOAD:
				String typeName = "[java.lang.Object";
				if (ins.getOpcode() == Opcodes.AALOAD) {					
				} else if (ins.getOpcode() == Opcodes.CALOAD) {
					typeName = "[java.lang.Character";
				} else if (ins.getOpcode() == Opcodes.BALOAD) {
					typeName = "[java.lang.Boolean";
				} else if (ins.getOpcode() == Opcodes.DALOAD) {
					typeName = "[java.lang.Double";
				} else if (ins.getOpcode() == Opcodes.IALOAD) {
					typeName = "[java.lang.Integer";
				} else if (ins.getOpcode() == Opcodes.LALOAD) {
					typeName = "[java.lang.Long";
				} else if (ins.getOpcode() == Opcodes.FALOAD) {
					typeName = "[java.lang.Float";
				} else if (ins.getOpcode() == Opcodes.SALOAD) {
					typeName = "[java.lang.Short";

				}
				final AbstractInsnNode dup = new InsnNode(Opcodes.DUP2, -1);
				mn.instructions.insertBefore(ins, dup);
				mn.maxLocals++;
				i++;
				LdcInsnNode fieldName = new LdcInsnNode (typeName, -1);
				mn.instructions.insert(dup, fieldName);
				mn.maxStack++;
				mn.maxLocals++;
				i++;
				MethodInsnNode min = new MethodInsnNode (INVOKESTATIC, "usi/instrumentation/FieldProfiler", "trackArray", "(Ljava/lang/Object;ILjava/lang/Object;)V",-1);
				mn.instructions.insert(fieldName, min);
				mn.maxStack++;
				mn.maxLocals++;
				i++;
				break;
			} 
		}

	}
	
	
	private void instrumentPutfields () {
		final InsnList instructions = mn.instructions;
		for (int i=0; i<instructions.size(); i++) {
			final AbstractInsnNode ins = instructions.get(i);
			switch (ins.getOpcode()) {
			case Opcodes.PUTFIELD:
				FieldInsnNode field = (FieldInsnNode)ins;
				if (field.desc.contains("[") && !FieldForestDeserializer.dataStructure.containsKey(field.owner+"->"+field.name)) {
					//put instrumentation for arrays
					/*final AbstractInsnNode dup = new InsnNode(Opcodes.DUP, -1);
					mn.instructions.insertBefore(ins, dup);
					i++;
					//final AbstractInsnNode pop = new InsnNode(Opcodes.POP, -1);
					//mn.instructions.insert(dup, pop);
					//i++;
					LdcInsnNode fieldName = new LdcInsnNode ("["+field.owner+"->"+field.name, -1);
					mn.instructions.insert(dup, fieldName);
					mn.maxStack++;
					i++;
					MethodInsnNode min = new MethodInsnNode (INVOKESTATIC, "usi/instrumentation/FieldProfiler", "trackArray", "(Ljava/lang/Object;Ljava/lang/Object;)V",-1);
					mn.instructions.insert(fieldName, min);
					mn.maxStack++;
					i++;*/
				} else if (CollectionFilter.contains(field.desc) && !FieldForestDeserializer.dataStructure.containsKey(field.owner+"->"+field.name)) {
					final AbstractInsnNode dup = new InsnNode(Opcodes.DUP, -1);
					mn.instructions.insertBefore(ins, dup);
					mn.maxLocals++;
					i++;
					LdcInsnNode fieldName = new LdcInsnNode (field.owner+"->"+field.name, -1);
					mn.instructions.insert(dup, fieldName);
					mn.maxLocals++;
					mn.maxStack++;
					i++;
					MethodInsnNode min = new MethodInsnNode (INVOKESTATIC, "usi/instrumentation/FieldProfiler", "trackCollection", "(Ljava/lang/Object;Ljava/lang/Object;)V",-1);
					mn.instructions.insert(fieldName, min);
					mn.maxLocals++;
					mn.maxStack++;
					i++;
				} else if (field.desc.contains("L")) {
					if (FieldForestDeserializer.dataStructure.containsKey(field.owner+"->"+field.name)) {
							final AbstractInsnNode dup = new InsnNode(Opcodes.DUP2, -1);
							mn.instructions.insertBefore(ins, dup); 
							mn.maxStack++;
							mn.maxLocals++;
							i++;
							LdcInsnNode fieldName = new LdcInsnNode (field.owner+"->"+field.name, -1);
							mn.instructions.insert(dup, fieldName);
							mn.maxStack++;
							mn.maxLocals++;
							i++;
							MethodInsnNode min = new MethodInsnNode (INVOKESTATIC, "usi/instrumentation/FieldProfiler", "trackReference", "(Ljava/lang/Object;Ljava/lang/Object;Ljava/lang/Object;)V",-1);
							mn.instructions.insert(fieldName, min);
							mn.maxStack++;
							mn.maxLocals++;
							i++;
							
					}					
				}
				break;
			case Opcodes.AASTORE:			
			case Opcodes.BASTORE: 
			case Opcodes.DASTORE:
			case Opcodes.IASTORE:
			case Opcodes.LASTORE:
			case Opcodes.CASTORE:
			case Opcodes.FASTORE:
			case Opcodes.SASTORE:
				
				
				mn.maxLocals+=2;
				VarInsnNode vin = null;
				VarInsnNode vinLoad = null;
				String typeName = "[java.lang.Object";
				if (ins.getOpcode() == Opcodes.AASTORE) {
					vin = new VarInsnNode(Opcodes.ASTORE, mn.maxLocals-1, -1);
					vinLoad = new VarInsnNode(Opcodes.ALOAD, mn.maxLocals-1, -1);
				} else if (ins.getOpcode() == Opcodes.BASTORE) {
					vin = new VarInsnNode(Opcodes.ISTORE, mn.maxLocals-1, -1);
					vinLoad = new VarInsnNode(Opcodes.ILOAD, mn.maxLocals-1, -1);
				} else if (ins.getOpcode() == Opcodes.DASTORE) {
					vin = new VarInsnNode(Opcodes.DSTORE, mn.maxLocals-1, -1);
					vinLoad = new VarInsnNode(Opcodes.DLOAD, mn.maxLocals-1, -1);
				} else if (ins.getOpcode() == Opcodes.IASTORE) {
					vin = new VarInsnNode(Opcodes.ISTORE, mn.maxLocals-1, -1);
					vinLoad = new VarInsnNode(Opcodes.ILOAD, mn.maxLocals-1, -1);
				} else if (ins.getOpcode() == Opcodes.LASTORE) {
					vin = new VarInsnNode(Opcodes.LSTORE, mn.maxLocals-1, -1);
					vinLoad = new VarInsnNode(Opcodes.LLOAD, mn.maxLocals-1, -1);
				} else if (ins.getOpcode() == Opcodes.CASTORE) {
					vin = new VarInsnNode(Opcodes.ISTORE, mn.maxLocals-1, -1);
					vinLoad = new VarInsnNode(Opcodes.ILOAD, mn.maxLocals-1, -1);
				} else if (ins.getOpcode() == Opcodes.FASTORE) {
					vin = new VarInsnNode(Opcodes.FSTORE, mn.maxLocals-1, -1);
					vinLoad = new VarInsnNode(Opcodes.FLOAD, mn.maxLocals-1, -1);
				} else if (ins.getOpcode() == Opcodes.SASTORE) {
					vin = new VarInsnNode(Opcodes.ISTORE, mn.maxLocals-1, -1);
					vinLoad = new VarInsnNode(Opcodes.ILOAD, mn.maxLocals-1, -1);
				}
				mn.instructions.insertBefore(ins, vin);
				mn.maxStack++;
				i++;
				
				AbstractInsnNode dup = new InsnNode(Opcodes.DUP2, -1);
				mn.maxLocals++;
				mn.instructions.insert(vin, dup);
				i++;
				LdcInsnNode fieldName = new LdcInsnNode (typeName, -1);
				mn.instructions.insert(dup, fieldName);
				mn.maxLocals++;
				mn.maxStack++;
				i++;
				MethodInsnNode min = new MethodInsnNode (INVOKESTATIC, "usi/instrumentation/FieldProfiler", "trackArray", "(Ljava/lang/Object;ILjava/lang/Object;)V",-1);
				mn.instructions.insert(fieldName, min);
				mn.maxLocals++;
				mn.maxStack++;
				i++;
				mn.instructions.insert(min, vinLoad);
				mn.maxLocals++;
				mn.maxStack++;
				i++;
				
				break;
				} 
		}

	}
	private void instrumentNews () {
		
		final InsnList instructions = mn.instructions;
		for (int i=0; i<instructions.size(); i++) {
			final AbstractInsnNode ins = instructions.get(i);
			switch (ins.getOpcode()) {
			case Opcodes.NEW:
				TypeInsnNode field = (TypeInsnNode)ins;
				
			
				LdcInsnNode fieldName = new LdcInsnNode (field.desc, -1);
				mn.instructions.insert(ins, fieldName);
				mn.maxLocals++;
				mn.maxStack++;
				i++;
				MethodInsnNode min = new MethodInsnNode (INVOKESTATIC, "usi/instrumentation/FieldProfiler", "trackNews", "(Ljava/lang/Object;)V",-1);
				mn.instructions.insert(fieldName, min);
				mn.maxLocals++;
				mn.maxStack++;
				i++;
				
			} 
		}


		
	}
	
	private void collectCallingContext() {
		LdcInsnNode number = new LdcInsnNode (""+counter,-1);
		mn.maxStack++;
		mn.instructions.insert(number);
		mn.maxStack++;
		mn.instructions.insert(number, new MethodInsnNode (INVOKESTATIC, "usi/instrumentation/CCT", "addToCCT", "(Ljava/lang/Object;)V",-1));
		
		if (Configuration.DEBUG || Configuration.DEBUG_REMOVE_METHOD_CCT) {
			System.out.println("DEBUG entry " + 
					cn.name + "." + mn.name);
		}
	}
	
	
	public void instrumentLoop (List<Loop> iterations) {
		
		AbstractInsnNode header; 
		List<AbstractInsnNode> inTails, outTails;
		List<AbstractInsnNode> p;
		Map<AbstractInsnNode,InstructionPointer> m;
		
		for (Loop loop: iterations) {
			
			header = loop.header;
			inTails = loop.inTails;
			outTails = loop.outTails;
			p = loop.instructionsPointingToHeader;
			m = loop.instructionsPointingOutOfTheLoop;
			Map<AbstractInsnNode, AbstractInsnNode> repetitionTracker = new HashMap<AbstractInsnNode, AbstractInsnNode>();
		
			int[] numbers = loop.numbers;
			
			LdcInsnNode preHeader = new LdcInsnNode (cn.name + "/" + mn.name + "/" + numbers[0]+"-"+numbers[1],-1);
			
			for (AbstractInsnNode ain: m.keySet()) {
				if (m.get(ain) == null) {
						mn.maxStack++;
						mn.instructions.insertBefore(ain, new LdcInsnNode (cn.name + "/" + mn.name + "/" + numbers[0]+"-"+numbers[1],-1));
						mn.maxStack++;
						mn.instructions.insertBefore(ain, new MethodInsnNode (INVOKESTATIC, "usi/instrumentation/LoopProfiler", "countIF_LOOP_After", "(Ljava/lang/Object;)V",-1));

						
					
				
				} else {
						
						MethodInsnNode min = new MethodInsnNode 
						(INVOKESTATIC, "usi/instrumentation/LoopProfiler", "countIF_LOOP_After", "(Ljava/lang/Object;)V",-1);
						
						if (ain instanceof JumpInsnNode) {
							if (m.get(ain).ain instanceof LabelNode)  {
								if (m.get(ain).edge.v.equals("T")) {
									
									if (Configuration.DEBUG_LOOP_AFTER) {
										System.out.println("true edge points out of the loop");
										System.out.println("Loop " + loop.numbers[0] + 
												"-" + loop.numbers[1] + " instruction " + OpcodeReader.giveName(ain) + 
												" " + mn.instructions.indexOf(ain) + " points to " + 
												OpcodeReader.giveName(m.get(ain).ain) + " " + 
												mn.instructions.indexOf(m.get(ain).ain));
									}
									
									if (!repetitionTracker.containsKey(m.get(ain).ain)) {
										LabelNode oLabel = new LabelNode();
										mn.instructions.insertBefore(((JumpInsnNode) ain).label, oLabel);
										mn.maxStack++;
										mn.instructions.insert(oLabel, min);
										mn.maxStack++;
										mn.instructions.insert(oLabel, new LdcInsnNode (cn.name + "/" + mn.name + "/" + numbers[0]+"-"+numbers[1],-1));
										mn.maxStack++;
										((JumpInsnNode) ain).label = oLabel;
										repetitionTracker.put(m.get(ain).ain, oLabel);										
									} else {
										((JumpInsnNode) ain).label = (LabelNode)repetitionTracker.get(m.get(ain).ain);
									}
								} else {
									
									if (Configuration.DEBUG_LOOP_AFTER) {
										System.out.println("false edge points out of the loop");
										System.out.println("Loop " + loop.numbers[0] + 
												"-" + loop.numbers[1] + " instruction " + OpcodeReader.giveName(ain) + 
												" " + mn.instructions.indexOf(ain) + " points to " + 
												OpcodeReader.giveName(m.get(ain).ain) + " " + 
												mn.instructions.indexOf(m.get(ain).ain));
									}
									if (!repetitionTracker.containsKey(m.get(ain).ain)) {
										mn.maxStack++;
										mn.instructions.insert(ain, min);
										mn.maxStack++;
										LdcInsnNode ldc = new LdcInsnNode (cn.name + "/" + mn.name + "/" + numbers[0]+"-"+numbers[1],-1);
										mn.instructions.insert(ain, ldc);
										
										LabelNode oLabel = new LabelNode();
										mn.instructions.insert(ain, oLabel);
										
										repetitionTracker.put(m.get(ain).ain, oLabel);
										
										Loop loopToUpdate = null;
										AbstractInsnNode tailToUpdate = null;
										boolean whichTail = true;
										for (Loop checkingLoop: iterations) {
											if (!checkingLoop.equals(loop)) {
												for (AbstractInsnNode tailAin: checkingLoop.inTails) {
													if (tailAin.equals(ain)) {
														loopToUpdate = checkingLoop;
														tailToUpdate = tailAin;
														break;
													}
												}
												for (AbstractInsnNode tailAin: checkingLoop.outTails) {
													if (tailAin.equals(ain)) {
														loopToUpdate = checkingLoop;
														tailToUpdate = tailAin;
														whichTail = false;
														break;
													}
												}
											}
										}
										if (loopToUpdate!=null) {
											if (whichTail) {
												loopToUpdate.inTails.remove(tailToUpdate);
												loopToUpdate.inTails.add(min);
											} else {
												loopToUpdate.outTails.remove(tailToUpdate);
												loopToUpdate.outTails.add(min);
											}
										}
									} else {
										/* .... */
									}
																				
									//}
									//JumpInsnNode jin = new JumpInsnNode(GOTO, ((JumpInsnNode) ain).label);
									//mn.instructions.insert(min, jin);
									//mn.instructions.insertBefore(min, jin);
									//((JumpInsnNode) ain).label = oLabel;
									//((LabelNode)m.get(ain).ain).resetLabel();
								}
							} else {
								if (Configuration.DEBUG_LOOP_AFTER) {
									System.out.println("not instance of label node we point out of loop");
									System.out.println("Loop " + loop.numbers[0] + 
											"-" + loop.numbers[1] + " instruction " + OpcodeReader.giveName(ain) + 
											" " + mn.instructions.indexOf(ain) + " points to " + 
											OpcodeReader.giveName(m.get(ain).ain) + " " + 
											mn.instructions.indexOf(m.get(ain).ain));
								}
								mn.maxStack++;
								mn.instructions.insert(ain, min);
								mn.maxStack++;
								mn.instructions.insert(ain, new LdcInsnNode (cn.name + "/" + mn.name + "/" + numbers[0]+"-"+numbers[1],-1));
								
							}
								
							
							
						} else {
							
							if (Configuration.DEBUG_LOOP_AFTER) {
								System.out.println("not jump node points out of loop, can't happen by logic");
								System.out.println("Loop " + loop.numbers[0] + 
										"-" + loop.numbers[1] + " instruction " + OpcodeReader.giveName(ain) + 
										" " + mn.instructions.indexOf(ain) + " points to " + 
										OpcodeReader.giveName(m.get(ain).ain) + " " + 
										mn.instructions.indexOf(m.get(ain).ain));
							}
							mn.maxStack++;
							mn.instructions.insert(ain, min);
							mn.maxStack++;
							mn.instructions.insert(ain, new LdcInsnNode (cn.name + "/" + mn.name + "/" + numbers[0]+"-"+numbers[1],-1));
						}
							
					
				}
				
			}
			
			mn.maxStack++;
			mn.instructions.insertBefore(header, preHeader);
			
			mn.maxStack++;
			mn.instructions.insertBefore(header, new MethodInsnNode (INVOKESTATIC, "usi/instrumentation/LoopProfiler", "countIF_LOOP_Before", "(Ljava/lang/Object;)V",-1));				
			
			for (AbstractInsnNode tail: inTails) {
				
				if (tail instanceof JumpInsnNode) {
					
					if (((JumpInsnNode) tail).label.equals((LabelNode) header)) {
						
						LabelNode ln = new LabelNode();
						mn.instructions.insert(tail, ln);
						JumpInsnNode jump = new JumpInsnNode(GOTO, ln, -1);
						mn.instructions.insert(tail, jump);
						
						LabelNode ln2 = new LabelNode();
						mn.instructions.insert(jump, ln2);
						((JumpInsnNode) tail).label = ln2;
						
						mn.maxStack++;
						mn.instructions.insert(ln2, new MethodInsnNode (INVOKESTATIC, "usi/instrumentation/LoopProfiler", "countIF_LOOP", "(Ljava/lang/Object;)V",-1));
						
						mn.maxStack++;
						mn.instructions.insert(ln2, new LdcInsnNode (cn.name + "/" + mn.name + "/" + numbers[0]+"-"+numbers[1],-1));
						
						JumpInsnNode jump2 = new JumpInsnNode(GOTO, (LabelNode) header,-1 );
						mn.maxStack++;
						mn.instructions.insertBefore(ln, jump2);
						
						
						
					} else {
						
						JumpInsnNode jump = new JumpInsnNode(GOTO, (LabelNode) header,-1 );
						mn.maxStack++;
						mn.instructions.insert(tail, jump);
						
						mn.maxStack++;
						mn.instructions.insert(tail, new MethodInsnNode (INVOKESTATIC, "usi/instrumentation/LoopProfiler", "countIF_LOOP", "(Ljava/lang/Object;)V", -1));
						
						mn.maxStack++;
						mn.instructions.insert(tail, new LdcInsnNode (cn.name + "/" + mn.name + "/" + numbers[0]+"-"+numbers[1], -1));
						
					}
					
					
					
					} else {
						JumpInsnNode jump = new JumpInsnNode(GOTO, (LabelNode) header, -1 );
						mn.maxStack++;
						mn.instructions.insert(tail, jump);
						
						mn.maxStack++;
						mn.instructions.insert(tail, new MethodInsnNode (INVOKESTATIC, "usi/instrumentation/LoopProfiler", "countIF_LOOP", "(Ljava/lang/Object;)V",-1));
						
						mn.maxStack++;
						mn.instructions.insert(tail, new LdcInsnNode (cn.name + "/" + mn.name + "/" + numbers[0]+"-"+numbers[1],-1));
						
					}
				} 
			
			for (AbstractInsnNode tail: outTails) {
				if (tail instanceof JumpInsnNode) {
					
					if (((JumpInsnNode) tail).label.equals((LabelNode) header)) {
						
						LabelNode ln = new LabelNode();
						mn.instructions.insert(tail, ln);
						JumpInsnNode jump = new JumpInsnNode(GOTO, ln, -1);
						mn.instructions.insert(tail, jump);
						
						LabelNode ln2 = new LabelNode();
						mn.instructions.insert(jump, ln2);
						((JumpInsnNode) tail).label = ln2;
						
						mn.maxStack++;
						mn.instructions.insert(ln2, new MethodInsnNode (INVOKESTATIC, "usi/instrumentation/LoopRofiler", "countIF_LOOP", "(Ljava/lang/Object;)V",-1));
						
						mn.maxStack++;
						mn.instructions.insert(ln2, new LdcInsnNode (cn.name + "/" + mn.name + "/" + numbers[0]+"-"+numbers[1],-1));
						
						JumpInsnNode jump2 = new JumpInsnNode(GOTO, (LabelNode) header, -1 );
						mn.maxStack++;
						mn.instructions.insertBefore(ln, jump2);
						
						
						
					} else {
						
						JumpInsnNode jump = new JumpInsnNode(GOTO, (LabelNode) header,-1 );
						mn.maxStack++;
						mn.instructions.insert(tail, jump);
						
						mn.maxStack++;
						mn.instructions.insert(tail, new MethodInsnNode (INVOKESTATIC, "usi/instrumentation/LoopRofiler", "countIF_LOOP", "(Ljava/lang/Object;)V",-1));
						
						mn.maxStack++;
						mn.instructions.insert(tail, new LdcInsnNode (cn.name + "/" + mn.name + "/" + numbers[0]+"-"+numbers[1],-1));
						
					}
					
					
					
					} else {
						JumpInsnNode jump = new JumpInsnNode(GOTO, (LabelNode) header,-1 );
						mn.maxStack++;
						mn.instructions.insert(tail, jump);
						
						mn.maxStack++;
						mn.instructions.insert(tail, new MethodInsnNode (INVOKESTATIC, "usi/instrumentation/LoopRofiler", "countIF_LOOP", "(Ljava/lang/Object;)V",-1));
						
						mn.maxStack++;
						mn.instructions.insert(tail, new LdcInsnNode (cn.name + "/" + mn.name + "/" + numbers[0]+"-"+numbers[1],-1));
						
					}
			

			}
			
			LabelNode label2 = new LabelNode();
			mn.maxStack++;
			mn.instructions.insertBefore(preHeader, label2);
			for (AbstractInsnNode node: p) {
				if (node instanceof JumpInsnNode) {
					
					if (((JumpInsnNode) node).label.equals((LabelNode)header)) {
							((JumpInsnNode) node).label = label2;
					}
				}
			}
		}
	}
	
	
}
