package usi.instrumentation;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;

import javax.print.attribute.standard.MediaSize.Other;

import org.objectweb.asm.Opcodes;
import org.objectweb.asm.tree.AbstractInsnNode;
import org.objectweb.asm.tree.ClassNode;
import org.objectweb.asm.tree.JumpInsnNode;
import org.objectweb.asm.tree.LabelNode;
import org.objectweb.asm.tree.LookupSwitchInsnNode;
import org.objectweb.asm.tree.MethodNode;
import org.objectweb.asm.tree.TableSwitchInsnNode;

import usi.instrumentation.tracker.BytecodeInstructionTracker;

import ch.unisi.inf.sp.cfg.Block;
import ch.unisi.inf.sp.cfg.Edge;
import ch.unisi.inf.sp.cfg.InstructionData;
import ch.unisi.inf.sp.cfg.InstructionPointer;
import ch.unisi.inf.sp.cfg.Loop;

public class LoopBuilder implements Opcodes {
	
	private Map<Block, Block> loops;
	private MethodNode mn; 
	private ClassNode cn;
	private BBlockExtractor bb;
	private List<Block>[] allB;
	
	public LoopBuilder(Map<Block, Block> loops, MethodNode mn, ClassNode cn, BBlockExtractor bb, List<Block>[] allB) {
		this.loops = loops;
		this.mn = mn;
		this.bb = bb;
		this.allB = allB;
		this.cn = cn;
	}
	
	private List<AbstractInsnNode> getPointerInstructions(AbstractInsnNode tail, Block b) {
		List<AbstractInsnNode> pointerInstr = new ArrayList<AbstractInsnNode>();
		List<Block> pointers = bb.predescessors.get(b);
		for (Block pointer: pointers) {
			if (pointer.getBlockNumber()!=-1) {
				AbstractInsnNode po = mn.instructions.get (pointer.getBlockNumber()+pointer.getInstrs().size()-1);
				if (!po.equals(tail)) {
					if (!pointerInstr.contains(po)) {
						pointerInstr.add(po);						
					}
					
					
				}	
			}
		}
		return pointerInstr;
	
	}
	
	private List<Block> getAllBlocksOfLoop (int counter) {
		
		return allB[counter];
	}
	
	private InstructionPointer tailpointsOutOfLoop (Block block, Block b, int[] numbers ) {
		for (Edge edge: block.getEdges()) {
			if ((edge.t.intValue() != b.getBlockNumber()) && (!edge.v.contains("ex"))) {
				boolean found = false;
				for (int i = 0; i<numbers.length; i++) {
					if (edge.t.intValue() == numbers[i]) {
						found = true;
						break;
					}
				}
				if (!found) {
					return new InstructionPointer(mn.instructions.get(edge.t.intValue()), edge);
				}
			}
		}
		return null;
	}
	
	private ConcurrentMap<AbstractInsnNode, InstructionPointer> getBlocksPointingOutOfALoop (List<Block> otherBlocks, Block b, AbstractInsnNode tail, Block block) {
		int[] numbers = new int[otherBlocks.size()];
		for (int i = 0; i<otherBlocks.size(); i++) {
			numbers[i] = otherBlocks.get(i).getBlockNumber(); 
		}
		ConcurrentMap<AbstractInsnNode, InstructionPointer> otherBlocksPointingOutOfLoop = 
			new ConcurrentHashMap<AbstractInsnNode, InstructionPointer>();

		InstructionPointer ip = tailpointsOutOfLoop(block, b, numbers);
		if (ip!=null) {
			otherBlocksPointingOutOfLoop.put(tail, ip);		
		}
		for (Block oBlock: otherBlocks) {
			InstructionData id = oBlock.getInstrs().get(oBlock.getInstrs().size()-1);
			if (id.isReturn()) {
					otherBlocksPointingOutOfLoop.put(mn.instructions.get(oBlock.getBlockNumber() + oBlock.getInstrs().size() -1), new InstructionPointer(null, null));
			} else if (id.isPEI()) {
				if (Configuration.DEBUG || Configuration.DEBUG_INSTRUMENTING_EXCEPTION_EDGES) {
					System.out.println("exception " + (oBlock.getBlockNumber() + oBlock.getInstrs().size() -1));
				}
				
				for (Edge edge: oBlock.getEdges()) {
					boolean found = false;
					for (int i = 0; i<numbers.length; i++) {
						if (edge.t.intValue() == numbers[i]) {
							found = true;
							break;
						}
					}
					if (!found && !edge.v.contains("ex")) {
						
						otherBlocksPointingOutOfLoop.put(mn.instructions.get(oBlock.getBlockNumber()+oBlock.getInstrs().size()-1),
								new InstructionPointer(mn.instructions.get(edge.t), edge));
					}
				}
			} else {
				if (mn.instructions.get(oBlock.getBlockNumber() + oBlock.getInstrs().size() -1) instanceof JumpInsnNode)  {
					
					for (Edge edge: oBlock.getEdges()) {
						boolean found = false;
						for (int i = 0; i<numbers.length; i++) {
							if (edge.t.intValue() == numbers[i]) {
								found = true;
								break;
							}
						}
						if (!found) {
							otherBlocksPointingOutOfLoop.put(mn.instructions.get(oBlock.getBlockNumber()+oBlock.getInstrs().size()-1),
									new InstructionPointer(mn.instructions.get(edge.t), edge));
						}
					}		
				} else if (mn.instructions.get(oBlock.getBlockNumber() + oBlock.getInstrs().size() -1) instanceof TableSwitchInsnNode ||
						mn.instructions.get(oBlock.getBlockNumber() + oBlock.getInstrs().size() -1) instanceof LookupSwitchInsnNode) {
					
						for (Edge edge: oBlock.getEdges()) {
							boolean found = false;
							for (int i = 0; i<numbers.length; i++) {
								if (edge.t.intValue() == numbers[i]) {
									found = true;
									break;
								}
							}
							if (!found) {
								otherBlocksPointingOutOfLoop.put(mn.instructions.get(oBlock.getBlockNumber()+oBlock.getInstrs().size()-1),
										new InstructionPointer(mn.instructions.get(edge.t), edge));
							}
						}
				} else {
					if (mn.name.contains("checkParents")) {
						System.out.println("aaaa checkParents " + OpcodeReader.giveName(mn.instructions.get(oBlock.getBlockNumber()+oBlock.getInstrs().size()-1)));
					}
					for (Edge edge: oBlock.getEdges()) {
						boolean found = false;
						for (int i = 0; i<numbers.length; i++) {
							if (edge.t.intValue() == numbers[i]) {
								found = true;
								break;
							}
						}
						if (!found) {
							
							otherBlocksPointingOutOfLoop.put(mn.instructions.get(oBlock.getBlockNumber()+oBlock.getInstrs().size()-1),
									new InstructionPointer(mn.instructions.get(edge.t), edge));
						}
					}
				}
			}
		}
		return otherBlocksPointingOutOfLoop;
	}
	
	public List<Loop> build () {
			
		Map<AbstractInsnNode, List<Block>> allBlocksPerLoop = new HashMap<AbstractInsnNode, List<Block>>();
		AbstractInsnNode tail, header;
			
		List<Loop> iterations = new ArrayList<Loop>();
		int counter = 0;
		for(Block block: loops.keySet()) {
			
			tail = mn.instructions.get (block.getBlockNumber()+block.getInstrs().size()-1);
			Block b = loops.get(block);
			header = mn.instructions.get (b.getBlockNumber());
			List<AbstractInsnNode> pointerInstr = getPointerInstructions(tail, b);
			List<Block> otherBlocks = getAllBlocksOfLoop(counter);
			counter++;
			allBlocksPerLoop.put(tail, otherBlocks);
			
			if (Configuration.DEBUG || Configuration.TRACK_INDEXES) {
				
				BytecodeInstructionTracker.writeIndexes(cn.name + "/" + mn.name + mn.desc + "/" + 
						b.getBlockNumber() + "-" + block.getBlockNumber(), otherBlocks);			
			}
			
			
			ConcurrentMap<AbstractInsnNode, InstructionPointer> otherBlocksPointingOutOfLoop = 
				getBlocksPointingOutOfALoop(otherBlocks, b, tail, block);

			
			List<AbstractInsnNode> outTails = new ArrayList<AbstractInsnNode>();
			List<AbstractInsnNode> inTails = new ArrayList<AbstractInsnNode>();
			outTails.add(tail);
			
			Loop newLoop = new Loop(header, outTails, inTails, pointerInstr, otherBlocksPointingOutOfLoop);
			newLoop.numbers[0] = b.getBlockNumber();
			newLoop.numbers[1] = block.getBlockNumber();
			iterations.add(newLoop);
		}
		return finalizeTails(mergeLoops(iterations, allBlocksPerLoop));
		
	}
	
	private List<Loop> finalizeTails (List<Loop> iterations) {
		for (Loop loop: iterations) {
			if (loop.inTails.isEmpty() && (loop.outTails.size() == 1)) {
				if (loop.outTails.get(0) instanceof JumpInsnNode) {
					if (((JumpInsnNode)loop.outTails.get(0)).getOpcode() == GOTO) {
						loop.inTails.add(loop.outTails.get(0));
						loop.outTails.remove(0);
					}
					continue;
				}
				loop.inTails.add(loop.outTails.get(0));
				loop.outTails.remove(0);
			}
			
			
		}
		return iterations;
	}
	
	private Map <AbstractInsnNode, Set <Loop>> memorizeLoopsWithSameHeader (List<Loop> iterations) {
		
		Map <AbstractInsnNode, Set <Loop>> sameLoops = new HashMap<AbstractInsnNode, Set <Loop>>();
		
		for (Loop loop: iterations) {
			AbstractInsnNode head = loop.header;
			for (Loop ol: iterations) {
				if (!ol.equals(loop)) {
					if (head.equals(ol.header)) {
						if (sameLoops.containsKey(head)) {
							Set <Loop> container = sameLoops.get(head);
							container.add(loop);
							container.add(ol);
							sameLoops.put(head, container);
						} else {
							Set<Loop> container = new HashSet <Loop> ();
							container.add(ol);
							container.add(loop);
							sameLoops.put(head, container);
						}
						
						List <AbstractInsnNode> intersectionHeaders = new ArrayList<AbstractInsnNode> ();
						for (AbstractInsnNode beforeHeader: ol.instructionsPointingToHeader) {
							if (loop.instructionsPointingToHeader.contains(beforeHeader)) {
								intersectionHeaders.add(beforeHeader);
							} 
						}
						loop.instructionsPointingToHeader = intersectionHeaders;						
					}
				}
			
			}
		}
		return sameLoops;
	}
	
	private void arrangeTails (Set<Loop> sameLoops, Map<AbstractInsnNode, List<Block>> allBlocksPerLoop, AbstractInsnNode head) {
		for (Loop loop: sameLoops) {
			ConcurrentMap<AbstractInsnNode, InstructionPointer> ipol = loop.instructionsPointingOutOfTheLoop;
				AbstractInsnNode loopTail;
				if (!loop.outTails.isEmpty()) {
					loopTail = loop.outTails.get(0);
				} else {
					loopTail = loop.inTails.get(0);
				}
				List<Block> loopBlocks = allBlocksPerLoop.get(loopTail);
				
				AbstractInsnNode otherTail;
				List<Block> otherBlocks;
				
				for (Loop other: sameLoops) {
					if (!loop.equals(other)) {
						if (!other.outTails.isEmpty()) {
							otherTail = other.outTails.get(0);
						} else {
							continue;
						}
						otherBlocks = allBlocksPerLoop.get(otherTail);
						if (otherTail instanceof JumpInsnNode) {
							LabelNode label = ((JumpInsnNode) otherTail).label;
							AbstractInsnNode nextInstr;
							if (label.equals(head)) {
								if (((JumpInsnNode) otherTail).getOpcode() == GOTO) {
									nextInstr = null;
								} else {
									nextInstr = otherTail.getNext();									
								}
								
							} else {
								nextInstr = label;
							}
							
							for (Block ob: loopBlocks) {
								if (nextInstr == null) {
									other.inTails.add(otherTail);
									other.outTails.remove(otherTail);
									break;
									
								} else {
									if (mn.instructions.get(ob.getBlockNumber()).equals(nextInstr)) {
										other.inTails.add(otherTail);
										other.outTails.remove(otherTail);
										break;
										
									}									
								}
							}
							
						} else {
							other.inTails.add(otherTail);
							other.outTails.remove(otherTail);
							
						}	
				}
			}
				
		}
	}
	
	private void arrangeInstructionsPointingOutOfLoop (Set<Loop> sameLoops, Map<AbstractInsnNode, List<Block>> allBlocksPerLoop) {
		
		for (Loop loop: sameLoops) {
			ConcurrentMap<AbstractInsnNode, InstructionPointer> ipol = loop.instructionsPointingOutOfTheLoop;
				AbstractInsnNode otherTail;
				
				for (Loop other: sameLoops) {
					if (!loop.equals(other)) {
						if (!other.outTails.isEmpty()) {
							otherTail = other.outTails.get(0);
						} else {
							otherTail = other.inTails.get(0);
						}
						List<Block> otherBlocks = allBlocksPerLoop.get(otherTail);
						for (Block ob: otherBlocks) {
							for (AbstractInsnNode inst: ipol.keySet()) {
								InstructionPointer insp = ipol.get(inst);
								if (insp.ain.equals(mn.instructions.get(ob.getBlockNumber()))) {
									loop.instructionsPointingOutOfTheLoop.remove(inst);
								}
							}
						}
					}
				}
		}
	}
	
	private void arrangeInstructionsPointingToHeader (Set<Loop> sameLoops, Map<AbstractInsnNode, List<Block>> allBlocksPerLoop) {
		
		for (Loop loop: sameLoops) {
			List<AbstractInsnNode> ipth = loop.instructionsPointingToHeader;
				AbstractInsnNode otherTail;
				
				for (Loop other: sameLoops) {
					if (!loop.equals(other)) {
						if (!other.outTails.isEmpty()) {
							otherTail = other.outTails.get(0);
						} else {
							otherTail = other.inTails.get(0);
						}
						if (ipth.contains(otherTail)) {
							ipth.remove(otherTail);
						}
	
					}
				}
		}

		
	}

	private List<Loop> mergeLoops (List<Loop> iterations, Map<AbstractInsnNode, List<Block>> allBlocksPerLoop) {
	Map <AbstractInsnNode, Set <Loop>> sameLoops = memorizeLoopsWithSameHeader(iterations);
	
	List<Loop> toPreserve = new ArrayList<Loop>();
	for (Loop loop : iterations) {
		boolean isUnique = true;
		for (AbstractInsnNode header: sameLoops.keySet()) {
			if (loop.header.equals(header)) {
				isUnique = false;
				break;
			}
		}
		if (isUnique) {
			toPreserve.add(loop);
		}
	}
	
	if (Configuration.DEBUG || Configuration.DEBUG_LOOP_BEFORE_MERGING) {
		for (AbstractInsnNode head: sameLoops.keySet()) {
			for (Loop loop: sameLoops.get(head)) {
				System.out.print("loop " + loop.numbers[1] + "-" + loop.numbers[0]  
				                            + " has following blocks pointing out: ");
				for (AbstractInsnNode ain: loop.instructionsPointingOutOfTheLoop.keySet()) {
					System.out.print(OpcodeReader.giveName(ain) + " " + loop.instructionsPointingOutOfTheLoop.get(ain).edge.v + ", ");
				}
				System.out.println();
			}
		}			
	}
	
	for (AbstractInsnNode head: sameLoops.keySet()) {
		arrangeTails(sameLoops.get(head), allBlocksPerLoop, head);
		arrangeInstructionsPointingOutOfLoop(sameLoops.get(head), allBlocksPerLoop);
		ConcurrentMap <AbstractInsnNode, InstructionPointer> otherInstructionPointingOutOfTheLoop = new ConcurrentHashMap<AbstractInsnNode, InstructionPointer>();
		List <AbstractInsnNode> inTails = new ArrayList<AbstractInsnNode>();
		List <AbstractInsnNode> outTails = new ArrayList<AbstractInsnNode>();
		
		
		for (Loop loop: sameLoops.get(head)) {
			otherInstructionPointingOutOfTheLoop.putAll(loop.instructionsPointingOutOfTheLoop);
			inTails.addAll(loop.inTails);
			outTails.addAll(loop.outTails);
		}
		
		
		
		for (Loop loop: sameLoops.get(head)) {
			loop.instructionsPointingOutOfTheLoop = otherInstructionPointingOutOfTheLoop;
			loop.inTails = inTails;
			loop.outTails = outTails;
			toPreserve.add(loop);
			break;
		}
	}
	
	if (Configuration.DEBUG_SPECIAL_INSTRUCTIONS_IN_LOOP) {
		
		for (Loop loop: toPreserve) {
			System.out.print("Loop " + loop.numbers[0] + "-" + loop.numbers[1] + 
			" contains instruction pointing out of the loop ");
			for (AbstractInsnNode ain: loop.instructionsPointingOutOfTheLoop.keySet()) {
				AbstractInsnNode toAin = loop.instructionsPointingOutOfTheLoop.get(ain).ain;
				System.out.print(OpcodeReader.giveName(ain) + " " + mn.instructions.indexOf(ain) + "==> " +
						OpcodeReader.giveName(toAin) + " " + mn.instructions.indexOf(toAin) + ", ");
			}
			System.out.print(" and pointing to header ");
			for (AbstractInsnNode ain: loop.instructionsPointingToHeader) {
				System.out.print(OpcodeReader.giveName(ain) + " " + mn.instructions.indexOf(ain) + ", ");
			}
			System.out.println();
		}
	}
	
	return toPreserve;

	}

}
