package usi.instrumentation;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.objectweb.asm.Opcodes;
import org.objectweb.asm.tree.AbstractInsnNode;
import org.objectweb.asm.tree.ClassNode;
import org.objectweb.asm.tree.JumpInsnNode;
import org.objectweb.asm.tree.LabelNode;
import org.objectweb.asm.tree.LdcInsnNode;
import org.objectweb.asm.tree.LookupSwitchInsnNode;
import org.objectweb.asm.tree.MethodInsnNode;
import org.objectweb.asm.tree.MethodNode;
import org.objectweb.asm.tree.TableSwitchInsnNode;

import ch.unisi.inf.sp.cfg.InstructionPair;
import ch.unisi.inf.sp.cfg.InstructionPointer;
import ch.unisi.inf.sp.cfg.Loop;

public class LoopInstrumenter implements Opcodes {
	

	private ClassNode cn;
	private MethodNode mn;
	private Map<InstructionPair, AbstractInsnNode> multipleInsertionStorage = new HashMap<InstructionPair, AbstractInsnNode>();
	
	private Map<AbstractInsnNode, AbstractInsnNode> instrumentedEdge;
	
	private List<Loop> iterations;
	
	public LoopInstrumenter(ClassNode cn, MethodNode mn, List<Loop> iterations) {
		this.cn = cn;
		this.mn = mn;
		this.iterations = iterations;
		instrumentedEdge = new HashMap<AbstractInsnNode, AbstractInsnNode>();
	}
	
	private void instrumentAfterLoop (Loop loop) {
		Map<AbstractInsnNode, AbstractInsnNode> repetitionTracker = new HashMap<AbstractInsnNode, AbstractInsnNode>();
		Map<AbstractInsnNode,InstructionPointer> m =  loop.instructionsPointingOutOfTheLoop;
		int[] numbers = loop.numbers;
		
		for (AbstractInsnNode ain: m.keySet()) {
			
			if (m.get(ain).ain == null) {
					mn.maxStack++;
					mn.instructions.insertBefore(mn.instructions.get(mn.instructions.indexOf(ain)-2), new LdcInsnNode (cn.name + "/" + mn.name + "/" + numbers[0]+"-"+numbers[1],-1));
					mn.maxStack++;
					mn.instructions.insertBefore(mn.instructions.get(mn.instructions.indexOf(ain)-2), new MethodInsnNode (INVOKESTATIC, "usi/instrumentation/LoopProfiler", "countIF_LOOP_After", "(Ljava/lang/Object;)V",-1));

			} else {
					MethodInsnNode min = new MethodInsnNode 
					(INVOKESTATIC, "usi/instrumentation/LoopProfiler", "countIF_LOOP_After", "(Ljava/lang/Object;)V",-1);
					
					if (ain instanceof JumpInsnNode) {
						if (m.get(ain).ain instanceof LabelNode)  {
							InstructionPair ip = new InstructionPair(ain, m.get(ain).ain);
							if (m.get(ain).edge.v.equals("T")) {
									
									
								if (Configuration.DEBUG_LOOP_AFTER) {
									System.out.println("true edge points out of the loop");
									System.out.println("Loop " + loop.numbers[0] + 
											"-" + loop.numbers[1] + " instruction " + OpcodeReader.giveName(ain) + 
											" " + mn.instructions.indexOf(ain) + " points to " + 
											OpcodeReader.giveName(m.get(ain).ain) + " " + 
											mn.instructions.indexOf(m.get(ain).ain));
								}
								
								if (!multipleInsertionStorage.containsKey(ip)) {
									if (repetitionTracker.containsKey(((JumpInsnNode) ain).label)) {
										((JumpInsnNode) ain).label = (LabelNode) repetitionTracker.get(((JumpInsnNode) ain).label);
										multipleInsertionStorage.put(ip, (LabelNode) repetitionTracker.get(((JumpInsnNode) ain).label));
									} else {
										LabelNode oLabel = new LabelNode();
										mn.instructions.insertBefore(((JumpInsnNode) ain).label, oLabel);
										mn.maxStack++;
										mn.instructions.insert(oLabel, new JumpInsnNode(GOTO, ((JumpInsnNode) ain).label,-1));
										mn.maxStack++;
										mn.instructions.insert(oLabel, min);
										mn.maxStack++;
										mn.instructions.insert(oLabel, new LdcInsnNode (cn.name + "/" + mn.name + "/" + numbers[0]+"-"+numbers[1],-1));
										mn.maxStack++;
										
										repetitionTracker.put(((JumpInsnNode) ain).label, oLabel);
										multipleInsertionStorage.put(ip, oLabel);
																				
										((JumpInsnNode) ain).label = oLabel;
									}	
									
								} else {
									
									if (repetitionTracker.containsKey(((JumpInsnNode) ain).label)) {
										((JumpInsnNode) ain).label = (LabelNode) repetitionTracker.get(((JumpInsnNode) ain).label);
										multipleInsertionStorage.put(ip, (LabelNode) repetitionTracker.get(((JumpInsnNode) ain).label));
									} else {
										
										LabelNode xxx = new LabelNode();
										//that became new
										mn.instructions.insertBefore(((JumpInsnNode) ain).label, xxx);
										mn.maxStack++;
										mn.maxStack++;
										mn.instructions.insert(xxx, new JumpInsnNode(GOTO, ((JumpInsnNode) ain).label,-1));
										mn.instructions.insert(xxx, min);
										mn.maxStack++;
										mn.instructions.insert(xxx, new LdcInsnNode (cn.name + "/" + mn.name + "/" + numbers[0]+"-"+numbers[1],-1));
										mn.maxStack++;
										
										repetitionTracker.put(((JumpInsnNode)ain).label, xxx);
										multipleInsertionStorage.put(ip, xxx);
										((JumpInsnNode) ain).label = xxx;
									}							
								}
							} else {
								
								
								
								if (Configuration.DEBUG_LOOP_AFTER) {
									System.out.println("false edge points out of the loop");
									System.out.println("Loop " + loop.numbers[0] + 
											"-" + loop.numbers[1] + " instruction " + OpcodeReader.giveName(ain) + 
											" " + mn.instructions.indexOf(ain) + " points to " + 
											OpcodeReader.giveName(m.get(ain).ain) + " " + 
											mn.instructions.indexOf(m.get(ain).ain));
								}
								
								
								
								
								if (!multipleInsertionStorage.containsKey(ip)) {
									
									if (repetitionTracker.containsKey(m.get(ain).ain)) {
										
									} else {
										
										mn.maxStack++;
										mn.instructions.insert(ain, min);
										mn.maxStack++;
										LdcInsnNode ldc = new LdcInsnNode (cn.name + "/" + mn.name + "/" + numbers[0]+"-"+numbers[1],-1);
										mn.instructions.insert(ain, ldc);
										
										LabelNode oLabel = new LabelNode();
										mn.instructions.insert(ain, oLabel);
										
										mn.maxStack++;
										mn.instructions.insert(ain, new JumpInsnNode(GOTO, oLabel,-1));
										
										repetitionTracker.put(m.get(ain).ain, oLabel);
										multipleInsertionStorage.put(ip, oLabel);
									}
									
								} else {
									
									if (repetitionTracker.containsKey(m.get(ain).ain)) {
									} else {
										
										LabelNode ln = new LabelNode();
										mn.instructions.insert(ip.from, ln);
										mn.maxStack++;
										mn.instructions.insert(ln, min);
										mn.maxStack++;
										LdcInsnNode ldc = new LdcInsnNode (cn.name + "/" + mn.name + "/" + numbers[0]+"-"+numbers[1],-1);
										mn.instructions.insert(ln, ldc);
										repetitionTracker.put(((JumpInsnNode) ip.from).label, ln);
										
									}
								}
							}
						} else {
							if (Configuration.DEBUG_LOOP_AFTER) {
								System.out.println("not instance of label node we point out of loop");
								System.out.println("Loop " + loop.numbers[0] + 
										"-" + loop.numbers[1] + " instruction " + OpcodeReader.giveName(ain) + 
										" " + mn.instructions.indexOf(ain) + " points to " + 
										OpcodeReader.giveName(m.get(ain).ain) + " " + 
										mn.instructions.indexOf(m.get(ain).ain));
							}
							mn.maxStack++;
							mn.instructions.insert(ain, min);
							mn.maxStack++;
							mn.instructions.insert(ain, new LdcInsnNode (cn.name + "/" + mn.name + "/" + numbers[0]+"-"+numbers[1],-1));
							
						}
							
						
						
					} else if (ain instanceof TableSwitchInsnNode ||
							ain instanceof LookupSwitchInsnNode) {
						if (Configuration.DEBUG_LOOP_AFTER) {
							System.out.println("Loop " + loop.numbers[0] + 
									"-" + loop.numbers[1] + " instruction " + OpcodeReader.giveName(ain) + 
									" " + mn.instructions.indexOf(ain) + " points to " + 
									OpcodeReader.giveName(m.get(ain).ain) + " " + 
									mn.instructions.indexOf(m.get(ain).ain) + " " + m.get(ain).edge.v);							
						}
						
						if (m.get(ain).ain instanceof LabelNode) {
							mn.maxStack++;
							mn.instructions.insert(m.get(ain).ain, min);
							mn.maxStack++;
							mn.instructions.insert(m.get(ain).ain, new LdcInsnNode (cn.name + "/" + mn.name + "/" + numbers[0]+"-"+numbers[1],-1));
						} else {
							System.out.println("error while tableswitch or lookupswitch instrumenting");
						}
						
					} else {
						
						if (Configuration.DEBUG_LOOP_AFTER) {
							System.out.println("not jump or switch node points out of loop");
							System.out.println("Loop " + loop.numbers[0] + 
									"-" + loop.numbers[1] + " instruction " + OpcodeReader.giveName(ain) + 
									" " + mn.instructions.indexOf(ain) + " points to " + 
									OpcodeReader.giveName(m.get(ain).ain) + " " + 
									mn.instructions.indexOf(m.get(ain).ain));
						}
						mn.maxStack++;
						mn.instructions.insert(ain, min);
						mn.maxStack++;
						mn.instructions.insert(ain, new LdcInsnNode (cn.name + "/" + mn.name + "/" + numbers[0]+"-"+numbers[1],-1));
					}
						
				
			}
			
		}

	}
	
	
	public void instrumentLoop () {
		AbstractInsnNode header; 
		List<AbstractInsnNode> inTails, outTails;
		List<AbstractInsnNode> p;
		Map<AbstractInsnNode,InstructionPointer> m;
		
		for (Loop loop: iterations) {
			
			header = loop.header;
			inTails = loop.inTails;
			outTails = loop.outTails;
			p = loop.instructionsPointingToHeader;
			m = loop.instructionsPointingOutOfTheLoop;
			int[] numbers = loop.numbers;
			
			instrumentAfterLoop(loop);
			LdcInsnNode preHeader = new LdcInsnNode (cn.name + "/" + mn.name + "/" + numbers[0]+"-"+numbers[1],-1);
			
			mn.maxStack++;
			mn.instructions.insertBefore(header, preHeader);
			
			mn.maxStack++;
			mn.instructions.insertBefore(header, new MethodInsnNode (INVOKESTATIC, "usi/instrumentation/LoopProfiler", "countIF_LOOP_Before", "(Ljava/lang/Object;)V",-1));				
			instrumentLoopIncrease(header, numbers, inTails, outTails);
			LabelNode label2 = new LabelNode();
			mn.maxStack++;
			mn.instructions.insertBefore(preHeader, label2);
			for (AbstractInsnNode node: p) {
				if (node instanceof JumpInsnNode) {	
					if (((JumpInsnNode) node).label.equals((LabelNode)header)) {
							((JumpInsnNode) node).label = label2;
					} 
					else {
						if (multipleInsertionStorage.containsKey(new InstructionPair(node, header))) {
							int i = 0;
							while (mn.instructions.get(mn.instructions.indexOf(((JumpInsnNode)node).label)+i).getOpcode()!=GOTO) {
								i++;
							}
							if (((JumpInsnNode)mn.instructions.get(mn.instructions.indexOf(((JumpInsnNode)node).label)+i)).label.equals(header)) {
								((JumpInsnNode)mn.instructions.get(mn.instructions.indexOf(((JumpInsnNode)node).label)+i)).label = label2;
							} else {
								((JumpInsnNode)mn.instructions.get(mn.instructions.indexOf(((JumpInsnNode)node).label)+i)).label = label2;
							}
						}
					}
				} else if (node instanceof TableSwitchInsnNode || node instanceof LookupSwitchInsnNode) {
					if (node instanceof TableSwitchInsnNode) {
						for (int i = 0; i<((TableSwitchInsnNode) node).labels.size(); i++) {
							if (((TableSwitchInsnNode) node).labels.get(i).equals(header)) {
								((TableSwitchInsnNode) node).labels.remove(i);
								((TableSwitchInsnNode) node).labels.add(i, label2);
							}
						}
												
					} else {
						
						for (int i = 0; i<((LookupSwitchInsnNode) node).labels.size(); i++) {
							if (((LookupSwitchInsnNode) node).labels.get(i).equals(header)) {
								((LookupSwitchInsnNode) node).labels.remove(i);
								((LookupSwitchInsnNode) node).labels.add(i, label2);
							}
						}
					}
				}
				else {
					mn.instructions.insert(node, new JumpInsnNode(GOTO, label2,-1));
				}
			}
		}
	}
	
	private void instrumentLoopIncrease (AbstractInsnNode header, int[] numbers, List<AbstractInsnNode> inTails, List<AbstractInsnNode> outTails) {
		List<AbstractInsnNode> tails = inTails;
		for (int i = 0; i<2; i++) {
			for (AbstractInsnNode tail: tails) {
				
				LabelNode mutatedHeader = null;
				InstructionPair ip = new InstructionPair(tail, header);
				if (multipleInsertionStorage.containsKey(ip)) {
					if (multipleInsertionStorage.get(ip)==null)
						continue;
					
					mutatedHeader = (LabelNode) multipleInsertionStorage.get(ip);
					mn.maxStack++;
					mn.instructions.insert(mutatedHeader, new MethodInsnNode (INVOKESTATIC, "usi/instrumentation/LoopProfiler", "countIF_LOOP", "(Ljava/lang/Object;)V",-1));
					
					mn.maxStack++;
					mn.instructions.insert(mutatedHeader, new LdcInsnNode (cn.name + "/" + mn.name + "/" + numbers[0]+"-"+numbers[1],-1));
					continue;
				} 
				
				if (tail instanceof JumpInsnNode) {
					if (header instanceof LabelNode)
					if (((JumpInsnNode) tail).label.equals((LabelNode) header)) {
						
						if (tail.getOpcode() == GOTO) {
							
							LabelNode ln = new LabelNode();
							mn.instructions.insert(tail, ln);
							
							
							LabelNode ln2 = new LabelNode();
							mn.instructions.insert(tail, ln2);
							((JumpInsnNode) tail).label = ln2;
							
							mn.maxStack++;
							mn.instructions.insert(ln2, new MethodInsnNode (INVOKESTATIC, "usi/instrumentation/LoopProfiler", "countIF_LOOP", "(Ljava/lang/Object;)V",-1));
							
							mn.maxStack++;
							mn.instructions.insert(ln2, new LdcInsnNode (cn.name + "/" + mn.name + "/" + numbers[0]+"-"+numbers[1],-1));
							
							JumpInsnNode jump2 = new JumpInsnNode(GOTO, (LabelNode) header,-1 );
							mn.maxStack++;
							mn.instructions.insertBefore(ln, jump2);
							mn.instructions.remove(ln);
							multipleInsertionStorage.put(ip, ln2);
						} else {
							
							
							LabelNode ln = new LabelNode();
							mn.instructions.insert(tail, ln);
							
							
							LabelNode ln2 = new LabelNode();
							mn.instructions.insert(tail, ln2);
							((JumpInsnNode) tail).label = ln2;
							
							mn.maxStack++;
							mn.instructions.insert(ln2, new MethodInsnNode (INVOKESTATIC, "usi/instrumentation/LoopProfiler", "countIF_LOOP", "(Ljava/lang/Object;)V",-1));
							
							mn.maxStack++;
							mn.instructions.insert(ln2, new LdcInsnNode (cn.name + "/" + mn.name + "/" + numbers[0]+"-"+numbers[1],-1));
							
							JumpInsnNode jump2 = new JumpInsnNode(GOTO, (LabelNode) header,-1 );
							mn.maxStack++;
							mn.instructions.insertBefore(ln, jump2);
							
							JumpInsnNode jump = new JumpInsnNode(GOTO, ln,-1);
							mn.instructions.insert(tail, jump);
							multipleInsertionStorage.put(ip, ln2);
						}
						
						
						
					} else {
						JumpInsnNode jump = new JumpInsnNode(GOTO, (LabelNode) header ,-1);
						mn.maxStack++;
						mn.instructions.insert(tail, jump);
						
						mn.maxStack++;
						mn.instructions.insert(tail, new MethodInsnNode (INVOKESTATIC, "usi/instrumentation/LoopProfiler", "countIF_LOOP", "(Ljava/lang/Object;)V",-1));
						
						mn.maxStack++;
						mn.instructions.insert(tail, new LdcInsnNode (cn.name + "/" + mn.name + "/" + numbers[0]+"-"+numbers[1],-1));
						
					}
					
					
					
				} else {
					LabelNode h;
					if (!(header instanceof LabelNode)) {
						h = new LabelNode();
						mn.maxStack++;
						mn.instructions.insertBefore(header, h);
					} else {
						h = (LabelNode) header;
						
					}
					JumpInsnNode jump = new JumpInsnNode(GOTO, h ,-1);
					mn.maxStack++;
					mn.instructions.insert(tail, jump);
					mn.maxStack++;
					mn.instructions.insert(tail, new MethodInsnNode (INVOKESTATIC, "usi/instrumentation/LoopProfiler", "countIF_LOOP", "(Ljava/lang/Object;)V",-1));
					
					mn.maxStack++;
					mn.instructions.insert(tail, new LdcInsnNode (cn.name + "/" + mn.name + "/" + numbers[0]+"-"+numbers[1],-1));
					
				}
			} 
			
			tails = outTails;
		}		

	}
} 