package usi.instrumentation;

import java.lang.instrument.ClassFileTransformer;
import java.lang.instrument.IllegalClassFormatException;
import java.lang.instrument.Instrumentation;
import java.security.ProtectionDomain;

import usi.instrumentation.deserializer.FieldForestDeserializer;
import usi.instrumentation.deserializer.RecursionForestDeserializer;
import usi.instrumentation.deserializer.StaticAlgorithmDetector;
import usi.instrumentation.dumper.AbstractWriter;
import usi.instrumentation.dumper.IterativeData;
import usi.instrumentation.tracker.AbstractSerializer;
import usi.instrumentation.tracker.BytecodeInstructionTracker;
import usi.instrumentation.tree.IterationTree;
import usi.instrumentation.tree.TreeIR;

public class ObjectInstancesCounter implements ClassFileTransformer {

	private InstrumentClass currentInst = InstrumentClass.ic;	
	private InlineJSR inlineJSR = InlineJSR.ic;
	public static String fileName;
	public static void premain(String agentArgs, Instrumentation inst) {
		fileName = agentArgs; 
		inst.addTransformer(new ObjectInstancesCounter ());
		
	}

 	private static class FileWritter implements Runnable {
		private String name;
		public FileWritter(String name) {
			this.name = name;
		}
		public void run() {
			
			if (Configuration.DEBUG || Configuration.TRACK_INDEXES) {
				
				BytecodeInstructionTracker.close();
			}
			
			if (Configuration.ALGORITHM_DETECT) {
				if (Configuration.STATIC_ALGORITHM_DETECTION || Configuration.HYBRID_ALGORITHM_DETECTION) {
					StaticAlgorithmDetector.getInstance();
				}
			}
				BufferFlusher.finish();
				BufferFlusher.flushRecursion();
			
			int i= 0;
			for (CCT cur: CCT.threadMap.values()) {
				if (Configuration.COUNT_COST) {
					
					AbstractSerializer.serialize(cur, ""+i);
					i++;
				}
				
			}
			
			AbstractWriter w;
			
			//w = new AbstractWriter();
			//w.read(fileName+"-loop-buffer.buf", false);
			//w.flush(fileName+"-loopDistribution.txt");
			
			//w = new AbstractWriter();
			//w.read(fileName+"-loop-buffer.buf", true);
			//w.flush(fileName+"-loopStep.txt");
			
			//w = new AbstractWriter();
			//w.read(fileName+"-rec-buffer.buf", false);
			//w.flush(fileName+"-recDistribution.txt");
			
			//w = new AbstractWriter();
			//w.read(fileName+"-rec-buffer.buf", true);
			//w.flush(fileName+"-recStep.txt");
			
				
			w = new AbstractWriter();
			w.read(StaticNames.output + StaticNames.inputdetectorPrefix + fileName+"-rec-buffer.buf", false);
			w.read(StaticNames.output +StaticNames.inputdetectorPrefix + fileName+"-loop-buffer.buf", false);
			w.flush(StaticNames.output +StaticNames.inputdetectorPrefix + fileName+"-iterDistribution.txt");
			
			w = new AbstractWriter();
			w.read(StaticNames.output +StaticNames.inputdetectorPrefix + fileName+"-rec-buffer.buf", true);
			w.read(StaticNames.output +StaticNames.inputdetectorPrefix + fileName+"-loop-buffer.buf", true);
			w.flush(StaticNames.output +StaticNames.inputdetectorPrefix + fileName+"-iterStep.txt");
			
				/*ProfilerMethodWriter pw = new ProfilerMethodWriter();
			pw.read(fileName+"-rec-buffer.buf", false);
			pw.read(fileName+"-loop-buffer.buf", false);
			pw.flushAndClose();*/
				
			IterativeData.readData(new String[]{StaticNames.output + StaticNames.inputdetectorPrefix + fileName+"-rec-buffer.buf", StaticNames.output + StaticNames.inputdetectorPrefix +fileName+"-loop-buffer.buf"});				
			//TreeIR treeIR1 = new FullTree();
			//TreeIR treeIR2 = new LoopTree();*/
			TreeIR treeIR3 = new IterationTree();
			//TreeIR treeIR4 = new RecursionTree();
			for (CCT cur: CCT.threadMap.values()) {
				//treeIR1.DFS(cur, fileName+"-fullTree");
				//treeIR2.DFS(cur, fileName+"-loopTree");
				treeIR3.DFS(cur, StaticNames.output + StaticNames.inputdetectorPrefix + fileName + "-iterationTree");
				//treeIR4.DFS(cur, "recTree");
			}
		
			} 		
	}

	private static FileWritter fw;
	private static Thread shutdownHookThread, dumper;
	private static boolean invokeOnce = true;
	
	synchronized public byte[] transform(ClassLoader loader, String className, Class<?> classBeingRedefined, ProtectionDomain protectionDomain, byte[] classfileBuffer) throws IllegalClassFormatException {
		
		if (Configuration.DEBUG || Configuration.DEBUG_INSTRUMENTATION_OVERHEAD) {
			System.out.println("starting " + className);			
		}
		long start = System.currentTimeMillis();
		if (invokeOnce) {
			StaticNames.algName = fileName + "-" + StaticNames.algName;
			StaticNames.indName = fileName + "-" + StaticNames.indName;
			fw = new FileWritter (fileName);
			shutdownHookThread = new Thread (fw);
			Runtime.getRuntime().addShutdownHook(shutdownHookThread);
			invokeOnce = false;
			if (Configuration.STATIC_RECURSIONS) {
				new RecursionForestDeserializer("lib/", fileName);
			}
			if (Configuration.TRACK_INPUTS) {
				new FieldForestDeserializer("lib/", fileName);
				new FieldProfiler();
			}
		}
		
			if (Filter.eligible(className)) {
				byte [] inlinedJSRBytes = inlineJSR.instrument(className, classfileBuffer, loader);
				byte [] instrumentedClass = currentInst.instrument(className, inlinedJSRBytes, loader);
				if (Configuration.DEBUG || Configuration.DEBUG_INSTRUMENTATION_OVERHEAD) {
					System.out.println("class: " + className + " took " + (System.currentTimeMillis()-start) + "ms");				
				}
				return instrumentedClass;
			} else {
				if (Configuration.DEBUG || Configuration.DEBUG_INSTRUMENTATION_OVERHEAD) {
					System.out.println("class: " + className + " is FILTERED but took " + (System.currentTimeMillis()-start) + "ms");				
				}
				return classfileBuffer;
			}  			
	}
}
