package usi.instrumentation.deserializer;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import usi.instrumentation.StaticNames;

public class StaticAlgorithmDetector {
	
	private static StaticAlgorithmDetector sad;
	Map<String, IndicesBlock> mapBlocks; 
	Map<Integer, List<String>> algorithms; 
	public Map<String, List<String>> algs;
	//String tempPass = "/Users/zaparanuks/Documents/SP_backup/AlgorithmicProfilingExamples/";
	//String algName;
	
	private StaticAlgorithmDetector() {
		algorithms = new HashMap<Integer, List<String>>();
		mapBlocks = new HashMap<String, IndicesBlock>();
		algs = new HashMap<String, List<String>>();
		readRepetitionTreeIndices();		
		combine(readConnectedIndices());
	}
	
	
	public static synchronized StaticAlgorithmDetector getInstance() {
        if (sad == null) {
                sad = new StaticAlgorithmDetector();
        }
        return sad;
	}
	
	private void combine(Map<Integer, List<AlgorithmBytecodeIndex>> algorithmIndices) {
		List<AlgorithmBytecodeIndex> toRemove = new ArrayList<AlgorithmBytecodeIndex>();
		for (Integer id: algorithmIndices.keySet()) {
			for (AlgorithmBytecodeIndex abi: algorithmIndices.get(id)) {
				if (mapBlocks.containsKey(abi.getName())) {
					IndicesBlock b = mapBlocks.get(abi.getName());
					String repName = findRepetition(b, abi.getId());
					if (repName!=null) {
						abi.setRepetitionName(repName);
					} else {
						toRemove.add(abi);
					}
				} else {
					toRemove.add(abi);
				}
				
			}
			if (!toRemove.isEmpty()) {
				algorithmIndices.get(id).removeAll(toRemove);
				toRemove.clear();
			}
		}
		
		cleanAlgorithms(algorithmIndices);
		adjustNames();
	
	}
	// ..Main/fillArray/(ILch/usi/inf/sape/algoprof/example/sort/insertion3/List;)V
	// ch/usi/inf/sape/algoprof/example/sort/bidirectional1/BBS1/fillArray(I)[I/47
	
	private void adjustNames() {
		List<String> toRemove = new ArrayList<String>();
		List<String> toAdd = new ArrayList<String>();
		for (Integer id: algorithms.keySet()) {
			List<String> l = algorithms.get(id);
			for (String s: l) {
				//test if loop
				if (s.contains("/")) {
					String tail = null;
					boolean isLoop = true;
					tail = s.substring(s.lastIndexOf("/")+1);
					try {
						Integer.parseInt(tail);
					} catch (NumberFormatException e) {
						isLoop = false;
					}
					if (isLoop) {
						String body = s.substring(0, s.indexOf("("));
						toAdd.add(body + "/" + tail);
						toRemove.add(s);
						//System.out.println("check " + s);
					}
					
				}
			}
			l.removeAll(toRemove);
			l.addAll(toAdd);
			toRemove.clear();
			toAdd.clear();
			for (String s: l) {
				algs.put(s, l);
			}
		}
	}
	
	private void cleanAlgorithms(Map<Integer, List<AlgorithmBytecodeIndex>> algorithmIndices) {
		List<String> l = new ArrayList<String>();
		for (Integer id: algorithmIndices.keySet()) {
			for (AlgorithmBytecodeIndex abi: algorithmIndices.get(id)) {
				if (!l.contains(abi.getRepetitionName())) {
					l.add(abi.getRepetitionName());
				}				
			}
			if (!l.isEmpty()) {
				algorithms.put(id, l);
				l = new ArrayList<String>();
			}
		}
		List<Integer> toRemove = new ArrayList<Integer>();
		List<String> toAdd = new ArrayList<String>();
		for (Integer id: algorithms.keySet()) {
			List<String> l1 = algorithms.get(id);
			for (Integer id2: algorithms.keySet()) {
				boolean next = false;
				List<String> l2 = algorithms.get(id2);
				if (id.intValue() != id2.intValue()) {
					for (String s: l1) {
						//must solve it differently, by merging
						if (l2.contains(s)) {
							if (l1.size()>l2.size()) {
								if (!toRemove.contains(id2)) {
									toRemove.add(id2);
								}
								for (String s2: l2) {
									if (!l1.contains(s2)) {
										toAdd.add(s2);
									}
								}
							} else if (l1.size()<l2.size()) {
								if (!toRemove.contains(id)) {
									toRemove.add(id);
									next = true;
								}
								for (String s1: l1) {
									if (!l2.contains(s1)) {
										l2.add(s1);
									}
								}
							} else {
								if (!toRemove.contains(id) && !toRemove.contains(id2)) {
									toRemove.add(id2);
								}
								for (String s2: l2) {
									if (!l1.contains(s2)) {
										toAdd.add(s2);
									}
								}
							}
							
							break;
						}
					}
					if (!toAdd.isEmpty()) {
						l1.addAll(toAdd);
						toAdd.clear();
					}
				}
				if (next) break;
			}
		}
		for (Integer key: toRemove)
			algorithms.remove(key);
	}
	
	public void printAlgorithms() {
		for (Integer id: algorithms.keySet()) {
			System.out.println("for algorithm " + id);
			for (String abi: algorithms.get(id)) {
				System.out.println(abi);
			}
		}
	}
	
	private String findRepetition (IndicesBlock b, int id) {
		String name = null;
		if (b.instructionIds.contains(id)) {
			return b.blockName;
		} else {
			if (b.getSucc()!=null) {
				for (IndicesBlock next: b.getSucc()) {
					name = findRepetition(next, id);
					if (name!=null) break;
				}				
			}
		}
		return name;
	}
	
	void readRepetitionTreeIndices () {
		List <IndicesBlock> treeList = new ArrayList<IndicesBlock>();
		try {
			BufferedReader br = new BufferedReader(new FileReader(new File(StaticNames.output + StaticNames.indName)));
			String line = null;			
			while ((line = br.readLine())!=null) {
				String nodeName = line.substring(0, line.indexOf(","));
				//System.out.println("REP " + nodeName);
				//System.out.println("***" + line + " " + line.indexOf(","+1));
				String numbers = line.substring(line.indexOf(",")+1);
				String[] arr = numbers.split(",");
				List<Integer> intNumbers = new ArrayList<Integer>();
				for (String s: arr) {
					if (!s.equals("")) {
						intNumbers.add(Integer.parseInt(s));
					}
				}
				
				treeList.add(new IndicesBlock(nodeName, intNumbers));
			}
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		if (!treeList.isEmpty()) {
			buildHierarchy(treeList);
		}
		
	}
	
	void buildHierarchy (List<IndicesBlock> treeList) {
		
		Map<String, List<IndicesBlock>> splitedBlocks = splitRepetitionsByMethods(treeList);
		
		for (String name: splitedBlocks.keySet()) {
			List<IndicesBlock> l = splitedBlocks.get(name);
			if (l.size()>1) {
				//sort in 
				l = sort(l);
				for (int i=0; i<l.size(); i++) {
					IndicesBlock b = l.get(i);
					for (int j=i+1; j<l.size(); j++) {
						IndicesBlock b2 = l.get(j);
						b.intersects(b2);
						
					}
				}
				mapBlocks.put(l.get(l.size()-1).blockName, l.get(l.size()-1));
			}
		}
	}
	
	private List<IndicesBlock> sort (List<IndicesBlock> l) {
		int n = l.size();
		int j;
		int st = -1;
		while (st <  n) {
			st++;
			n--;
			for (j = st; j <  n; j++) {
				if (l.get(j).instructionIds.size() > l.get(j + 1).instructionIds.size()) {
					IndicesBlock T = l.get(j);
					l.remove(j);
					l.add(j, l.get(j));
					l.remove(j+1);
					l.add(j+1,T);
				}
			}
			for (j =  n; --j >= st;) {
				if (l.get(j).instructionIds.size() > l.get(j + 1).instructionIds.size()) {
					IndicesBlock T = l.get(j);
					l.remove(j);
					l.add(j, l.get(j));
					l.remove(j+1);
					l.add(j+1,T);
				}

			}
		}
		
		if (l.get(0).fakeBlock) {
			IndicesBlock root = l.remove(0);
			l.add(root);
		}
		
		/*System.out.println("sorted: ");
		for (int i = 0; i<l.size(); i++) {
			IndicesBlock b = l.get(i);
			System.out.println(b.blockName + " " + b.instructionIds.size());
		}*/
		
		return l;
	}
	//create fake method nodes for non-recursive methods that don't show up
	
	Map<String, List<IndicesBlock>> splitRepetitionsByMethods(List<IndicesBlock> treeList) {
		Map<String, List<IndicesBlock>> m = new HashMap<String, List<IndicesBlock>>();
		
		for (IndicesBlock ib: treeList) {
			String methodName = null;
			if (ib.blockName.contains("-")) {
				methodName = ib.blockName.substring(0, ib.blockName.lastIndexOf("/"));
				ib.blockName = ib.blockName.substring(0, ib.blockName.lastIndexOf("-"));
			} else {
				methodName = ib.blockName;
			}
			if (m.containsKey(methodName)) {
				List<IndicesBlock> l = m.get(methodName);
				boolean merges = false;
				for (IndicesBlock b: l) {
					if (b.blockName.equals(ib.blockName)) {
						b.merge(ib);
						merges = true;
						break;
					}
				}
				if (!merges)
					l.add(ib);
			} else {
				List<IndicesBlock> l = new ArrayList<IndicesBlock>();
				l.add(ib);
				m.put(methodName, l);
			}
		}
		//TODO merge loops with same header
		
		for (List<IndicesBlock> l: m.values()) {
			boolean found = false;
			for (IndicesBlock b: l) {
				if (b.fakeBlock) {
					found = true;
				}
			}
			if (!found) {
				String methodName = l.get(0).blockName.substring(0, l.get(0).blockName.lastIndexOf("/"));
				IndicesBlock fakeRoot = new IndicesBlock(methodName, new ArrayList<Integer>());
				fakeRoot.fakeBlock = true;
				l.add(fakeRoot);
			}
		}
		return m;
	}
	
	public void printBlocks () {
		
		for (String s: mapBlocks.keySet()) {
			System.out.println("for method: " + s + " = ");
			IndicesBlock method = mapBlocks.get(s);
			for (Integer i: method.getInstructionIds()) {
				System.out.print(i + ", ");
			}
			System.out.println();
			if (method.getSucc()!=null) {
				for (IndicesBlock suc: method.getSucc()) {
					recursivePrint(suc, "\t");
				}
			}
		}
	}
	
	void recursivePrint(IndicesBlock b, String offset) {
		System.out.println(offset + "==> "+ b.blockName);
		for (Integer i: b.instructionIds) {
			System.out.print(i+", ");
		}
		System.out.println();		
		if (b.getSucc()!=null) {
			offset = offset + "\t";
			for (IndicesBlock ib: b.getSucc()) {
				recursivePrint(ib, offset);
			}
		}
	}
	
	Map<Integer, List<AlgorithmBytecodeIndex>> readConnectedIndices () {
		
		Map<Integer, List<AlgorithmBytecodeIndex>> algorithmIndices = new HashMap<Integer, List<AlgorithmBytecodeIndex>>();
		try {
			BufferedReader br = new BufferedReader(new FileReader(new File(StaticNames.path + StaticNames.algName)));
			String line = null;
			boolean processBlock = false;
			List<AlgorithmBytecodeIndex> l = null;
			int id = 0;
			while ((line=br.readLine())!=null) {
				if (processBlock) {
					if (line.equals("}")) {
						processBlock = false;
						if (!l.isEmpty()) {
							algorithmIndices.put(id, l);
							id++;
						}
						continue;
					}
					
					String[] data = line.split(" ");
					String name = data[0].replaceAll("\\.", "/");
					int index = Integer.parseInt(data[1]);
					//System.out.println("adding " + name + " " + index);
					l.add(new AlgorithmBytecodeIndex(name, index));
					
				}
				
				if (line.equals("{")) {
					l = new ArrayList<AlgorithmBytecodeIndex>();
					processBlock = true;
				} 
					
			}
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		return algorithmIndices;
		
	}
	
	public static void main(String[] args) {
		StaticAlgorithmDetector.getInstance().printAlgorithms();
	}
	
	//ch.usi.inf.sape.algoprof.example.sort.bidirectional1.BBS1.sort([I)V 98
	//ch/usi/inf/sape/algoprof/example/sort/bidirectional1/BBS1/sort([I)V 
	

}
