package usi.instrumentation.dumper;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

/*
 * this class constructs a profiler information such as:
 * 
 * 1. name (loop, method) 
 * 2. overall number of iterations
 * 3. overall number of 0 iterations
 * 4. number of entries
 * 5. maximum number of iterations
 * 6. minimal number of iterations
 * 7. mean 
 * 8. type (recursion, loop)
 */

public class ProfilerLoopWriter {
	
	Map<String, ArrayList<Integer>> itStorage = new HashMap<String, ArrayList<Integer>>(); 
	BufferedWriter bw;
	
	private void print(boolean step, String fileName) {
		try {
			File file = new File(fileName+".txt");
			if (bw == null) {
				bw = new BufferedWriter(new FileWriter(file));
			}
			for (String name : itStorage.keySet()) {
				bw.write(name + "," + compose(itStorage.get(name)) + "\r\n");
			
			}
			
			bw.flush();
		} catch (IOException e) {
			e.printStackTrace();
		}
		
	}
	
	public void flushAndClose ()  {
		try {
			bw.flush();
			bw.close();
		} catch (IOException e) {
			
			e.printStackTrace();
		}
		
	
	}
	
	private String compose (ArrayList<Integer> list) {
		int[] numbers = calculate (list);
		String s = "";
		for (int i : numbers) {
			s = s + "," + i;
		}
		return s;
	}

	/*
	 * 2. overall number of iterations
	 * 3. number of entries of 0 iterations
	 * 4. overall number of entries
	 * 5. maximum number of iterations
	 * 6. minimal number of iterations
	 * 7. mean 
	 */
 
	private int[] calculate (ArrayList<Integer> list) {
		
		int iter = 0;
		int counter0 = 0;
		int counter = 0;
		int max = 0;
		int min = Integer.MAX_VALUE;
		int mean = 0;
		
		int[] all = new int[6];
		
		for (int i: list) {
			iter = iter + i;
			if (i == 0) {
				counter0++;
			}
			counter++;
			if (i>max) {
				max=i;
			} else if (i<min) {
				min = i;
			}
			
		}
		if (counter-counter0!=0) {
			mean = iter/(counter-counter0);			
		} 
		all[0] = iter;
		all[1] = counter0;
		all[2] = counter;
		all[3] = max;
		all[4] = min;
		all[5] = mean;
		
		return all;
	}

	

	
	public void read(String name, boolean step) {
		try {
			
			BufferedReader br = new BufferedReader(new FileReader(new File(name)));
			String line;
			String entry = null;
			String[] elements;
			System.out.println("dumping from .... " + name);
			ArrayList<Integer> el;
			while ((line = br.readLine()) != null) {
				if (line.contains(";")) {
					entry = line.substring(0, line.lastIndexOf(";"));
					elements = line.substring(line.lastIndexOf(";")+1, line.length()).split(",");
					
				} else {
					elements = line.split(",");
				}
				
				
				el = new ArrayList<Integer>();
				for (int i = 0; i<elements.length; i++) {
					if (elements[i].matches("\\d+"))
						el.add(Integer.parseInt(elements[i]));
				}
				
				if (entry!=null) {
					if (itStorage.containsKey(entry)) {
						itStorage.get(entry).addAll(el);
					} else {
						itStorage.put(entry, el);
					}					
				} 
				
			}
			
			print(step, name.substring(0, name.indexOf("-")) + "-statistics");
			br.close();
			
			
		} catch (FileNotFoundException e) {
			System.err.println("couldn't read from " + name);
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}

}
