package usi.instrumentation.graph;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;

import usi.instrumentation.CCT;
import usi.instrumentation.Table;
import usi.instrumentation.stack.Node;
import usi.instrumentation.stack.RecursionStorage;
import usi.instrumentation.stack.Snode;
import usi.instrumentation.stack.Tree;
import usi.instrumentation.stack.Snode.Kind;
import usi.instrumentation.tree.TreeIR;

public class TreeToGraph implements TreeIR {
	
	private BufferedWriter bw;
	private int id = 0;
	private HashSet<RecursionStorage> recursions;
	
	
	/*
	 * we should maintain a HashMap with full name of the method
	 * and node and type of node (snode)
	 * 
	 */
	
	private Map<String, Snode<String>> g = new HashMap<String, Snode<String>>(); 
	
	public void DFS(final CCT cur, final String name) {
		
		recursions = cur.recSet;
		Tree<String> tree = cur.tree;
		//dotPrinter(name + "-", id);
		
		id++;
		List<Node<String>> treeList = tree.toList();
		makeGraph(treeList, name);
		/*int vertexCount = treeList.size();
		HashMap<Node<String>, VertexState> state = new HashMap<Node<String>, VertexState>();
          for (int i = 0; i < vertexCount; i++)
                state.put(treeList.get(i), VertexState.White);
        runDFS(tree.getRootElement(), state);
		*/
	}
	
	public void makeGraph(List<Node<String>> treeList, String fileName) {
		
		dotPrinter(fileName, id);
		for (Node<String> node : treeList) {
			Kind kind;
			
			if (isRecursive(node)) {
				kind = Kind.RECURSIVE;
				System.out.println(Table.table.get(Integer.parseInt(node.data)));
			} else if (isLoop(node)) {
				kind = Kind.LOOP;
			} else {
				kind = Kind.NORMAL;
			}
			String name; 
			
			if (kind != Kind.LOOP && !node.data.equals("ENTRY")) {
				name = Table.table.get(Integer.parseInt(node.data));
			} else {
				name = node.data;
			}
			Snode snode;
			
			if (!g.containsKey(name)) {
				snode = new Snode(name, kind);
				g.put(name, snode);
			} else {
				snode = g.get(name);
				if (kind == Kind.RECURSIVE) {
					snode.setKind(Kind.RECURSIVE);
					
				}
			}
			String style; 
			if (kind == Kind.RECURSIVE) {
				style = "bold";
			} else if (kind == Kind.LOOP)  {
				style = "dotted";
			} else {
				style = "solid";
			}
			drawNode(node, style);
			drawEdges(node);
		}
		closePrinter();
		
		int normal = 0;
		int recursive = 0;
		int loop = 0;
		for (Snode snode: g.values()) {
			if (snode.getKind() == Kind.RECURSIVE) {
				recursive++;
			} else if (snode.getKind() == Kind.LOOP) {
				loop++;
			} else {
				normal++;
			}
		}
		statisticPrinter(fileName + "-", id, loop, recursive, normal);
		System.out.println("normal " + (normal-1));
		System.out.println("recursive " + recursive);
		System.out.println("loop " + loop);
		
		
	}
	
	
	protected void draw (int hash1, int hash2, String label) {
		   try {
				  bw.write(hash1 + "->" + hash2 + 
						  "[ label =\"" +  label + "\"]\n");
				  
			  } catch (IOException e) {
				  e.printStackTrace();
			  } 
	   }
	    
	    private void drawEdges (Node<String> node) {
	    	
	    	if (node.getNumberOfChildren()!=0) {
	    		for (int i = 0; i<node.getNumberOfChildren(); i++) {
	      		  Node<String> child = node.children.get(i);
	      		  draw(node.hashCode(), child.hashCode(), "");       			  
	      		  
	    		}    		
	    	}
	    	
	    }
	    
	    private void drawNode (Node<String> node, String style) {
	    	try {
	    		String name;
	    		if (!node.data.contains("/") && !node.data.equals("ENTRY")) {
	    			name = Table.table.get(Integer.parseInt(node.data));
	    		} else {
	    			name = node.data;
	    		}
	    			
				bw.write(node.hashCode() + "[shape=record, label = \"" + 
						name + "\" style = " + style + "]\n");
			} catch (IOException e) {
				e.printStackTrace();
			}
			
	    }
	    
	    public void dotPrinter(String name, long id) {
	    	try {
				bw = new BufferedWriter(new FileWriter(new File(name + 
						+ id + ".dot")));
				bw.write("digraph G {\n");
			} catch (IOException e) {
				System.out.println("Nests-loop tree dot file failed");
				e.printStackTrace();
			}
	    }

	
	public void statisticPrinter(String name, long id, int loop, int recursive, int normal) {
    	try {
			bw = new BufferedWriter(new FileWriter(new File(name + 
					+ id + ".txt")));
			bw.write("normal " + (normal-1) + "\r\n");
			bw.write("recursive " + recursive + "\r\n");
			bw.write("loop " + loop + "\r\n");
			
		} catch (IOException e) {
			System.out.println("Normal graph dot file failed");
			e.printStackTrace();
		} finally {
			try {
				bw.flush();
				bw.close();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
		}
    }
	

	
	public void runDFS (Node<String> node, HashMap<Node<String>, VertexState> state) {
        state.put(node, VertexState.Gray);
        if (node.getNumberOfChildren()!=0) {
      	  int updatedChildrenNumber = node.getNumberOfChildren();
      	  List<Node<String>> ch = new ArrayList<Node<String>>();
        	  for (int i = 0; i<updatedChildrenNumber; i++) {
        		  ch.add(node.children.get(i));
        	  }
        	  for (Node<String> child: ch) {  
    			  if (state.get(child) == VertexState.White) {
    				  runDFS(child, state);
    				  
    			  }  
    		  }

        }
        state.put(node, VertexState.Black);
        //time to generate new Tree out of it
        //System.out.println(node.data + " " + Table.table.get(Integer.parseInt(node.data)));
   
  }
	
	private boolean isLoop (Node<String> node) {
    	if (node.data.contains("/")) return true;
    	return false;
    }
    
    private boolean isRecursive (Node<String> node) {
    	for (RecursionStorage rs: recursions) {
    		if (rs.node.equals(node)) {
    			return true;
    		}
    	}
    	
    	return false;
    }
    private void closePrinter() {
    	try {
    		bw.write("}");
			bw.flush();
			bw.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
    	
    }
}
