package usi.instrumentation.stack;

import usi.instrumentation.Table;
import usi.instrumentation.stack.Node.RepetitionEntrance;

/**
 * Array-based implementation of the stack.
 * @author Mark Allen Weiss
 */
public class ArrayStack implements Stack {
    /**
     * Construct the stack.
     */
    public ArrayStack( ) {
        theArray = new Object[ DEFAULT_CAPACITY ];
        topOfStack = -1;
    }
    
    /**
     * Test if the stack is logically empty.
     * @return true if empty, false otherwise.
     */
    public boolean isEmpty( ) {
        return topOfStack == -1;
    }
    
    /**
     * Make the stack logically empty.
     */
    public void makeEmpty( ) {
        topOfStack = -1;
    }
    
    /**
     * Get the most recently inserted item in the stack.
     * Does not alter the stack.
     * @return the most recently inserted item in the stack.
     * @throws UnderflowException if the stack is empty.
     */
    public Object top( ) {
        if( isEmpty( ) ) {
        	UnderflowException e = new UnderflowException( "ArrayStack top" );
        	e.printStackTrace();
        	throw e;
        	
        }
        return theArray[ topOfStack ];
    }
    
    /**
     * Remove the most recently inserted item from the stack.
     * @throws UnderflowException if the stack is empty.
     */
    public void pop( ) {
        if( isEmpty( ) )
            throw new UnderflowException( "ArrayStack pop" );
        topOfStack--;
        //System.out.println(this);
        //System.out.println("******");
    }
    
    public boolean contains (Object x) {
    	for (int i = 0; i<topOfStack; i++) {
    		if (theArray[i].equals(x)) {
    			return true;
    		}
    	}
    	return false;
    }
    
    public Object containsData (String x) {
    	
    	for (int i = topOfStack; i>=0; i--) {
    		if (((Node<String>)theArray[i]).data.equals(x)) {
    			
    			return theArray[i];
    		}
    	}

    	return null;
    }
    
    /**
     * Return and remove the most recently inserted item
     * from the stack.
     * @return the most recently inserted item in the stack.
     * @throws Underflow if the stack is empty.
     */
    public Object topAndPop( ) {
        if( isEmpty( ) )
            throw new UnderflowException( "ArrayStack topAndPop" );
        return theArray[ topOfStack-- ];
    }
    
    /**
     * Insert a new item into the stack.
     * @param x the item to insert.
     */
    public void push( Object x ) {
        if( topOfStack + 1 == theArray.length )
            doubleArray( );
        theArray[ ++topOfStack ] = x;
    }
    
    /**
     * Internal method to extend theArray.
     */
    private void doubleArray( ) {
        Object [ ] newArray;
        
        newArray = new Object[ theArray.length * 2 ];
        for( int i = 0; i < theArray.length; i++ )
            newArray[ i ] = theArray[ i ];
        theArray = newArray;
    }
    
    private Object [ ] theArray;
    private int        topOfStack;
    
    private static final int DEFAULT_CAPACITY = 100;
    
    @Override
    public String toString() {
    	String str = "";
    	boolean is = false;
    	for (Object obj: theArray) {
    		if (obj!=null) {
    			str = str + "-" + obj;    			
    		} else {
    			break;
    		}
    	}
    	return str;
    }

	@Override
	public boolean containsMoreThanOne(String x) {
		int counter = 0;
		for (int i = topOfStack; i>=0; i--) {
			//System.out.println(((Node<String>)theArray[i]).data + " " + x + " uuu");
    		if (((Node<String>)theArray[i]).data.equals(x)) {
    			counter++;
    			if (counter>1)
    				return true;
    		}
    	}
		
		return false;
	}

	@Override
	public Object[] enteredRepetitionFromTop() {
		for (int i = topOfStack; i>=0; i--) {
			
    		if (((Node<String>)theArray[i]).enteredRepetition.equals(RepetitionEntrance.ENTERED)) {
    			//System.out.println(((Node<String>)theArray[i]).enteredRepetition);
    			return new Object[] {theArray[i], theArray[i-1]};
    		} else if (((Node<String>)theArray[i]).enteredRepetition.equals(RepetitionEntrance.ITERATING)){
    			//System.out.println(((Node<String>)theArray[i]).enteredRepetition);
    			return new Object[] {theArray[i], theArray[i-1]};
    		}
    		
    	}
		return null;
	}
    
    
}