package usi.instrumentation.stack;

public class ArrayTree implements Stack {
    /**
     * Construct the stack tree.
     */
    public ArrayTree( ) {
        theArray = new Object[ DEFAULT_CAPACITY ];
        topOfStack = -1;
    }
    
    /**
     * Test if the stack is logically empty.
     * @return true if empty, false otherwise.
     */
    public boolean isEmpty( ) {
        return topOfStack == -1;
    }
    
    /**
     * Make the stack logically empty.
     */
    public void makeEmpty( ) {
        topOfStack = -1;
    }
    
    /**
     * Get the most recently inserted item in the stack.
     * Does not alter the stack.
     * @return the most recently inserted item in the stack.
     * @throws UnderflowException if the stack is empty.
     */
    public Object top( ) {
        if( isEmpty( ) )
            throw new UnderflowException( "ArrayStack top" );
        return theArray[ topOfStack ];
    }
    
    /**
     * Remove the most recently inserted item from the stack.
     * Move it to extra branch associated with the stack
     * @throws UnderflowException if the stack is empty.
     */
    public void pop( ) {
        if( isEmpty( ) )
            throw new UnderflowException( "ArrayStack pop" );
        new ArrayTree().push(theArray[topOfStack]);
        topOfStack--;
    }
    
    public boolean contains (Object x) {
    	for (int i = 0; i<topOfStack; i++) {
    		if (theArray[i].equals(x)) {
    			return true;
    		}
    	}
    	return false;
    }
    
    /**
     * Return and remove the most recently inserted item
     * from the stack.
     * @return the most recently inserted item in the stack.
     * @throws Underflow if the stack is empty.
     */

    //should be also modified
    public Object topAndPop( ) {
        if( isEmpty( ) )
            throw new UnderflowException( "ArrayStack topAndPop" );
        return theArray[ topOfStack-- ];
    }
    
    /**
     * Insert a new item into the stack.
     * @param x the item to insert.
     */
    public void push( Object x ) {
        if( topOfStack + 1 == theArray.length )
            doubleArray( );
        theArray[ ++topOfStack ] = x;
    }
    
    /**
     * Internal method to extend theArray.
     */
    private void doubleArray( ) {
        Object [ ] newArray;
        
        newArray = new Object[ theArray.length * 2 ];
        for( int i = 0; i < theArray.length; i++ )
            newArray[ i ] = theArray[ i ];
        theArray = newArray;
    }
    
    private Object [ ] theArray;
    private int        topOfStack;
    
    private static final int DEFAULT_CAPACITY = 100;
    
    @Override
    public String toString() {
    	String str = "";
    	boolean is = false;
    	for (Object obj: theArray) {
    		if (obj!=null) {
    			str = str + "-" + obj;    			
    		} else {
    			break;
    		}
    	}
    	return str;
    }

	@Override
	public Object containsData(String x) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public boolean containsMoreThanOne(String x) {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public Object[] enteredRepetitionFromTop() {
		// TODO Auto-generated method stub
		return null;
	}
    
    
}