package usi.instrumentation.tracker;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutput;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;

import usi.instrumentation.CCT;
import usi.instrumentation.Configuration;
import usi.instrumentation.StaticNames;
import usi.instrumentation.Table;
import usi.instrumentation.stack.Node;
import usi.instrumentation.stack.RecursionStorage;
import usi.instrumentation.stack.Tree;

public abstract class AbstractSerializer {
	
	private static HashSet<RecursionStorage> recursions;
	
	public static void serialize (final CCT cur, final String name) {
		
		File f = new File(StaticNames.output + "tree-" + name +".obj");
		
		 try{
			   
			   ObjectOutput ObjOut = new ObjectOutputStream(new FileOutputStream(f));
			   //give real extras to recursive nodes
			   giveExtrasToRecursiveNodes(cur);
			   //give names to methods instead if ids
			   assignNamesToNodes(cur.tree);
			   ObjOut.writeObject(cur.tree);
			   ObjOut.close();
			 
			   
		} catch(IOException e){
			System.err.println(e.getMessage());
			e.printStackTrace();
		}
	}
	
	private static void assignNamesToNodes (Tree<String> tree) {
		for (Node<String> node: tree.toList()) {
				if (node.data.contains("/")) {
					//System.out.println("adding " + cur.rootElement.data);											
				} else {
					if (!node.data.equals("ENTRY"))
						node.data = Table.table.get(Integer.parseInt(node.data));
				}
			
		}
	}
	
	public static void deserialize (String path, int number) {
		try {
		    
			for (int i = 0; i<number; i++) {
				File file = new File(path + "tree-" + i + ".obj");
				ObjectInputStream in = new ObjectInputStream(new FileInputStream(file));
				
				Tree<String> tree = (Tree<String>)in.readObject();
				System.out.println(tree.getRootElement().data + " " + tree.getRootElement().extra.totalNumberOfIterations);
				in.close();
			}
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}

		    

	}
	
	private static boolean isRecursive (Node<String> node) {
    	for (RecursionStorage rs: recursions) {
    		if (rs.node.equals(node)) {
    			return true;
    		}
    	}
    	
    	return false;
    }
	
	private static void giveExtrasToRecursiveNodes(final CCT cur) {
		recursions = cur.recSet;
		for (Node<String> node: cur.tree.toList()) {
			
			if (isRecursive(node)) {
				for (RecursionStorage rs: recursions) {
					if (rs.node.equals(node)) {
						if (node.extra.maxNumberOfIterations<rs.extra.maxNumberOfIterations)
							node.extra.maxNumberOfIterations = rs.extra.maxNumberOfIterations;
						if (node.extra.minNumberOfIterations>rs.extra.minNumberOfIterations)
							node.extra.minNumberOfIterations = rs.extra.minNumberOfIterations;
						
						node.extra.totalNumberOfIterations = node.extra.totalNumberOfIterations  + rs.extra.totalNumberOfIterations;
						node.extra.numberOfEntrancies = node.extra.numberOfEntrancies  + rs.extra.numberOfEntrancies;
					}
				}
				if (node.extra.minNumberOfIterations==Integer.MAX_VALUE) {
					node.extra.minNumberOfIterations = 1;
					node.extra.maxNumberOfIterations = node.extra.totalNumberOfIterations - node.extra.numberOfEntrancies + 1;
				}
			}
		}
		
	}
	
     
    public static void main(String[] args) {
		deserialize("/Users/zaparanuks/Documents/dacapo/", 2);
	}

}


