package usi.instrumentation.tree;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import usi.instrumentation.stack.Node;
import usi.instrumentation.stack.Tree;
import usi.instrumentation.tree.TreeIR.VertexState;

public class LoopTreeTest {
	
	private BufferedWriter bw;
	private int id = 0;
	
    public void DFS (final Tree<String> trees) {
		Tree<String> tree = trees;
		dotPrinter("loopTree-", id);
		id++;
		List<Node<String>> treeList = tree.toList();
		int vertexCount = treeList.size();
		HashMap<Node<String>, VertexState> state = new HashMap<Node<String>, VertexState>();
          for (int i = 0; i < vertexCount; i++)
                state.put(treeList.get(i), VertexState.White);
       runDFS(tree.getRootElement(), state);
       closePrinter();   

    }

    private void closePrinter() {
    	try {
    		bw.write("}");
			bw.flush();
			bw.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
    	
    }

    public void runDFS (Node<String> node, HashMap<Node<String>, VertexState> state) {
    	  System.out.println("entered " + node.data);
          state.put(node, VertexState.Gray);
          if (node.getNumberOfChildren()!=0) {
        	int updatedChildrenNumber = node.getNumberOfChildren();
        	List<Node<String>> ch = new ArrayList<Node<String>>();
        	for (int i = 0; i<updatedChildrenNumber; i++) {
        		ch.add(node.children.get(i));
        	}
    	    for (Node<String> child: ch) {
    			  
    			  if (state.get(child) == VertexState.White) {
    				  runDFS(child, state);
    				  
    			  }  
    		  }
          	}
          //if it is black and method we may eliminate it and shift all the children up
          state.put(node, VertexState.Black);
          System.out.println("black " + node.data);
          //time to print node and where it points to
          if (putUp(node)) {
        	  drawNode(node);
        	  drawEdges(node);        	  
          }
    }
    
    private boolean putUp (Node<String> node) {
    	if (!isLoop(node) && node.father!=null) {
    		if (node.getNumberOfChildren()>0) {
    			for (Node<String> child: node.children) {
    				node.father.addChild(child);
    				child.father = node.father;
    			}
    			
    		}
    		node.father.children.remove(node);
    		return false;
    	}
    	return true;
    }
    
    private boolean isLoop (Node<String> node) {
    	if (node.data.contains("/")) return true;
    	return false;
    }
    
    private void drawEdges (Node<String> node) {
    	
    	if (node.getNumberOfChildren()!=0) {
    		for (int i = 0; i<node.getNumberOfChildren(); i++) {
      		  Node<String> child = node.children.get(i);
      		  if (isLoop(child)) {
      			  try {
      				  bw.write(node.hashCode() + "->" + child.hashCode() + 
      						  //"[ label =\"" +  myProfiler.map.get(node).count_LOOP + "\"]\n");
      				  "\n");
      			  } catch (IOException e) {
      				  e.printStackTrace();
      			  }      			  
      		  }
    		}    		
    	}
    	
    }
    
    private void drawNode (Node<String> node) {
    	try {
			bw.write(node.hashCode() + "[shape=record, label = \"" + 
					node.data + "\"]\n");
		} catch (IOException e) {
			e.printStackTrace();
		}
		
    }
    
    public void dotPrinter(String name, long id) {
    	try {
			bw = new BufferedWriter(new FileWriter(new File(name + 
					+ id + ".dot")));
			bw.write("digraph G {\n");
		} catch (IOException e) {
			System.out.println("Nests-loop tree dot file failed");
			e.printStackTrace();
		}
    }

}
