package usi.types.instrumentation;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.objectweb.asm.tree.AbstractInsnNode;
import org.objectweb.asm.tree.ClassNode;
import org.objectweb.asm.tree.MethodNode;

import usi.instrumentation.BBlockExtractor;
import usi.instrumentation.Configuration;
import ch.unisi.inf.sp.cfg.Block;
import ch.unisi.inf.sp.cfg.Loop;

public class LoopFinder {
	
	public List<ch.unisi.inf.sp.type.LoopConstruct> findLoops(ClassNode cn, MethodNode mn) {
		
		BBlockExtractor bb = new BBlockExtractor();
		//<tail, header>
		Map<Block, Block> loops = bb.disassembleMethod(mn, cn);
		
		List<Block>[] allB = getAllBlocksOfLoop(bb, loops);
		List<ch.unisi.inf.sp.type.LoopConstruct> listOfLoops = new ArrayList<ch.unisi.inf.sp.type.LoopConstruct>();
		int i = 0;
		for (Block header: loops.values()) {
			AbstractInsnNode headerInstruction = mn.instructions.get (header.getBlockNumber());
			List<AbstractInsnNode> otherInstructions = new ArrayList<AbstractInsnNode>();
			for (Block block: allB[i]) {
				for (int j = 0; j<block.getInstrs().size(); j++) {
					otherInstructions.add(mn.instructions.get(block.getBlockNumber()+j));
				}
			}
			//add other instructions of header
			for (int j = 1; j<header.getInstrs().size(); j++) {
				otherInstructions.add(mn.instructions.get(header.getBlockNumber()+j));
			}
			listOfLoops.add(new ch.unisi.inf.sp.type.LoopConstruct(headerInstruction,otherInstructions));
			i++;
		}
		
		return mergeLoops(listOfLoops);
		
		
		
		//loop with all blocks should be extracted to a different subroutine
		
	}
	
	private List<ch.unisi.inf.sp.type.LoopConstruct> mergeLoops (List<ch.unisi.inf.sp.type.LoopConstruct> listOfLoops) {
		List<ch.unisi.inf.sp.type.LoopConstruct> newList = new ArrayList<ch.unisi.inf.sp.type.LoopConstruct>();
		Map<ch.unisi.inf.sp.type.LoopConstruct, Boolean> marked = new HashMap<ch.unisi.inf.sp.type.LoopConstruct, Boolean>();
		for (ch.unisi.inf.sp.type.LoopConstruct loop: listOfLoops) {
			marked.put(loop, false);
		}
		for (ch.unisi.inf.sp.type.LoopConstruct loop: listOfLoops) {
			if (!marked.get(loop)) {
				newList.add(loop);
				for (ch.unisi.inf.sp.type.LoopConstruct otherLoop: listOfLoops) {
					if (!loop.equals(otherLoop)) {
						if (loop.getHeader().equals(otherLoop.getHeader())) {
							//mark it that we don't need to put it in a new list
							marked.put(otherLoop, true);
							for (AbstractInsnNode instr: otherLoop.getInstructions()) {
								if (!loop.getInstructions().contains(instr)) {
									loop.getInstructions().add(instr);
								}
							}
						}
					}
				}				
			}
		}
		//need to sort newList by size of loop.getInstructions.size
		//call to sort
		newList = sort(newList);
		
		//assign owner loop to every loop if exists
		for (ch.unisi.inf.sp.type.LoopConstruct loop: newList) {
			for (ch.unisi.inf.sp.type.LoopConstruct otherLoop: newList) {
				if (!loop.equals(otherLoop)) {
					//if other loop contains header of a loop then it is outer loop for the given loop
					//but not sure that the most outer
					if (otherLoop.getInstructions().contains(loop.getHeader())) {
						loop.setOwner(otherLoop);
						break;
					}
				}
			}
		}
		
		return newList;
	}
	
	private List<ch.unisi.inf.sp.type.LoopConstruct> sort (List<ch.unisi.inf.sp.type.LoopConstruct> unsortedList) {
		List<ch.unisi.inf.sp.type.LoopConstruct> list = new ArrayList<ch.unisi.inf.sp.type.LoopConstruct>();
		
		//map of position and size of instruction
		Map<Integer, Integer> map = new HashMap<Integer, Integer>();
		Integer[] toSort = new Integer[unsortedList.size()];
		for (int i = 0; i<unsortedList.size(); i++) {
			map.put(i, unsortedList.get(i).getInstructions().size());
			toSort[i] = unsortedList.get(i).getInstructions().size();
		}
		//Integer[]
		
		toSort = new BBS3().sort(toSort);
		
		for (int i = 0; i<toSort.length; i++) {
			for (Integer position: map.keySet()) {
				if (map.get(position) == toSort[i]) {
					list.add(unsortedList.get(position));
					break;
				}
			}
			
		}
		
		return list;
	}
	
	private Block addBlockToLoop (Block currentBlock, Block header, Map<Block, List<Block>> pred) {
		//check if we traversed all blocks from tail to header
		//if yes, then returned the last block of a loop => header
		if (currentBlock.equals(header)) {
			allBlocksOfTheLoop.add(header);
			return header;
		}
		for (Block b: pred.get(currentBlock)) {
			if (!allBlocksOfTheLoop.contains(b)) {
				/*if (mn.name.equals("checkParents")) {
					for (InstructionData ida : b.getInstrs()) {
						System.out.println(ida.getName());
					}
					System.out.println();
				}*/
				/*if (Configuration.classToSee.equals(cn.name) && mn.name.equals("sort")) {
					System.out.println(b.getInstructions(""));
				}*/
				allBlocksOfTheLoop.add(b);
				addBlockToLoop(b, header, pred);
				//allBlocksOfTheLoop.add(addBlockToLoop(b, header, pred));
			}
		}
		
		return currentBlock;
	}
	
	
	private Set<Block> allBlocksOfTheLoop;
	//private List<Block> temp;
	
	
	/**
	 * Method that gives all the blocks of a loop 
	 * @param bb BBlockExtractor that holds also loops actually
	 * @param loops end and beginning of the loop
	 * @return blocks that are in between of these two blocks
	 */
	private List<Block>[] getAllBlocksOfLoop(BBlockExtractor bb, Map<Block, Block> loops) {
		//how many loops per method are
		List<Block> all[] = new List[loops.values().size()];
		//take all predescessors
		
		Map<Block, List<Block>> pred = bb.predescessors;
		
		
		int i = 0;
		for (Block tail: loops.keySet()) {
			Block header = loops.get(tail);
			allBlocksOfTheLoop = new HashSet<Block>();
			//only initialized here but not used
			//temp = new ArrayList<Block>();
			
			//recursively add all the block of a loop
			/*if (Configuration.classToSee.equals(cn.name) && mn.name.equals("sort")) {
				System.out.println("analyzing loop " + tail.getInstructions("") + " - " + header.getInstructions("") );
			}*/
			allBlocksOfTheLoop.add(tail);
			addBlockToLoop(tail, header, pred);
			all[i] = new ArrayList<Block>(allBlocksOfTheLoop);
			i++;	
		}
		
		if (Configuration.DEBUG || Configuration.DEBUG_TRAVERSE_PREDESCESSORS) {
			int count = 0;
			for (Block block: loops.keySet()) {
				System.out.print("For tail " + block.getInstructions("") + " predescesssors are: ");
				for (Block pr: all[count]) {
					System.out.print(pr.getInstructions("") + ", ");
				}
				System.out.println();
				count++;
			}
		}
		
		return all;
	}

}
